/*
 * Copyright 2023 , NXP
 * All rights reserved.
 *
 * SPDX-License-Identifier: BSD-3-Clause
 */

#ifndef _FSL_INPUTMUX_CONNECTIONS_
#define _FSL_INPUTMUX_CONNECTIONS_

/*******************************************************************************
 * Definitions
 ******************************************************************************/
/* Component ID definition, used by tools. */
#ifndef FSL_COMPONENT_ID
#define FSL_COMPONENT_ID "platform.drivers.inputmux_connections"
#endif

#define INPUTMUX_GpioPortPinToPintsel(port, pin) ((pin) + (PINTSEL_PMUX_ID << PMUX_SHIFT))

/*!
 * @addtogroup inputmux_driver
 * @{
 */

/*!
 * @name Input multiplexing connections
 * @{
 */

/*! @brief Periphinmux IDs */
#define TIMER0CAPTSEL0            0x20U
#define TIMER0TRIGIN              0x30U
#define TIMER1CAPTSEL0            0x40U
#define TIMER1TRIGIN              0x50U
#define TIMER2CAPTSEL0            0x60U
#define TIMER2TRIGIN              0x70U
#define FREQMEAS_REF_REG          0x180U
#define FREQMEAS_TAR_REG          0x184U
#define TIMER3CAPTSEL0            0x1A0U
#define TIMER3TRIGIN              0x1B0U
#define TIMER4CAPTSEL0            0x1C0U
#define TIMER4TRIGIN              0x1D0U
#define AOI1_MUX_REG              0x200U
#define CMP0_TRIG_REG             0x260U
#define ADC0_TRIG0_REG            0x280U
#define ADC1_TRIG0_REG            0x2C0U
#define DAC0_TRIG0_REG            0x300U
#define QDC0_TRIG_REG             0x360U
#define QDC0_HOME_REG             0x364U
#define QDC0_INDEX_REG            0x368U
#define QDC0_PHASEB_REG           0x36CU
#define QDC0_PHASEA_REG           0x370U
#define QDC0_ICAP1_REG            0x374U
#define QDC1_TRIG_REG             0x380U
#define QDC1_HOME_REG             0x384U
#define QDC1_INDEX_REG            0x388U
#define QDC1_PHASEB_REG           0x38CU
#define QDC1_PHASEA_REG           0x390U
#define QDC1_ICAP1_REG            0x394U
#define FlexPWM0_SM0_EXTA0_REG    0x3A0U
#define FlexPWM0_SM0_EXTSYNC0_REG 0x3A4U
#define FlexPWM0_SM1_EXTA1_REG    0x3A8U
#define FlexPWM0_SM1_EXTSYNC1_REG 0x3ACU
#define FlexPWM0_SM2_EXTA2_REG    0x3B0U
#define FlexPWM0_SM2_EXTSYNC2_REG 0x3B4U
#define FlexPWM0_FAULT_REG        0x3C0U
#define FlexPWM0_FORCE_REG        0x3D0U
#define FlexPWM1_SM0_EXTA0_REG    0x3E0U
#define FlexPWM1_SM0_EXTSYNC0_REG 0x3E4U
#define FlexPWM1_SM1_EXTA1_REG    0x3E8U
#define FlexPWM1_SM1_EXTSYNC1_REG 0x3ECU
#define FlexPWM1_SM2_EXTA2_REG    0x3F0U
#define FlexPWM1_SM2_EXTSYNC2_REG 0x3F4U
#define FlexPWM1_FAULT_REG        0x400U
#define FlexPWM1_FORCE_REG        0x410U
#define PWM0_EXT_CLK_REG          0x420U
#define PWM1_EXT_CLK_REG          0x424U
#define AOI0_MUX_REG              0x440U
#define USBFS_TRIG_REG            0x480U
#define EXT_TRIG0_REG             0x4C0U
#define CMP1_TRIG_REG             0x4E0U
#define LPI2C2_TRIG_REG           0x540U
#define OPAMP0_TRIG_REG           0x580U
#define LPI2C0_TRIG_REG           0x5A0U
#define LPI2C1_TRIG_REG           0x5C0U
#define LPSPI0_TRIG_REG           0x5E0U
#define LPSPI1_TRIG_REG           0x600U
#define LPUART0_TRIG_REG          0x620U
#define LPUART1_TRIG_REG          0x640U
#define LPUART2_TRIG_REG          0x660U
#define LPUART3_TRIG_REG          0x680U
#define LPUART4_TRIG_REG          0x6A0U
#define FLEXIO_TRIG0_REG          0x6E0U
#define PMUX_SHIFT 20U

typedef enum _inputmux_index_t
{
    kINPUTMUX_INDEX_CTIMER0CAPTSEL0    = 0U,
    kINPUTMUX_INDEX_CTIMER0CAPTSEL1    = 1U,
    kINPUTMUX_INDEX_CTIMER0CAPTSEL2    = 2U,
    kINPUTMUX_INDEX_CTIMER0CAPTSEL3    = 3U,
    kINPUTMUX_INDEX_CTIMER1CAPTSEL0    = 0U,
    kINPUTMUX_INDEX_CTIMER1CAPTSEL1    = 1U,
    kINPUTMUX_INDEX_CTIMER1CAPTSEL2    = 2U,
    kINPUTMUX_INDEX_CTIMER1CAPTSEL3    = 3U,
    kINPUTMUX_INDEX_CTIMER2CAPTSEL0    = 0U,
    kINPUTMUX_INDEX_CTIMER2CAPTSEL1    = 1U,
    kINPUTMUX_INDEX_CTIMER2CAPTSEL2    = 2U,
    kINPUTMUX_INDEX_CTIMER2CAPTSEL3    = 3U,
    kINPUTMUX_INDEX_ADC0_TRIGSEL0      = 0U,
    kINPUTMUX_INDEX_ADC0_TRIGSEL1      = 1U,
    kINPUTMUX_INDEX_ADC0_TRIGSEL2      = 2U,
    kINPUTMUX_INDEX_ADC0_TRIGSEL3      = 3U,
    kINPUTMUX_INDEX_QDC0_ICAPSEL0      = 0U,
    kINPUTMUX_INDEX_QDC0_ICAPSEL1      = 1U,
    kINPUTMUX_INDEX_QDC0_ICAPSEL2      = 2U,
    kINPUTMUX_INDEX_FLEXPWM0_FAULTSEL0 = 0U,
    kINPUTMUX_INDEX_FLEXPWM0_FAULTSEL1 = 1U,
    kINPUTMUX_INDEX_FLEXPWM0_FAULTSEL2 = 2U,
    kINPUTMUX_INDEX_FLEXPWM0_FAULTSEL3 = 3U,
    kINPUTMUX_INDEX_AOI0_TRIGSEL0      = 0U,
    kINPUTMUX_INDEX_AOI0_TRIGSEL1      = 1U,
    kINPUTMUX_INDEX_AOI0_TRIGSEL2      = 2U,
    kINPUTMUX_INDEX_AOI0_TRIGSEL3      = 3U,
    kINPUTMUX_INDEX_AOI0_TRIGSEL4      = 4U,
    kINPUTMUX_INDEX_AOI0_TRIGSEL5      = 5U,
    kINPUTMUX_INDEX_AOI0_TRIGSEL6      = 6U,
    kINPUTMUX_INDEX_AOI0_TRIGSEL7      = 7U,
    kINPUTMUX_INDEX_AOI0_TRIGSEL8      = 8U,
    kINPUTMUX_INDEX_AOI0_TRIGSEL9      = 9U,
    kINPUTMUX_INDEX_AOI0_TRIGSEL10     = 10U,
    kINPUTMUX_INDEX_AOI0_TRIGSEL11     = 11U,
    kINPUTMUX_INDEX_AOI0_TRIGSEL12     = 12U,
    kINPUTMUX_INDEX_AOI0_TRIGSEL13     = 13U,
    kINPUTMUX_INDEX_AOI0_TRIGSEL14     = 14U,
    kINPUTMUX_INDEX_AOI0_TRIGSEL15     = 15U,
    kINPUTMUX_INDEX_EXT_TRIGSEL0       = 0U,
    kINPUTMUX_INDEX_EXT_TRIGSEL1       = 1U,
    kINPUTMUX_INDEX_EXT_TRIGSEL2       = 2U,
    kINPUTMUX_INDEX_EXT_TRIGSEL3       = 3U,
    kINPUTMUX_INDEX_EXT_TRIGSEL4       = 4U,
    kINPUTMUX_INDEX_EXT_TRIGSEL6       = 6U,
    kINPUTMUX_INDEX_EXT_TRIGSEL7       = 7U
} inputmux_index_t;

/*! @brief INPUTMUX connections type */
typedef enum _inputmux_connection_t
{
    /*!< TIMER0 CAPTSEL. */
    kINPUTMUX_CtimerInp0ToTimer0Captsel                 = 1U + (TIMER0CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_CtimerInp1ToTimer0Captsel                 = 2U + (TIMER0CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_CtimerInp2ToTimer0Captsel                 = 3U + (TIMER0CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_CtimerInp3ToTimer0Captsel                 = 4U + (TIMER0CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_CtimerInp4ToTimer0Captsel                 = 5U + (TIMER0CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_CtimerInp5ToTimer0Captsel                 = 6U + (TIMER0CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_CtimerInp6ToTimer0Captsel                 = 7U + (TIMER0CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_CtimerInp7ToTimer0Captsel                 = 8U + (TIMER0CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_CtimerInp8ToTimer0Captsel                 = 9U + (TIMER0CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_CtimerInp9ToTimer0Captsel                 = 10U + (TIMER0CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_CtimerInp10ToTimer0Captsel                = 11U + (TIMER0CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_CtimerInp11ToTimer0Captsel                = 12U + (TIMER0CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_CtimerInp12ToTimer0Captsel                = 13U + (TIMER0CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_CtimerInp13ToTimer0Captsel                = 14U + (TIMER0CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_CtimerInp14ToTimer0Captsel                = 15U + (TIMER0CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_CtimerInp15ToTimer0Captsel                = 16U + (TIMER0CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_CtimerInp16ToTimer0Captsel                = 17U + (TIMER0CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_CtimerInp17ToTimer0Captsel                = 18U + (TIMER0CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_CtimerInp18ToTimer0Captsel                = 19U + (TIMER0CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_CtimerInp19ToTimer0Captsel                = 20U + (TIMER0CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Usb0StartOfFrameToTimer0Captsel           = 21U + (TIMER0CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out0ToTimer0Captsel                   = 22U + (TIMER0CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out1ToTimer0Captsel                   = 23U + (TIMER0CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out2ToTimer0Captsel                   = 24U + (TIMER0CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out3ToTimer0Captsel                   = 25U + (TIMER0CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Adc0Tcomp0ToTimer0Captsel                 = 26U + (TIMER0CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Adc0Tcomp1ToTimer0Captsel                 = 27U + (TIMER0CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Adc0Tcomp2ToTimer0Captsel                 = 28U + (TIMER0CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Adc0Tcomp3ToTimer0Captsel                 = 29U + (TIMER0CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Cmp0OutToTimer0Captsel                    = 30U + (TIMER0CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Cmp1OutToTimer0Captsel                    = 31U + (TIMER0CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Ctimer1M1ToTimer0Captsel                  = 33U + (TIMER0CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Ctimer1M2ToTimer0Captsel                  = 34U + (TIMER0CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Ctimer1M3ToTimer0Captsel                  = 35U + (TIMER0CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Ctimer2M1ToTimer0Captsel                  = 36U + (TIMER0CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Ctimer2M2ToTimer0Captsel                  = 37U + (TIMER0CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Ctimer2M3ToTimer0Captsel                  = 38U + (TIMER0CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Qdc0CmpFlag0ToTimer0Captsel               = 39U + (TIMER0CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Qdc0CmpFlag1ToTimer0Captsel               = 40U + (TIMER0CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Qdc0CmpFlag2ToTimer0Captsel               = 41U + (TIMER0CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Qdc0CmpFlag3ToTimer0Captsel               = 42U + (TIMER0CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Qdc0PosMatch0ToTimer0Captsel              = 43U + (TIMER0CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm0OutTrig0ToTimer0Captsel            = 44U + (TIMER0CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm1OutTrig0ToTimer0Captsel            = 45U + (TIMER0CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm2OutTrig0ToTimer0Captsel            = 46U + (TIMER0CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Lpi2c0MasterEndOfPacketToTimer0Captsel    = 48U + (TIMER0CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Lpi2c0SlaveEndOfPacketToTimer0Captsel     = 49U + (TIMER0CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Lpspi0EndOfFrameToTimer0Captsel           = 52U + (TIMER0CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Lpspi0ReceivedDataWordToTimer0Captsel     = 53U + (TIMER0CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Lpspi1EndOfFrameToTimer0Captsel           = 54U + (TIMER0CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Lpspi1ReceivedDataWordToTimer0Captsel     = 55U + (TIMER0CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Lpuart0ReceivedDataWordToTimer0Captsel    = 56U + (TIMER0CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Lpuart0TransmittedDataWordToTimer0Captsel = 57U + (TIMER0CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Lpuart0ReceiveLineIdleToTimer0Captsel     = 58U + (TIMER0CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Lpuart1ReceivedDataWordToTimer0Captsel    = 59U + (TIMER0CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Lpuart1TransmittedDataWordToTimer0Captsel = 60U + (TIMER0CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Lpuart1ReceiveLineIdleToTimer0Captsel     = 61U + (TIMER0CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Lpuart2ReceivedDataWordToTimer0Captsel    = 62U + (TIMER0CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Lpuart2TransmittedDataWordToTimer0Captsel = 63U + (TIMER0CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Lpuart2ReceiveLineIdleToTimer0Captsel     = 64U + (TIMER0CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Lpuart3ReceivedDataWordToTimer0Captsel    = 65U + (TIMER0CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Lpuart3TransmittedDataWordToTimer0Captsel = 66U + (TIMER0CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Lpuart3ReceiveLineIdleToTimer0Captsel     = 67U + (TIMER0CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Lpuart4ReceivedDataWordToTimer0Captsel    = 68U + (TIMER0CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Lpuart4TransmittedDataWordToTimer0Captsel = 69U + (TIMER0CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Lpuart4ReceiveLineIdleToTimer0Captsel     = 70U + (TIMER0CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out0ToTimer0Captsel                   = 71U + (TIMER0CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out1ToTimer0Captsel                   = 72U + (TIMER0CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out2ToTimer0Captsel                   = 73U + (TIMER0CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out3ToTimer0Captsel                   = 74U + (TIMER0CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Adc1Tcomp0ToTimer0Captsel                 = 75U + (TIMER0CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Adc1Tcomp1ToTimer0Captsel                 = 76U + (TIMER0CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Adc1Tcomp2ToTimer0Captsel                 = 77U + (TIMER0CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Adc1Tcomp3ToTimer0Captsel                 = 78U + (TIMER0CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Ctimer3M1ToTimer0Captsel                  = 79U + (TIMER0CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Ctimer3M2ToTimer0Captsel                  = 80U + (TIMER0CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Ctimer3M3ToTimer0Captsel                  = 81U + (TIMER0CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Ctimer4M1ToTimer0Captsel                  = 82U + (TIMER0CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Ctimer4M2ToTimer0Captsel                  = 83U + (TIMER0CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Ctimer4M3ToTimer0Captsel                  = 84U + (TIMER0CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Qdc1CmpFlag0ToTimer0Captsel               = 85U + (TIMER0CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Qdc1CmpFlag1ToTimer0Captsel               = 86U + (TIMER0CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Qdc1CmpFlag2ToTimer0Captsel               = 87U + (TIMER0CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Qdc1CmpFlag3ToTimer0Captsel               = 88U + (TIMER0CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Qdc1PosMatch0ToTimer0Captsel              = 89U + (TIMER0CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Pwm1Sm0OutTrig0ToTimer0Captsel            = 90U + (TIMER0CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Pwm1Sm1OutTrig0ToTimer0Captsel            = 91U + (TIMER0CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Pwm1Sm2OutTrig0ToTimer0Captsel            = 92U + (TIMER0CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Lpi2c2MasterEndOfPacketToTimer0Captsel    = 94U + (TIMER0CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Lpi2c2SlaveEndOfPacketToTimer0Captsel     = 95U + (TIMER0CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Lpi2c3MasterEndOfPacketToTimer0Captsel    = 96U + (TIMER0CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Lpi2c3SlaveEndOfPacketToTimer0Captsel     = 97U + (TIMER0CAPTSEL0 << PMUX_SHIFT),

    /*!< Timer1 CAPTSEL. */
    kINPUTMUX_CtimerInp0ToTimer1Captsel                 = 1U + (TIMER1CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_CtimerInp1ToTimer1Captsel                 = 2U + (TIMER1CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_CtimerInp2ToTimer1Captsel                 = 3U + (TIMER1CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_CtimerInp3ToTimer1Captsel                 = 4U + (TIMER1CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_CtimerInp4ToTimer1Captsel                 = 5U + (TIMER1CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_CtimerInp5ToTimer1Captsel                 = 6U + (TIMER1CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_CtimerInp6ToTimer1Captsel                 = 7U + (TIMER1CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_CtimerInp7ToTimer1Captsel                 = 8U + (TIMER1CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_CtimerInp8ToTimer1Captsel                 = 9U + (TIMER1CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_CtimerInp9ToTimer1Captsel                 = 10U + (TIMER1CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_CtimerInp10ToTimer1Captsel                = 11U + (TIMER1CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_CtimerInp11ToTimer1Captsel                = 12U + (TIMER1CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_CtimerInp12ToTimer1Captsel                = 13U + (TIMER1CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_CtimerInp13ToTimer1Captsel                = 14U + (TIMER1CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_CtimerInp14ToTimer1Captsel                = 15U + (TIMER1CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_CtimerInp15ToTimer1Captsel                = 16U + (TIMER1CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_CtimerInp16ToTimer1Captsel                = 17U + (TIMER1CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_CtimerInp17ToTimer1Captsel                = 18U + (TIMER1CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_CtimerInp18ToTimer1Captsel                = 19U + (TIMER1CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_CtimerInp19ToTimer1Captsel                = 20U + (TIMER1CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Usb0StartOfFrameToTimer1Captsel           = 21U + (TIMER1CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out0ToTimer1Captsel                   = 22U + (TIMER1CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out1ToTimer1Captsel                   = 23U + (TIMER1CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out2ToTimer1Captsel                   = 24U + (TIMER1CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out3ToTimer1Captsel                   = 25U + (TIMER1CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Adc0Tcomp0ToTimer1Captsel                 = 26U + (TIMER1CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Adc0Tcomp1ToTimer1Captsel                 = 27U + (TIMER1CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Adc0Tcomp2ToTimer1Captsel                 = 28U + (TIMER1CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Adc0Tcomp3ToTimer1Captsel                 = 29U + (TIMER1CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Cmp0OutToTimer1Captsel                    = 30U + (TIMER1CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Cmp1OutToTimer1Captsel                    = 31U + (TIMER1CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Ctimer0M1ToTimer1Captsel                  = 33U + (TIMER1CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Ctimer0M2ToTimer1Captsel                  = 34U + (TIMER1CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Ctimer0M3ToTimer1Captsel                  = 35U + (TIMER1CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Ctimer2M1ToTimer1Captsel                  = 36U + (TIMER1CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Ctimer2M2ToTimer1Captsel                  = 37U + (TIMER1CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Ctimer2M3ToTimer1Captsel                  = 38U + (TIMER1CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Qdc0CmpFlag0ToTimer1Captsel               = 39U + (TIMER1CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Qdc0CmpFlag1ToTimer1Captsel               = 40U + (TIMER1CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Qdc0CmpFlag2ToTimer1Captsel               = 41U + (TIMER1CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Qdc0CmpFlag3ToTimer1Captsel               = 42U + (TIMER1CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Qdc0PosMatch0ToTimer1Captsel              = 43U + (TIMER1CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm0OutTrig0ToTimer1Captsel            = 44U + (TIMER1CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm1OutTrig0ToTimer1Captsel            = 45U + (TIMER1CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm2OutTrig0ToTimer1Captsel            = 46U + (TIMER1CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Lpi2c0MasterEndOfPacketToTimer1Captsel    = 48U + (TIMER1CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Lpi2c0SlaveEndOfPacketToTimer1Captsel     = 49U + (TIMER1CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Lpspi0EndOfFrameToTimer1Captsel           = 52U + (TIMER1CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Lpspi0ReceivedDataWordToTimer1Captsel     = 53U + (TIMER1CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Lpspi1EndOfFrameToTimer1Captsel           = 54U + (TIMER1CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Lpspi1ReceivedDataWordToTimer1Captsel     = 55U + (TIMER1CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Lpuart0ReceivedDataWordToTimer1Captsel    = 56U + (TIMER1CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Lpuart0TransmittedDataWordToTimer1Captsel = 57U + (TIMER1CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Lpuart0ReceiveLineIdleToTimer1Captsel     = 58U + (TIMER1CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Lpuart1ReceivedDataWordToTimer1Captsel    = 59U + (TIMER1CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Lpuart1TransmittedDataWordToTimer1Captsel = 60U + (TIMER1CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Lpuart1ReceiveLineIdleToTimer1Captsel     = 61U + (TIMER1CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Lpuart2ReceivedDataWordToTimer1Captsel    = 62U + (TIMER1CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Lpuart2TransmittedDataWordToTimer1Captsel = 63U + (TIMER1CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Lpuart2ReceiveLineIdleToTimer1Captsel     = 64U + (TIMER1CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Lpuart3ReceivedDataWordToTimer1Captsel    = 65U + (TIMER1CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Lpuart3TransmittedDataWordToTimer1Captsel = 66U + (TIMER1CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Lpuart3ReceiveLineIdleToTimer1Captsel     = 67U + (TIMER1CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Lpuart4ReceivedDataWordToTimer1Captsel    = 68U + (TIMER1CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Lpuart4TransmittedDataWordToTimer1Captsel = 69U + (TIMER1CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Lpuart4ReceiveLineIdleToTimer1Captsel     = 70U + (TIMER1CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out0ToTimer1Captsel                   = 71U + (TIMER1CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out1ToTimer1Captsel                   = 72U + (TIMER1CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out2ToTimer1Captsel                   = 73U + (TIMER1CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out3ToTimer1Captsel                   = 74U + (TIMER1CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Adc1Tcomp0ToTimer1Captsel                 = 75U + (TIMER1CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Adc1Tcomp1ToTimer1Captsel                 = 76U + (TIMER1CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Adc1Tcomp2ToTimer1Captsel                 = 77U + (TIMER1CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Adc1Tcomp3ToTimer1Captsel                 = 78U + (TIMER1CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Ctimer3M1ToTimer1Captsel                  = 79U + (TIMER1CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Ctimer3M2ToTimer1Captsel                  = 80U + (TIMER1CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Ctimer3M3ToTimer1Captsel                  = 81U + (TIMER1CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Ctimer4M1ToTimer1Captsel                  = 82U + (TIMER1CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Ctimer4M2ToTimer1Captsel                  = 83U + (TIMER1CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Ctimer4M3ToTimer1Captsel                  = 84U + (TIMER1CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Qdc1CmpFlag0ToTimer1Captsel               = 85U + (TIMER1CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Qdc1CmpFlag1ToTimer1Captsel               = 86U + (TIMER1CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Qdc1CmpFlag2ToTimer1Captsel               = 87U + (TIMER1CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Qdc1CmpFlag3ToTimer1Captsel               = 88U + (TIMER1CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Qdc1PosMatch0ToTimer1Captsel              = 89U + (TIMER1CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Pwm1Sm0OutTrig0ToTimer1Captsel            = 90U + (TIMER1CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Pwm1Sm1OutTrig0ToTimer1Captsel            = 91U + (TIMER1CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Pwm1Sm2OutTrig0ToTimer1Captsel            = 92U + (TIMER1CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Lpi2c2MasterEndOfPacketToTimer1Captsel    = 94U + (TIMER1CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Lpi2c2SlaveEndOfPacketToTimer1Captsel     = 95U + (TIMER1CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Lpi2c3MasterEndOfPacketToTimer1Captsel    = 96U + (TIMER1CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Lpi2c3SlaveEndOfPacketToTimer1Captsel     = 97U + (TIMER1CAPTSEL0 << PMUX_SHIFT),

    /*!< Timer2 CAPTSEL. */
    kINPUTMUX_CtimerInp0ToTimer2Captsel                 = 1U + (TIMER2CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_CtimerInp1ToTimer2Captsel                 = 2U + (TIMER2CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_CtimerInp2ToTimer2Captsel                 = 3U + (TIMER2CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_CtimerInp3ToTimer2Captsel                 = 4U + (TIMER2CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_CtimerInp4ToTimer2Captsel                 = 5U + (TIMER2CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_CtimerInp5ToTimer2Captsel                 = 6U + (TIMER2CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_CtimerInp6ToTimer2Captsel                 = 7U + (TIMER2CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_CtimerInp7ToTimer2Captsel                 = 8U + (TIMER2CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_CtimerInp8ToTimer2Captsel                 = 9U + (TIMER2CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_CtimerInp9ToTimer2Captsel                 = 10U + (TIMER2CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_CtimerInp10ToTimer2Captsel                = 11U + (TIMER2CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_CtimerInp11ToTimer2Captsel                = 12U + (TIMER2CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_CtimerInp12ToTimer2Captsel                = 13U + (TIMER2CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_CtimerInp13ToTimer2Captsel                = 14U + (TIMER2CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_CtimerInp14ToTimer2Captsel                = 15U + (TIMER2CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_CtimerInp15ToTimer2Captsel                = 16U + (TIMER2CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_CtimerInp16ToTimer2Captsel                = 17U + (TIMER2CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_CtimerInp17ToTimer2Captsel                = 18U + (TIMER2CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_CtimerInp18ToTimer2Captsel                = 19U + (TIMER2CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_CtimerInp19ToTimer2Captsel                = 20U + (TIMER2CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Usb0StartOfFrameToTimer2Captsel           = 21U + (TIMER2CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out0ToTimer2Captsel                   = 22U + (TIMER2CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out1ToTimer2Captsel                   = 23U + (TIMER2CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out2ToTimer2Captsel                   = 24U + (TIMER2CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out3ToTimer2Captsel                   = 25U + (TIMER2CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Adc0Tcomp0ToTimer2Captsel                 = 26U + (TIMER2CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Adc0Tcomp1ToTimer2Captsel                 = 27U + (TIMER2CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Adc0Tcomp2ToTimer2Captsel                 = 28U + (TIMER2CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Adc0Tcomp3ToTimer2Captsel                 = 29U + (TIMER2CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Cmp0OutToTimer2Captsel                    = 30U + (TIMER2CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Cmp1OutToTimer2Captsel                    = 31U + (TIMER2CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Ctimer0M1ToTimer2Captsel                  = 33U + (TIMER2CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Ctimer0M2ToTimer2Captsel                  = 34U + (TIMER2CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Ctimer0M3ToTimer2Captsel                  = 35U + (TIMER2CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Ctimer1M1ToTimer2Captsel                  = 36U + (TIMER2CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Ctimer1M2ToTimer2Captsel                  = 37U + (TIMER2CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Ctimer1M3ToTimer2Captsel                  = 38U + (TIMER2CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Qdc0CmpFlag0ToTimer2Captsel               = 39U + (TIMER2CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Qdc0CmpFlag1ToTimer2Captsel               = 40U + (TIMER2CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Qdc0CmpFlag2ToTimer2Captsel               = 41U + (TIMER2CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Qdc0CmpFlag3ToTimer2Captsel               = 42U + (TIMER2CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Qdc0PosMatch0ToTimer2Captsel              = 43U + (TIMER2CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm0OutTrig0ToTimer2Captsel            = 44U + (TIMER2CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm1OutTrig0ToTimer2Captsel            = 45U + (TIMER2CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm2OutTrig0ToTimer2Captsel            = 46U + (TIMER2CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Lpi2c0MasterEndOfPacketToTimer2Captsel    = 48U + (TIMER2CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Lpi2c0SlaveEndOfPacketToTimer2Captsel     = 49U + (TIMER2CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Lpspi0EndOfFrameToTimer2Captsel           = 52U + (TIMER2CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Lpspi0ReceivedDataWordToTimer2Captsel     = 53U + (TIMER2CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Lpspi1EndOfFrameToTimer2Captsel           = 54U + (TIMER2CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Lpspi1ReceivedDataWordToTimer2Captsel     = 55U + (TIMER2CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Lpuart0ReceivedDataWordToTimer2Captsel    = 56U + (TIMER2CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Lpuart0TransmittedDataWordToTimer2Captsel = 57U + (TIMER2CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Lpuart0ReceiveLineIdleToTimer2Captsel     = 58U + (TIMER2CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Lpuart1ReceivedDataWordToTimer2Captsel    = 59U + (TIMER2CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Lpuart1TransmittedDataWordToTimer2Captsel = 60U + (TIMER2CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Lpuart1ReceiveLineIdleToTimer2Captsel     = 61U + (TIMER2CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Lpuart2ReceivedDataWordToTimer2Captsel    = 62U + (TIMER2CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Lpuart2TransmittedDataWordToTimer2Captsel = 63U + (TIMER2CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Lpuart2ReceiveLineIdleToTimer2Captsel     = 64U + (TIMER2CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Lpuart3ReceivedDataWordToTimer2Captsel    = 65U + (TIMER2CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Lpuart3TransmittedDataWordToTimer2Captsel = 66U + (TIMER2CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Lpuart3ReceiveLineIdleToTimer2Captsel     = 67U + (TIMER2CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Lpuart4ReceivedDataWordToTimer2Captsel    = 68U + (TIMER2CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Lpuart4TransmittedDataWordToTimer2Captsel = 69U + (TIMER2CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Lpuart4ReceiveLineIdleToTimer2Captsel     = 70U + (TIMER2CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out0ToTimer2Captsel                   = 71U + (TIMER2CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out1ToTimer2Captsel                   = 72U + (TIMER2CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out2ToTimer2Captsel                   = 73U + (TIMER2CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out3ToTimer2Captsel                   = 74U + (TIMER2CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Adc1Tcomp0ToTimer2Captsel                 = 75U + (TIMER2CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Adc1Tcomp1ToTimer2Captsel                 = 76U + (TIMER2CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Adc1Tcomp2ToTimer2Captsel                 = 77U + (TIMER2CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Adc1Tcomp3ToTimer2Captsel                 = 78U + (TIMER2CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Ctimer3M1ToTimer2Captsel                  = 79U + (TIMER2CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Ctimer3M2ToTimer2Captsel                  = 80U + (TIMER2CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Ctimer3M3ToTimer2Captsel                  = 81U + (TIMER2CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Ctimer4M1ToTimer2Captsel                  = 82U + (TIMER2CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Ctimer4M2ToTimer2Captsel                  = 83U + (TIMER2CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Ctimer4M3ToTimer2Captsel                  = 84U + (TIMER2CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Qdc1CmpFlag0ToTimer2Captsel               = 85U + (TIMER2CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Qdc1CmpFlag1ToTimer2Captsel               = 86U + (TIMER2CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Qdc1CmpFlag2ToTimer2Captsel               = 87U + (TIMER2CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Qdc1CmpFlag3ToTimer2Captsel               = 88U + (TIMER2CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Qdc1PosMatch0ToTimer2Captsel              = 89U + (TIMER2CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Pwm1Sm0OutTrig0ToTimer2Captsel            = 90U + (TIMER2CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Pwm1Sm1OutTrig0ToTimer2Captsel            = 91U + (TIMER2CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Pwm1Sm2OutTrig0ToTimer2Captsel            = 92U + (TIMER2CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Lpi2c2MasterEndOfPacketToTimer2Captsel    = 94U + (TIMER2CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Lpi2c2SlaveEndOfPacketToTimer2Captsel     = 95U + (TIMER2CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Lpi2c3MasterEndOfPacketToTimer2Captsel    = 96U + (TIMER2CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Lpi2c3SlaveEndOfPacketToTimer2Captsel     = 97U + (TIMER2CAPTSEL0 << PMUX_SHIFT),

    /*!< Timer3 CAPTSEL. */
    kINPUTMUX_CtimerInp0ToTimer3Captsel                 = 1U + (TIMER3CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_CtimerInp1ToTimer3Captsel                 = 2U + (TIMER3CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_CtimerInp2ToTimer3Captsel                 = 3U + (TIMER3CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_CtimerInp3ToTimer3Captsel                 = 4U + (TIMER3CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_CtimerInp4ToTimer3Captsel                 = 5U + (TIMER3CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_CtimerInp5ToTimer3Captsel                 = 6U + (TIMER3CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_CtimerInp6ToTimer3Captsel                 = 7U + (TIMER3CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_CtimerInp7ToTimer3Captsel                 = 8U + (TIMER3CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_CtimerInp8ToTimer3Captsel                 = 9U + (TIMER3CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_CtimerInp9ToTimer3Captsel                 = 10U + (TIMER3CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_CtimerInp10ToTimer3Captsel                = 11U + (TIMER3CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_CtimerInp11ToTimer3Captsel                = 12U + (TIMER3CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_CtimerInp12ToTimer3Captsel                = 13U + (TIMER3CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_CtimerInp13ToTimer3Captsel                = 14U + (TIMER3CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_CtimerInp14ToTimer3Captsel                = 15U + (TIMER3CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_CtimerInp15ToTimer3Captsel                = 16U + (TIMER3CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_CtimerInp16ToTimer3Captsel                = 17U + (TIMER3CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_CtimerInp17ToTimer3Captsel                = 18U + (TIMER3CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_CtimerInp18ToTimer3Captsel                = 19U + (TIMER3CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_CtimerInp19ToTimer3Captsel                = 20U + (TIMER3CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Usb0StartOfFrameToTimer3Captsel           = 21U + (TIMER3CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out0ToTimer3Captsel                   = 22U + (TIMER3CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out1ToTimer3Captsel                   = 23U + (TIMER3CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out2ToTimer3Captsel                   = 24U + (TIMER3CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out3ToTimer3Captsel                   = 25U + (TIMER3CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Adc0Tcomp0ToTimer3Captsel                 = 26U + (TIMER3CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Adc0Tcomp1ToTimer3Captsel                 = 27U + (TIMER3CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Adc0Tcomp2ToTimer3Captsel                 = 28U + (TIMER3CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Adc0Tcomp3ToTimer3Captsel                 = 29U + (TIMER3CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Cmp0OutToTimer3Captsel                    = 30U + (TIMER3CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Cmp1OutToTimer3Captsel                    = 31U + (TIMER3CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Ctimer0M1ToTimer3Captsel                  = 33U + (TIMER3CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Ctimer0M2ToTimer3Captsel                  = 34U + (TIMER3CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Ctimer0M3ToTimer3Captsel                  = 35U + (TIMER3CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Ctimer1M1ToTimer3Captsel                  = 36U + (TIMER3CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Ctimer1M2ToTimer3Captsel                  = 37U + (TIMER3CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Ctimer1M3ToTimer3Captsel                  = 38U + (TIMER3CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Qdc0CmpFlag0ToTimer3Captsel               = 39U + (TIMER3CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Qdc0CmpFlag1ToTimer3Captsel               = 40U + (TIMER3CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Qdc0CmpFlag2ToTimer3Captsel               = 41U + (TIMER3CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Qdc0CmpFlag3ToTimer3Captsel               = 42U + (TIMER3CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Qdc0PosMatch0ToTimer3Captsel              = 43U + (TIMER3CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm0OutTrig0ToTimer3Captsel            = 44U + (TIMER3CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm1OutTrig0ToTimer3Captsel            = 45U + (TIMER3CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm2OutTrig0ToTimer3Captsel            = 46U + (TIMER3CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Lpi2c0MasterEndOfPacketToTimer3Captsel    = 48U + (TIMER3CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Lpi2c0SlaveEndOfPacketToTimer3Captsel     = 49U + (TIMER3CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Lpspi0EndOfFrameToTimer3Captsel           = 52U + (TIMER3CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Lpspi0ReceivedDataWordToTimer3Captsel     = 53U + (TIMER3CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Lpspi1EndOfFrameToTimer3Captsel           = 54U + (TIMER3CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Lpspi1ReceivedDataWordToTimer3Captsel     = 55U + (TIMER3CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Lpuart0ReceivedDataWordToTimer3Captsel    = 56U + (TIMER3CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Lpuart0TransmittedDataWordToTimer3Captsel = 57U + (TIMER3CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Lpuart0ReceiveLineIdleToTimer3Captsel     = 58U + (TIMER3CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Lpuart1ReceivedDataWordToTimer3Captsel    = 59U + (TIMER3CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Lpuart1TransmittedDataWordToTimer3Captsel = 60U + (TIMER3CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Lpuart1ReceiveLineIdleToTimer3Captsel     = 61U + (TIMER3CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Lpuart2ReceivedDataWordToTimer3Captsel    = 62U + (TIMER3CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Lpuart2TransmittedDataWordToTimer3Captsel = 63U + (TIMER3CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Lpuart2ReceiveLineIdleToTimer3Captsel     = 64U + (TIMER3CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Lpuart3ReceivedDataWordToTimer3Captsel    = 65U + (TIMER3CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Lpuart3TransmittedDataWordToTimer3Captsel = 66U + (TIMER3CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Lpuart3ReceiveLineIdleToTimer3Captsel     = 67U + (TIMER3CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Lpuart4ReceivedDataWordToTimer3Captsel    = 68U + (TIMER3CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Lpuart4TransmittedDataWordToTimer3Captsel = 69U + (TIMER3CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Lpuart4ReceiveLineIdleToTimer3Captsel     = 70U + (TIMER3CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out0ToTimer3Captsel                   = 71U + (TIMER3CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out1ToTimer3Captsel                   = 72U + (TIMER3CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out2ToTimer3Captsel                   = 73U + (TIMER3CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out3ToTimer3Captsel                   = 74U + (TIMER3CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Adc1Tcomp0ToTimer3Captsel                 = 75U + (TIMER3CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Adc1Tcomp1ToTimer3Captsel                 = 76U + (TIMER3CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Adc1Tcomp2ToTimer3Captsel                 = 77U + (TIMER3CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Adc1Tcomp3ToTimer3Captsel                 = 78U + (TIMER3CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Ctimer2M1ToTimer3Captsel                  = 79U + (TIMER3CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Ctimer2M2ToTimer3Captsel                  = 80U + (TIMER3CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Ctimer2M3ToTimer3Captsel                  = 81U + (TIMER3CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Ctimer4M1ToTimer3Captsel                  = 82U + (TIMER3CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Ctimer4M2ToTimer3Captsel                  = 83U + (TIMER3CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Ctimer4M3ToTimer3Captsel                  = 84U + (TIMER3CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Qdc1CmpFlag0ToTimer3Captsel               = 85U + (TIMER3CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Qdc1CmpFlag1ToTimer3Captsel               = 86U + (TIMER3CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Qdc1CmpFlag2ToTimer3Captsel               = 87U + (TIMER3CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Qdc1CmpFlag3ToTimer3Captsel               = 88U + (TIMER3CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Qdc1PosMatch0ToTimer3Captsel              = 89U + (TIMER3CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Pwm1Sm0OutTrig0ToTimer3Captsel            = 90U + (TIMER3CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Pwm1Sm1OutTrig0ToTimer3Captsel            = 91U + (TIMER3CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Pwm1Sm2OutTrig0ToTimer3Captsel            = 92U + (TIMER3CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Lpi2c2MasterEndOfPacketToTimer3Captsel    = 94U + (TIMER3CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Lpi2c2SlaveEndOfPacketToTimer3Captsel     = 95U + (TIMER3CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Lpi2c3MasterEndOfPacketToTimer3Captsel    = 96U + (TIMER3CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Lpi2c3SlaveEndOfPacketToTimer3Captsel     = 97U + (TIMER3CAPTSEL0 << PMUX_SHIFT),

    /*!< Timer4 CAPTSEL. */
    kINPUTMUX_CtimerInp0ToTimer4Captsel                 = 1U + (TIMER4CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_CtimerInp1ToTimer4Captsel                 = 2U + (TIMER4CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_CtimerInp2ToTimer4Captsel                 = 3U + (TIMER4CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_CtimerInp3ToTimer4Captsel                 = 4U + (TIMER4CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_CtimerInp4ToTimer4Captsel                 = 5U + (TIMER4CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_CtimerInp5ToTimer4Captsel                 = 6U + (TIMER4CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_CtimerInp6ToTimer4Captsel                 = 7U + (TIMER4CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_CtimerInp7ToTimer4Captsel                 = 8U + (TIMER4CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_CtimerInp8ToTimer4Captsel                 = 9U + (TIMER4CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_CtimerInp9ToTimer4Captsel                 = 10U + (TIMER4CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_CtimerInp10ToTimer4Captsel                = 11U + (TIMER4CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_CtimerInp11ToTimer4Captsel                = 12U + (TIMER4CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_CtimerInp12ToTimer4Captsel                = 13U + (TIMER4CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_CtimerInp13ToTimer4Captsel                = 14U + (TIMER4CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_CtimerInp14ToTimer4Captsel                = 15U + (TIMER4CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_CtimerInp15ToTimer4Captsel                = 16U + (TIMER4CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_CtimerInp16ToTimer4Captsel                = 17U + (TIMER4CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_CtimerInp17ToTimer4Captsel                = 18U + (TIMER4CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_CtimerInp18ToTimer4Captsel                = 19U + (TIMER4CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_CtimerInp19ToTimer4Captsel                = 20U + (TIMER4CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Usb0StartOfFrameToTimer4Captsel           = 21U + (TIMER4CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out0ToTimer4Captsel                   = 22U + (TIMER4CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out1ToTimer4Captsel                   = 23U + (TIMER4CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out2ToTimer4Captsel                   = 24U + (TIMER4CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out3ToTimer4Captsel                   = 25U + (TIMER4CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Adc0Tcomp0ToTimer4Captsel                 = 26U + (TIMER4CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Adc0Tcomp1ToTimer4Captsel                 = 27U + (TIMER4CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Adc0Tcomp2ToTimer4Captsel                 = 28U + (TIMER4CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Adc0Tcomp3ToTimer4Captsel                 = 29U + (TIMER4CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Cmp0OutToTimer4Captsel                    = 30U + (TIMER4CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Cmp1OutToTimer4Captsel                    = 31U + (TIMER4CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Ctimer0M1ToTimer4Captsel                  = 33U + (TIMER4CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Ctimer0M2ToTimer4Captsel                  = 34U + (TIMER4CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Ctimer0M3ToTimer4Captsel                  = 35U + (TIMER4CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Ctimer1M1ToTimer4Captsel                  = 36U + (TIMER4CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Ctimer1M2ToTimer4Captsel                  = 37U + (TIMER4CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Ctimer1M3ToTimer4Captsel                  = 38U + (TIMER4CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Qdc0CmpFlag0ToTimer4Captsel               = 39U + (TIMER4CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Qdc0CmpFlag1ToTimer4Captsel               = 40U + (TIMER4CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Qdc0CmpFlag2ToTimer4Captsel               = 41U + (TIMER4CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Qdc0CmpFlag3ToTimer4Captsel               = 42U + (TIMER4CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Qdc0PosMatch0ToTimer4Captsel              = 43U + (TIMER4CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm0OutTrig0ToTimer4Captsel            = 44U + (TIMER4CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm1OutTrig0ToTimer4Captsel            = 45U + (TIMER4CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm2OutTrig0ToTimer4Captsel            = 46U + (TIMER4CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Lpi2c0MasterEndOfPacketToTimer4Captsel    = 48U + (TIMER4CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Lpi2c0SlaveEndOfPacketToTimer4Captsel     = 49U + (TIMER4CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Lpspi0EndOfFrameToTimer4Captsel           = 52U + (TIMER4CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Lpspi0ReceivedDataWordToTimer4Captsel     = 53U + (TIMER4CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Lpspi1EndOfFrameToTimer4Captsel           = 54U + (TIMER4CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Lpspi1ReceivedDataWordToTimer4Captsel     = 55U + (TIMER4CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Lpuart0ReceivedDataWordToTimer4Captsel    = 56U + (TIMER4CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Lpuart0TransmittedDataWordToTimer4Captsel = 57U + (TIMER4CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Lpuart0ReceiveLineIdleToTimer4Captsel     = 58U + (TIMER4CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Lpuart1ReceivedDataWordToTimer4Captsel    = 59U + (TIMER4CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Lpuart1TransmittedDataWordToTimer4Captsel = 60U + (TIMER4CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Lpuart1ReceiveLineIdleToTimer4Captsel     = 61U + (TIMER4CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Lpuart2ReceivedDataWordToTimer4Captsel    = 62U + (TIMER4CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Lpuart2TransmittedDataWordToTimer4Captsel = 63U + (TIMER4CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Lpuart2ReceiveLineIdleToTimer4Captsel     = 64U + (TIMER4CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Lpuart3ReceivedDataWordToTimer4Captsel    = 65U + (TIMER4CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Lpuart3TransmittedDataWordToTimer4Captsel = 66U + (TIMER4CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Lpuart3ReceiveLineIdleToTimer4Captsel     = 67U + (TIMER4CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Lpuart4ReceivedDataWordToTimer4Captsel    = 68U + (TIMER4CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Lpuart4TransmittedDataWordToTimer4Captsel = 69U + (TIMER4CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Lpuart4ReceiveLineIdleToTimer4Captsel     = 70U + (TIMER4CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out0ToTimer4Captsel                   = 71U + (TIMER4CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out1ToTimer4Captsel                   = 72U + (TIMER4CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out2ToTimer4Captsel                   = 73U + (TIMER4CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out3ToTimer4Captsel                   = 74U + (TIMER4CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Adc1Tcomp0ToTimer4Captsel                 = 75U + (TIMER4CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Adc1Tcomp1ToTimer4Captsel                 = 76U + (TIMER4CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Adc1Tcomp2ToTimer4Captsel                 = 77U + (TIMER4CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Adc1Tcomp3ToTimer4Captsel                 = 78U + (TIMER4CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Ctimer2M1ToTimer4Captsel                  = 79U + (TIMER4CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Ctimer2M2ToTimer4Captsel                  = 80U + (TIMER4CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Ctimer2M3ToTimer4Captsel                  = 81U + (TIMER4CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Ctimer3M1ToTimer4Captsel                  = 82U + (TIMER4CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Ctimer3M2ToTimer4Captsel                  = 83U + (TIMER4CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Ctimer3M3ToTimer4Captsel                  = 84U + (TIMER4CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Qdc1CmpFlag0ToTimer4Captsel               = 85U + (TIMER4CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Qdc1CmpFlag1ToTimer4Captsel               = 86U + (TIMER4CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Qdc1CmpFlag2ToTimer4Captsel               = 87U + (TIMER4CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Qdc1CmpFlag3ToTimer4Captsel               = 88U + (TIMER4CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Qdc1PosMatch0ToTimer4Captsel              = 89U + (TIMER4CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Pwm1Sm0OutTrig0ToTimer4Captsel            = 90U + (TIMER4CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Pwm1Sm1OutTrig0ToTimer4Captsel            = 91U + (TIMER4CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Pwm1Sm2OutTrig0ToTimer4Captsel            = 92U + (TIMER4CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Lpi2c2MasterEndOfPacketToTimer4Captsel    = 94U + (TIMER4CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Lpi2c2SlaveEndOfPacketToTimer4Captsel     = 95U + (TIMER4CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Lpi2c3MasterEndOfPacketToTimer4Captsel    = 96U + (TIMER4CAPTSEL0 << PMUX_SHIFT),
    kINPUTMUX_Lpi2c3SlaveEndOfPacketToTimer4Captsel     = 97U + (TIMER4CAPTSEL0 << PMUX_SHIFT),
     
    /*!< TIMER0 Trigger. */
    kINPUTMUX_CtimerInp0ToTimer0Trigger                 = 1U + (TIMER0TRIGIN << PMUX_SHIFT),
    kINPUTMUX_CtimerInp1ToTimer0Trigger                 = 2U + (TIMER0TRIGIN << PMUX_SHIFT),
    kINPUTMUX_CtimerInp2ToTimer0Trigger                 = 3U + (TIMER0TRIGIN << PMUX_SHIFT),
    kINPUTMUX_CtimerInp3ToTimer0Trigger                 = 4U + (TIMER0TRIGIN << PMUX_SHIFT),
    kINPUTMUX_CtimerInp4ToTimer0Trigger                 = 5U + (TIMER0TRIGIN << PMUX_SHIFT),
    kINPUTMUX_CtimerInp5ToTimer0Trigger                 = 6U + (TIMER0TRIGIN << PMUX_SHIFT),
    kINPUTMUX_CtimerInp6ToTimer0Trigger                 = 7U + (TIMER0TRIGIN << PMUX_SHIFT),
    kINPUTMUX_CtimerInp7ToTimer0Trigger                 = 8U + (TIMER0TRIGIN << PMUX_SHIFT),
    kINPUTMUX_CtimerInp8ToTimer0Trigger                 = 9U + (TIMER0TRIGIN << PMUX_SHIFT),
    kINPUTMUX_CtimerInp9ToTimer0Trigger                 = 10U + (TIMER0TRIGIN << PMUX_SHIFT),
    kINPUTMUX_CtimerInp10ToTimer0Trigger                = 11U + (TIMER0TRIGIN << PMUX_SHIFT),
    kINPUTMUX_CtimerInp11ToTimer0Trigger                = 12U + (TIMER0TRIGIN << PMUX_SHIFT),
    kINPUTMUX_CtimerInp12ToTimer0Trigger                = 13U + (TIMER0TRIGIN << PMUX_SHIFT),
    kINPUTMUX_CtimerInp13ToTimer0Trigger                = 14U + (TIMER0TRIGIN << PMUX_SHIFT),
    kINPUTMUX_CtimerInp14ToTimer0Trigger                = 15U + (TIMER0TRIGIN << PMUX_SHIFT),
    kINPUTMUX_CtimerInp15ToTimer0Trigger                = 16U + (TIMER0TRIGIN << PMUX_SHIFT),
    kINPUTMUX_CtimerInp16ToTimer0Trigger                = 17U + (TIMER0TRIGIN << PMUX_SHIFT),
    kINPUTMUX_CtimerInp17ToTimer0Trigger                = 18U + (TIMER0TRIGIN << PMUX_SHIFT),
    kINPUTMUX_CtimerInp18ToTimer0Trigger                = 19U + (TIMER0TRIGIN << PMUX_SHIFT),
    kINPUTMUX_CtimerInp19ToTimer0Trigger                = 20U + (TIMER0TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Usb0StartOfFrameToTimer0Trigger           = 21U + (TIMER0TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out0ToTimer0Trigger                   = 22U + (TIMER0TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out1ToTimer0Trigger                   = 23U + (TIMER0TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out2ToTimer0Trigger                   = 24U + (TIMER0TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out3ToTimer0Trigger                   = 25U + (TIMER0TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Adc0Tcomp0ToTimer0Trigger                 = 26U + (TIMER0TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Adc0Tcomp1ToTimer0Trigger                 = 27U + (TIMER0TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Adc0Tcomp2ToTimer0Trigger                 = 28U + (TIMER0TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Adc0Tcomp3ToTimer0Trigger                 = 29U + (TIMER0TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Cmp0OutToTimer0Trigger                    = 30U + (TIMER0TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Cmp1OutToTimer0Trigger                    = 31U + (TIMER0TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Ctimer1M1ToTimer0Trigger                  = 33U + (TIMER0TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Ctimer1M2ToTimer0Trigger                  = 34U + (TIMER0TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Ctimer1M3ToTimer0Trigger                  = 35U + (TIMER0TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Ctimer2M1ToTimer0Trigger                  = 36U + (TIMER0TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Ctimer2M2ToTimer0Trigger                  = 37U + (TIMER0TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Ctimer2M3ToTimer0Trigger                  = 38U + (TIMER0TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Qdc0CmpFlag0ToTimer0Trigger               = 39U + (TIMER0TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Qdc0CmpFlag1ToTimer0Trigger               = 40U + (TIMER0TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Qdc0CmpFlag2ToTimer0Trigger               = 41U + (TIMER0TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Qdc0CmpFlag3ToTimer0Trigger               = 42U + (TIMER0TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Qdc0PosMatch0ToTimer0Trigger              = 43U + (TIMER0TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm0OutTrig0ToTimer0Trigger            = 44U + (TIMER0TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm1OutTrig0ToTimer0Trigger            = 45U + (TIMER0TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm2OutTrig0ToTimer0Trigger            = 46U + (TIMER0TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Lpi2c0MasterEndOfPacketToTimer0Trigger    = 48U + (TIMER0TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Lpi2c0SlaveEndOfPacketToTimer0Trigger     = 49U + (TIMER0TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Lpspi0EndOfFrameToTimer0Trigger           = 52U + (TIMER0TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Lpspi0ReceivedDataWordToTimer0Trigger     = 53U + (TIMER0TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Lpspi1EndOfFrameToTimer0Trigger           = 54U + (TIMER0TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Lpspi1ReceivedDataWordToTimer0Trigger     = 55U + (TIMER0TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Lpuart0ReceivedDataWordToTimer0Trigger    = 56U + (TIMER0TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Lpuart0TransmittedDataWordToTimer0Trigger = 57U + (TIMER0TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Lpuart0ReceiveLineIdleToTimer0Trigger     = 58U + (TIMER0TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Lpuart1ReceivedDataWordToTimer0Trigger    = 59U + (TIMER0TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Lpuart1TransmittedDataWordToTimer0Trigger = 60U + (TIMER0TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Lpuart1ReceiveLineIdleToTimer0Trigger     = 61U + (TIMER0TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Lpuart2ReceivedDataWordToTimer0Trigger    = 62U + (TIMER0TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Lpuart2TransmittedDataWordToTimer0Trigger = 63U + (TIMER0TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Lpuart2ReceiveLineIdleToTimer0Trigger     = 64U + (TIMER0TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Lpuart3ReceivedDataWordToTimer0Trigger    = 65U + (TIMER0TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Lpuart3TransmittedDataWordToTimer0Trigger = 66U + (TIMER0TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Lpuart3ReceiveLineIdleToTimer0Trigger     = 67U + (TIMER0TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Lpuart4ReceivedDataWordToTimer0Trigger    = 68U + (TIMER0TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Lpuart4TransmittedDataWordToTimer0Trigger = 69U + (TIMER0TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Lpuart4ReceiveLineIdleToTimer0Trigger     = 70U + (TIMER0TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out0ToTimer0Trigger                   = 71U + (TIMER0TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out1ToTimer0Trigger                   = 72U + (TIMER0TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out2ToTimer0Trigger                   = 73U + (TIMER0TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out3ToTimer0Trigger                   = 74U + (TIMER0TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Adc1Tcomp0ToTimer0Trigger                 = 75U + (TIMER0TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Adc1Tcomp1ToTimer0Trigger                 = 76U + (TIMER0TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Adc1Tcomp2ToTimer0Trigger                 = 77U + (TIMER0TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Adc1Tcomp3ToTimer0Trigger                 = 78U + (TIMER0TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Ctimer3M1ToTimer0Trigger                  = 79U + (TIMER0TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Ctimer3M2ToTimer0Trigger                  = 80U + (TIMER0TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Ctimer3M3ToTimer0Trigger                  = 81U + (TIMER0TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Ctimer4M1ToTimer0Trigger                  = 82U + (TIMER0TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Ctimer4M2ToTimer0Trigger                  = 83U + (TIMER0TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Ctimer4M3ToTimer0Trigger                  = 84U + (TIMER0TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Qdc1CmpFlag0ToTimer0Trigger               = 85U + (TIMER0TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Qdc1CmpFlag1ToTimer0Trigger               = 86U + (TIMER0TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Qdc1CmpFlag2ToTimer0Trigger               = 87U + (TIMER0TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Qdc1CmpFlag3ToTimer0Trigger               = 88U + (TIMER0TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Qdc1PosMatch0ToTimer0Trigger              = 89U + (TIMER0TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Pwm1Sm0OutTrig0ToTimer0Trigger            = 90U + (TIMER0TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Pwm1Sm1OutTrig0ToTimer0Trigger            = 91U + (TIMER0TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Pwm1Sm2OutTrig0ToTimer0Trigger            = 92U + (TIMER0TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Lpi2c2MasterEndOfPacketToTimer0Trigger    = 94U + (TIMER0TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Lpi2c2SlaveEndOfPacketToTimer0Trigger     = 95U + (TIMER0TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Lpi2c3MasterEndOfPacketToTimer0Trigger    = 96U + (TIMER0TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Lpi2c3SlaveEndOfPacketToTimer0Trigger     = 97U + (TIMER0TRIGIN << PMUX_SHIFT),

    /*!< Timer1 Trigger. */
    kINPUTMUX_CtimerInp0ToTimer1Trigger                 = 1U + (TIMER1TRIGIN << PMUX_SHIFT),
    kINPUTMUX_CtimerInp1ToTimer1Trigger                 = 2U + (TIMER1TRIGIN << PMUX_SHIFT),
    kINPUTMUX_CtimerInp2ToTimer1Trigger                 = 3U + (TIMER1TRIGIN << PMUX_SHIFT),
    kINPUTMUX_CtimerInp3ToTimer1Trigger                 = 4U + (TIMER1TRIGIN << PMUX_SHIFT),
    kINPUTMUX_CtimerInp4ToTimer1Trigger                 = 5U + (TIMER1TRIGIN << PMUX_SHIFT),
    kINPUTMUX_CtimerInp5ToTimer1Trigger                 = 6U + (TIMER1TRIGIN << PMUX_SHIFT),
    kINPUTMUX_CtimerInp6ToTimer1Trigger                 = 7U + (TIMER1TRIGIN << PMUX_SHIFT),
    kINPUTMUX_CtimerInp7ToTimer1Trigger                 = 8U + (TIMER1TRIGIN << PMUX_SHIFT),
    kINPUTMUX_CtimerInp8ToTimer1Trigger                 = 9U + (TIMER1TRIGIN << PMUX_SHIFT),
    kINPUTMUX_CtimerInp9ToTimer1Trigger                 = 10U + (TIMER1TRIGIN << PMUX_SHIFT),
    kINPUTMUX_CtimerInp10ToTimer1Trigger                = 11U + (TIMER1TRIGIN << PMUX_SHIFT),
    kINPUTMUX_CtimerInp11ToTimer1Trigger                = 12U + (TIMER1TRIGIN << PMUX_SHIFT),
    kINPUTMUX_CtimerInp12ToTimer1Trigger                = 13U + (TIMER1TRIGIN << PMUX_SHIFT),
    kINPUTMUX_CtimerInp13ToTimer1Trigger                = 14U + (TIMER1TRIGIN << PMUX_SHIFT),
    kINPUTMUX_CtimerInp14ToTimer1Trigger                = 15U + (TIMER1TRIGIN << PMUX_SHIFT),
    kINPUTMUX_CtimerInp15ToTimer1Trigger                = 16U + (TIMER1TRIGIN << PMUX_SHIFT),
    kINPUTMUX_CtimerInp16ToTimer1Trigger                = 17U + (TIMER1TRIGIN << PMUX_SHIFT),
    kINPUTMUX_CtimerInp17ToTimer1Trigger                = 18U + (TIMER1TRIGIN << PMUX_SHIFT),
    kINPUTMUX_CtimerInp18ToTimer1Trigger                = 19U + (TIMER1TRIGIN << PMUX_SHIFT),
    kINPUTMUX_CtimerInp19ToTimer1Trigger                = 20U + (TIMER1TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Usb0StartOfFrameToTimer1Trigger           = 21U + (TIMER1TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out0ToTimer1Trigger                   = 22U + (TIMER1TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out1ToTimer1Trigger                   = 23U + (TIMER1TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out2ToTimer1Trigger                   = 24U + (TIMER1TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out3ToTimer1Trigger                   = 25U + (TIMER1TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Adc0Tcomp0ToTimer1Trigger                 = 26U + (TIMER1TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Adc0Tcomp1ToTimer1Trigger                 = 27U + (TIMER1TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Adc0Tcomp2ToTimer1Trigger                 = 28U + (TIMER1TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Adc0Tcomp3ToTimer1Trigger                 = 29U + (TIMER1TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Cmp0OutToTimer1Trigger                    = 30U + (TIMER1TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Cmp1OutToTimer1Trigger                    = 31U + (TIMER1TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Ctimer0M1ToTimer1Trigger                  = 33U + (TIMER1TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Ctimer0M2ToTimer1Trigger                  = 34U + (TIMER1TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Ctimer0M3ToTimer1Trigger                  = 35U + (TIMER1TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Ctimer2M1ToTimer1Trigger                  = 36U + (TIMER1TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Ctimer2M2ToTimer1Trigger                  = 37U + (TIMER1TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Ctimer2M3ToTimer1Trigger                  = 38U + (TIMER1TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Qdc0CmpFlag0ToTimer1Trigger               = 39U + (TIMER1TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Qdc0CmpFlag1ToTimer1Trigger               = 40U + (TIMER1TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Qdc0CmpFlag2ToTimer1Trigger               = 41U + (TIMER1TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Qdc0CmpFlag3ToTimer1Trigger               = 42U + (TIMER1TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Qdc0PosMatch0ToTimer1Trigger              = 43U + (TIMER1TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm0OutTrig0ToTimer1Trigger            = 44U + (TIMER1TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm1OutTrig0ToTimer1Trigger            = 45U + (TIMER1TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm2OutTrig0ToTimer1Trigger            = 46U + (TIMER1TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Lpi2c0MasterEndOfPacketToTimer1Trigger    = 48U + (TIMER1TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Lpi2c0SlaveEndOfPacketToTimer1Trigger     = 49U + (TIMER1TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Lpspi0EndOfFrameToTimer1Trigger           = 52U + (TIMER1TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Lpspi0ReceivedDataWordToTimer1Trigger     = 53U + (TIMER1TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Lpspi1EndOfFrameToTimer1Trigger           = 54U + (TIMER1TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Lpspi1ReceivedDataWordToTimer1Trigger     = 55U + (TIMER1TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Lpuart0ReceivedDataWordToTimer1Trigger    = 56U + (TIMER1TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Lpuart0TransmittedDataWordToTimer1Trigger = 57U + (TIMER1TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Lpuart0ReceiveLineIdleToTimer1Trigger     = 58U + (TIMER1TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Lpuart1ReceivedDataWordToTimer1Trigger    = 59U + (TIMER1TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Lpuart1TransmittedDataWordToTimer1Trigger = 60U + (TIMER1TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Lpuart1ReceiveLineIdleToTimer1Trigger     = 61U + (TIMER1TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Lpuart2ReceivedDataWordToTimer1Trigger    = 62U + (TIMER1TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Lpuart2TransmittedDataWordToTimer1Trigger = 63U + (TIMER1TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Lpuart2ReceiveLineIdleToTimer1Trigger     = 64U + (TIMER1TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Lpuart3ReceivedDataWordToTimer1Trigger    = 65U + (TIMER1TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Lpuart3TransmittedDataWordToTimer1Trigger = 66U + (TIMER1TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Lpuart3ReceiveLineIdleToTimer1Trigger     = 67U + (TIMER1TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Lpuart4ReceivedDataWordToTimer1Trigger    = 68U + (TIMER1TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Lpuart4TransmittedDataWordToTimer1Trigger = 69U + (TIMER1TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Lpuart4ReceiveLineIdleToTimer1Trigger     = 70U + (TIMER1TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out0ToTimer1Trigger                   = 71U + (TIMER1TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out1ToTimer1Trigger                   = 72U + (TIMER1TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out2ToTimer1Trigger                   = 73U + (TIMER1TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out3ToTimer1Trigger                   = 74U + (TIMER1TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Adc1Tcomp0ToTimer1Trigger                 = 75U + (TIMER1TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Adc1Tcomp1ToTimer1Trigger                 = 76U + (TIMER1TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Adc1Tcomp2ToTimer1Trigger                 = 77U + (TIMER1TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Adc1Tcomp3ToTimer1Trigger                 = 78U + (TIMER1TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Ctimer3M1ToTimer1Trigger                  = 79U + (TIMER1TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Ctimer3M2ToTimer1Trigger                  = 80U + (TIMER1TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Ctimer3M3ToTimer1Trigger                  = 81U + (TIMER1TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Ctimer4M1ToTimer1Trigger                  = 82U + (TIMER1TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Ctimer4M2ToTimer1Trigger                  = 83U + (TIMER1TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Ctimer4M3ToTimer1Trigger                  = 84U + (TIMER1TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Qdc1CmpFlag0ToTimer1Trigger               = 85U + (TIMER1TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Qdc1CmpFlag1ToTimer1Trigger               = 86U + (TIMER1TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Qdc1CmpFlag2ToTimer1Trigger               = 87U + (TIMER1TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Qdc1CmpFlag3ToTimer1Trigger               = 88U + (TIMER1TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Qdc1PosMatch0ToTimer1Trigger              = 89U + (TIMER1TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Pwm1Sm0OutTrig0ToTimer1Trigger            = 90U + (TIMER1TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Pwm1Sm1OutTrig0ToTimer1Trigger            = 91U + (TIMER1TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Pwm1Sm2OutTrig0ToTimer1Trigger            = 92U + (TIMER1TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Lpi2c2MasterEndOfPacketToTimer1Trigger    = 94U + (TIMER1TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Lpi2c2SlaveEndOfPacketToTimer1Trigger     = 95U + (TIMER1TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Lpi2c3MasterEndOfPacketToTimer1Trigger    = 96U + (TIMER1TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Lpi2c3SlaveEndOfPacketToTimer1Trigger     = 97U + (TIMER1TRIGIN << PMUX_SHIFT),

    /*!< Timer2 Trigger. */
    kINPUTMUX_CtimerInp0ToTimer2Trigger                 = 1U + (TIMER2TRIGIN << PMUX_SHIFT),
    kINPUTMUX_CtimerInp1ToTimer2Trigger                 = 2U + (TIMER2TRIGIN << PMUX_SHIFT),
    kINPUTMUX_CtimerInp2ToTimer2Trigger                 = 3U + (TIMER2TRIGIN << PMUX_SHIFT),
    kINPUTMUX_CtimerInp3ToTimer2Trigger                 = 4U + (TIMER2TRIGIN << PMUX_SHIFT),
    kINPUTMUX_CtimerInp4ToTimer2Trigger                 = 5U + (TIMER2TRIGIN << PMUX_SHIFT),
    kINPUTMUX_CtimerInp5ToTimer2Trigger                 = 6U + (TIMER2TRIGIN << PMUX_SHIFT),
    kINPUTMUX_CtimerInp6ToTimer2Trigger                 = 7U + (TIMER2TRIGIN << PMUX_SHIFT),
    kINPUTMUX_CtimerInp7ToTimer2Trigger                 = 8U + (TIMER2TRIGIN << PMUX_SHIFT),
    kINPUTMUX_CtimerInp8ToTimer2Trigger                 = 9U + (TIMER2TRIGIN << PMUX_SHIFT),
    kINPUTMUX_CtimerInp9ToTimer2Trigger                 = 10U + (TIMER2TRIGIN << PMUX_SHIFT),
    kINPUTMUX_CtimerInp10ToTimer2Trigger                = 11U + (TIMER2TRIGIN << PMUX_SHIFT),
    kINPUTMUX_CtimerInp11ToTimer2Trigger                = 12U + (TIMER2TRIGIN << PMUX_SHIFT),
    kINPUTMUX_CtimerInp12ToTimer2Trigger                = 13U + (TIMER2TRIGIN << PMUX_SHIFT),
    kINPUTMUX_CtimerInp13ToTimer2Trigger                = 14U + (TIMER2TRIGIN << PMUX_SHIFT),
    kINPUTMUX_CtimerInp14ToTimer2Trigger                = 15U + (TIMER2TRIGIN << PMUX_SHIFT),
    kINPUTMUX_CtimerInp15ToTimer2Trigger                = 16U + (TIMER2TRIGIN << PMUX_SHIFT),
    kINPUTMUX_CtimerInp16ToTimer2Trigger                = 17U + (TIMER2TRIGIN << PMUX_SHIFT),
    kINPUTMUX_CtimerInp17ToTimer2Trigger                = 18U + (TIMER2TRIGIN << PMUX_SHIFT),
    kINPUTMUX_CtimerInp18ToTimer2Trigger                = 19U + (TIMER2TRIGIN << PMUX_SHIFT),
    kINPUTMUX_CtimerInp19ToTimer2Trigger                = 20U + (TIMER2TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Usb0StartOfFrameToTimer2Trigger           = 21U + (TIMER2TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out0ToTimer2Trigger                   = 22U + (TIMER2TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out1ToTimer2Trigger                   = 23U + (TIMER2TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out2ToTimer2Trigger                   = 24U + (TIMER2TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out3ToTimer2Trigger                   = 25U + (TIMER2TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Adc0Tcomp0ToTimer2Trigger                 = 26U + (TIMER2TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Adc0Tcomp1ToTimer2Trigger                 = 27U + (TIMER2TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Adc0Tcomp2ToTimer2Trigger                 = 28U + (TIMER2TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Adc0Tcomp3ToTimer2Trigger                 = 29U + (TIMER2TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Cmp0OutToTimer2Trigger                    = 30U + (TIMER2TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Cmp1OutToTimer2Trigger                    = 31U + (TIMER2TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Ctimer0M1ToTimer2Trigger                  = 33U + (TIMER2TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Ctimer0M2ToTimer2Trigger                  = 34U + (TIMER2TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Ctimer0M3ToTimer2Trigger                  = 35U + (TIMER2TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Ctimer1M1ToTimer2Trigger                  = 36U + (TIMER2TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Ctimer1M2ToTimer2Trigger                  = 37U + (TIMER2TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Ctimer1M3ToTimer2Trigger                  = 38U + (TIMER2TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Qdc0CmpFlag0ToTimer2Trigger               = 39U + (TIMER2TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Qdc0CmpFlag1ToTimer2Trigger               = 40U + (TIMER2TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Qdc0CmpFlag2ToTimer2Trigger               = 41U + (TIMER2TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Qdc0CmpFlag3ToTimer2Trigger               = 42U + (TIMER2TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Qdc0PosMatch0ToTimer2Trigger              = 43U + (TIMER2TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm0OutTrig0ToTimer2Trigger            = 44U + (TIMER2TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm1OutTrig0ToTimer2Trigger            = 45U + (TIMER2TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm2OutTrig0ToTimer2Trigger            = 46U + (TIMER2TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Lpi2c0MasterEndOfPacketToTimer2Trigger    = 48U + (TIMER2TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Lpi2c0SlaveEndOfPacketToTimer2Trigger     = 49U + (TIMER2TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Lpspi0EndOfFrameToTimer2Trigger           = 52U + (TIMER2TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Lpspi0ReceivedDataWordToTimer2Trigger     = 53U + (TIMER2TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Lpspi1EndOfFrameToTimer2Trigger           = 54U + (TIMER2TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Lpspi1ReceivedDataWordToTimer2Trigger     = 55U + (TIMER2TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Lpuart0ReceivedDataWordToTimer2Trigger    = 56U + (TIMER2TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Lpuart0TransmittedDataWordToTimer2Trigger = 57U + (TIMER2TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Lpuart0ReceiveLineIdleToTimer2Trigger     = 58U + (TIMER2TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Lpuart1ReceivedDataWordToTimer2Trigger    = 59U + (TIMER2TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Lpuart1TransmittedDataWordToTimer2Trigger = 60U + (TIMER2TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Lpuart1ReceiveLineIdleToTimer2Trigger     = 61U + (TIMER2TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Lpuart2ReceivedDataWordToTimer2Trigger    = 62U + (TIMER2TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Lpuart2TransmittedDataWordToTimer2Trigger = 63U + (TIMER2TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Lpuart2ReceiveLineIdleToTimer2Trigger     = 64U + (TIMER2TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Lpuart3ReceivedDataWordToTimer2Trigger    = 65U + (TIMER2TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Lpuart3TransmittedDataWordToTimer2Trigger = 66U + (TIMER2TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Lpuart3ReceiveLineIdleToTimer2Trigger     = 67U + (TIMER2TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Lpuart4ReceivedDataWordToTimer2Trigger    = 68U + (TIMER2TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Lpuart4TransmittedDataWordToTimer2Trigger = 69U + (TIMER2TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Lpuart4ReceiveLineIdleToTimer2Trigger     = 70U + (TIMER2TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out0ToTimer2Trigger                   = 71U + (TIMER2TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out1ToTimer2Trigger                   = 72U + (TIMER2TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out2ToTimer2Trigger                   = 73U + (TIMER2TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out3ToTimer2Trigger                   = 74U + (TIMER2TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Adc1Tcomp0ToTimer2Trigger                 = 75U + (TIMER2TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Adc1Tcomp1ToTimer2Trigger                 = 76U + (TIMER2TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Adc1Tcomp2ToTimer2Trigger                 = 77U + (TIMER2TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Adc1Tcomp3ToTimer2Trigger                 = 78U + (TIMER2TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Ctimer3M1ToTimer2Trigger                  = 79U + (TIMER2TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Ctimer3M2ToTimer2Trigger                  = 80U + (TIMER2TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Ctimer3M3ToTimer2Trigger                  = 81U + (TIMER2TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Ctimer4M1ToTimer2Trigger                  = 82U + (TIMER2TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Ctimer4M2ToTimer2Trigger                  = 83U + (TIMER2TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Ctimer4M3ToTimer2Trigger                  = 84U + (TIMER2TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Qdc1CmpFlag0ToTimer2Trigger               = 85U + (TIMER2TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Qdc1CmpFlag1ToTimer2Trigger               = 86U + (TIMER2TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Qdc1CmpFlag2ToTimer2Trigger               = 87U + (TIMER2TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Qdc1CmpFlag3ToTimer2Trigger               = 88U + (TIMER2TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Qdc1PosMatch0ToTimer2Trigger              = 89U + (TIMER2TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Pwm1Sm0OutTrig0ToTimer2Trigger            = 90U + (TIMER2TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Pwm1Sm1OutTrig0ToTimer2Trigger            = 91U + (TIMER2TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Pwm1Sm2OutTrig0ToTimer2Trigger            = 92U + (TIMER2TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Lpi2c2MasterEndOfPacketToTimer2Trigger    = 94U + (TIMER2TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Lpi2c2SlaveEndOfPacketToTimer2Trigger     = 95U + (TIMER2TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Lpi2c3MasterEndOfPacketToTimer2Trigger    = 96U + (TIMER2TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Lpi2c3SlaveEndOfPacketToTimer2Trigger     = 97U + (TIMER2TRIGIN << PMUX_SHIFT),

    /*!< Timer3 Trigger. */
    kINPUTMUX_CtimerInp0ToTimer3Trigger                 = 1U + (TIMER3TRIGIN << PMUX_SHIFT),
    kINPUTMUX_CtimerInp1ToTimer3Trigger                 = 2U + (TIMER3TRIGIN << PMUX_SHIFT),
    kINPUTMUX_CtimerInp2ToTimer3Trigger                 = 3U + (TIMER3TRIGIN << PMUX_SHIFT),
    kINPUTMUX_CtimerInp3ToTimer3Trigger                 = 4U + (TIMER3TRIGIN << PMUX_SHIFT),
    kINPUTMUX_CtimerInp4ToTimer3Trigger                 = 5U + (TIMER3TRIGIN << PMUX_SHIFT),
    kINPUTMUX_CtimerInp5ToTimer3Trigger                 = 6U + (TIMER3TRIGIN << PMUX_SHIFT),
    kINPUTMUX_CtimerInp6ToTimer3Trigger                 = 7U + (TIMER3TRIGIN << PMUX_SHIFT),
    kINPUTMUX_CtimerInp7ToTimer3Trigger                 = 8U + (TIMER3TRIGIN << PMUX_SHIFT),
    kINPUTMUX_CtimerInp8ToTimer3Trigger                 = 9U + (TIMER3TRIGIN << PMUX_SHIFT),
    kINPUTMUX_CtimerInp9ToTimer3Trigger                 = 10U + (TIMER3TRIGIN << PMUX_SHIFT),
    kINPUTMUX_CtimerInp10ToTimer3Trigger                = 11U + (TIMER3TRIGIN << PMUX_SHIFT),
    kINPUTMUX_CtimerInp11ToTimer3Trigger                = 12U + (TIMER3TRIGIN << PMUX_SHIFT),
    kINPUTMUX_CtimerInp12ToTimer3Trigger                = 13U + (TIMER3TRIGIN << PMUX_SHIFT),
    kINPUTMUX_CtimerInp13ToTimer3Trigger                = 14U + (TIMER3TRIGIN << PMUX_SHIFT),
    kINPUTMUX_CtimerInp14ToTimer3Trigger                = 15U + (TIMER3TRIGIN << PMUX_SHIFT),
    kINPUTMUX_CtimerInp15ToTimer3Trigger                = 16U + (TIMER3TRIGIN << PMUX_SHIFT),
    kINPUTMUX_CtimerInp16ToTimer3Trigger                = 17U + (TIMER3TRIGIN << PMUX_SHIFT),
    kINPUTMUX_CtimerInp17ToTimer3Trigger                = 18U + (TIMER3TRIGIN << PMUX_SHIFT),
    kINPUTMUX_CtimerInp18ToTimer3Trigger                = 19U + (TIMER3TRIGIN << PMUX_SHIFT),
    kINPUTMUX_CtimerInp19ToTimer3Trigger                = 20U + (TIMER3TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Usb0StartOfFrameToTimer3Trigger           = 21U + (TIMER3TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out0ToTimer3Trigger                   = 22U + (TIMER3TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out1ToTimer3Trigger                   = 23U + (TIMER3TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out2ToTimer3Trigger                   = 24U + (TIMER3TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out3ToTimer3Trigger                   = 25U + (TIMER3TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Adc0Tcomp0ToTimer3Trigger                 = 26U + (TIMER3TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Adc0Tcomp1ToTimer3Trigger                 = 27U + (TIMER3TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Adc0Tcomp2ToTimer3Trigger                 = 28U + (TIMER3TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Adc0Tcomp3ToTimer3Trigger                 = 29U + (TIMER3TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Cmp0OutToTimer3Trigger                    = 30U + (TIMER3TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Cmp1OutToTimer3Trigger                    = 31U + (TIMER3TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Ctimer0M1ToTimer3Trigger                  = 33U + (TIMER3TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Ctimer0M2ToTimer3Trigger                  = 34U + (TIMER3TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Ctimer0M3ToTimer3Trigger                  = 35U + (TIMER3TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Ctimer1M1ToTimer3Trigger                  = 36U + (TIMER3TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Ctimer1M2ToTimer3Trigger                  = 37U + (TIMER3TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Ctimer1M3ToTimer3Trigger                  = 38U + (TIMER3TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Qdc0CmpFlag0ToTimer3Trigger               = 39U + (TIMER3TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Qdc0CmpFlag1ToTimer3Trigger               = 40U + (TIMER3TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Qdc0CmpFlag2ToTimer3Trigger               = 41U + (TIMER3TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Qdc0CmpFlag3ToTimer3Trigger               = 42U + (TIMER3TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Qdc0PosMatch0ToTimer3Trigger              = 43U + (TIMER3TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm0OutTrig0ToTimer3Trigger            = 44U + (TIMER3TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm1OutTrig0ToTimer3Trigger            = 45U + (TIMER3TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm2OutTrig0ToTimer3Trigger            = 46U + (TIMER3TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Lpi2c0MasterEndOfPacketToTimer3Trigger    = 48U + (TIMER3TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Lpi2c0SlaveEndOfPacketToTimer3Trigger     = 49U + (TIMER3TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Lpspi0EndOfFrameToTimer3Trigger           = 52U + (TIMER3TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Lpspi0ReceivedDataWordToTimer3Trigger     = 53U + (TIMER3TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Lpspi1EndOfFrameToTimer3Trigger           = 54U + (TIMER3TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Lpspi1ReceivedDataWordToTimer3Trigger     = 55U + (TIMER3TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Lpuart0ReceivedDataWordToTimer3Trigger    = 56U + (TIMER3TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Lpuart0TransmittedDataWordToTimer3Trigger = 57U + (TIMER3TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Lpuart0ReceiveLineIdleToTimer3Trigger     = 58U + (TIMER3TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Lpuart1ReceivedDataWordToTimer3Trigger    = 59U + (TIMER3TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Lpuart1TransmittedDataWordToTimer3Trigger = 60U + (TIMER3TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Lpuart1ReceiveLineIdleToTimer3Trigger     = 61U + (TIMER3TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Lpuart2ReceivedDataWordToTimer3Trigger    = 62U + (TIMER3TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Lpuart2TransmittedDataWordToTimer3Trigger = 63U + (TIMER3TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Lpuart2ReceiveLineIdleToTimer3Trigger     = 64U + (TIMER3TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Lpuart3ReceivedDataWordToTimer3Trigger    = 65U + (TIMER3TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Lpuart3TransmittedDataWordToTimer3Trigger = 66U + (TIMER3TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Lpuart3ReceiveLineIdleToTimer3Trigger     = 67U + (TIMER3TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Lpuart4ReceivedDataWordToTimer3Trigger    = 68U + (TIMER3TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Lpuart4TransmittedDataWordToTimer3Trigger = 69U + (TIMER3TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Lpuart4ReceiveLineIdleToTimer3Trigger     = 70U + (TIMER3TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out0ToTimer3Trigger                   = 71U + (TIMER3TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out1ToTimer3Trigger                   = 72U + (TIMER3TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out2ToTimer3Trigger                   = 73U + (TIMER3TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out3ToTimer3Trigger                   = 74U + (TIMER3TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Adc1Tcomp0ToTimer3Trigger                 = 75U + (TIMER3TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Adc1Tcomp1ToTimer3Trigger                 = 76U + (TIMER3TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Adc1Tcomp2ToTimer3Trigger                 = 77U + (TIMER3TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Adc1Tcomp3ToTimer3Trigger                 = 78U + (TIMER3TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Ctimer2M1ToTimer3Trigger                  = 79U + (TIMER3TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Ctimer2M2ToTimer3Trigger                  = 80U + (TIMER3TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Ctimer2M3ToTimer3Trigger                  = 81U + (TIMER3TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Ctimer4M1ToTimer3Trigger                  = 82U + (TIMER3TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Ctimer4M2ToTimer3Trigger                  = 83U + (TIMER3TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Ctimer4M3ToTimer3Trigger                  = 84U + (TIMER3TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Qdc1CmpFlag0ToTimer3Trigger               = 85U + (TIMER3TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Qdc1CmpFlag1ToTimer3Trigger               = 86U + (TIMER3TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Qdc1CmpFlag2ToTimer3Trigger               = 87U + (TIMER3TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Qdc1CmpFlag3ToTimer3Trigger               = 88U + (TIMER3TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Qdc1PosMatch0ToTimer3Trigger              = 89U + (TIMER3TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Pwm1Sm0OutTrig0ToTimer3Trigger            = 90U + (TIMER3TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Pwm1Sm1OutTrig0ToTimer3Trigger            = 91U + (TIMER3TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Pwm1Sm2OutTrig0ToTimer3Trigger            = 92U + (TIMER3TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Lpi2c2MasterEndOfPacketToTimer3Trigger    = 94U + (TIMER3TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Lpi2c2SlaveEndOfPacketToTimer3Trigger     = 95U + (TIMER3TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Lpi2c3MasterEndOfPacketToTimer3Trigger    = 96U + (TIMER3TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Lpi2c3SlaveEndOfPacketToTimer3Trigger     = 97U + (TIMER3TRIGIN << PMUX_SHIFT),

    /*!< Timer4 Trigger. */
    kINPUTMUX_CtimerInp0ToTimer4Trigger                 = 1U + (TIMER4TRIGIN << PMUX_SHIFT),
    kINPUTMUX_CtimerInp1ToTimer4Trigger                 = 2U + (TIMER4TRIGIN << PMUX_SHIFT),
    kINPUTMUX_CtimerInp2ToTimer4Trigger                 = 3U + (TIMER4TRIGIN << PMUX_SHIFT),
    kINPUTMUX_CtimerInp3ToTimer4Trigger                 = 4U + (TIMER4TRIGIN << PMUX_SHIFT),
    kINPUTMUX_CtimerInp4ToTimer4Trigger                 = 5U + (TIMER4TRIGIN << PMUX_SHIFT),
    kINPUTMUX_CtimerInp5ToTimer4Trigger                 = 6U + (TIMER4TRIGIN << PMUX_SHIFT),
    kINPUTMUX_CtimerInp6ToTimer4Trigger                 = 7U + (TIMER4TRIGIN << PMUX_SHIFT),
    kINPUTMUX_CtimerInp7ToTimer4Trigger                 = 8U + (TIMER4TRIGIN << PMUX_SHIFT),
    kINPUTMUX_CtimerInp8ToTimer4Trigger                 = 9U + (TIMER4TRIGIN << PMUX_SHIFT),
    kINPUTMUX_CtimerInp9ToTimer4Trigger                 = 10U + (TIMER4TRIGIN << PMUX_SHIFT),
    kINPUTMUX_CtimerInp10ToTimer4Trigger                = 11U + (TIMER4TRIGIN << PMUX_SHIFT),
    kINPUTMUX_CtimerInp11ToTimer4Trigger                = 12U + (TIMER4TRIGIN << PMUX_SHIFT),
    kINPUTMUX_CtimerInp12ToTimer4Trigger                = 13U + (TIMER4TRIGIN << PMUX_SHIFT),
    kINPUTMUX_CtimerInp13ToTimer4Trigger                = 14U + (TIMER4TRIGIN << PMUX_SHIFT),
    kINPUTMUX_CtimerInp14ToTimer4Trigger                = 15U + (TIMER4TRIGIN << PMUX_SHIFT),
    kINPUTMUX_CtimerInp15ToTimer4Trigger                = 16U + (TIMER4TRIGIN << PMUX_SHIFT),
    kINPUTMUX_CtimerInp16ToTimer4Trigger                = 17U + (TIMER4TRIGIN << PMUX_SHIFT),
    kINPUTMUX_CtimerInp17ToTimer4Trigger                = 18U + (TIMER4TRIGIN << PMUX_SHIFT),
    kINPUTMUX_CtimerInp18ToTimer4Trigger                = 19U + (TIMER4TRIGIN << PMUX_SHIFT),
    kINPUTMUX_CtimerInp19ToTimer4Trigger                = 20U + (TIMER4TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Usb0StartOfFrameToTimer4Trigger           = 21U + (TIMER4TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out0ToTimer4Trigger                   = 22U + (TIMER4TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out1ToTimer4Trigger                   = 23U + (TIMER4TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out2ToTimer4Trigger                   = 24U + (TIMER4TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out3ToTimer4Trigger                   = 25U + (TIMER4TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Adc0Tcomp0ToTimer4Trigger                 = 26U + (TIMER4TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Adc0Tcomp1ToTimer4Trigger                 = 27U + (TIMER4TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Adc0Tcomp2ToTimer4Trigger                 = 28U + (TIMER4TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Adc0Tcomp3ToTimer4Trigger                 = 29U + (TIMER4TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Cmp0OutToTimer4Trigger                    = 30U + (TIMER4TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Cmp1OutToTimer4Trigger                    = 31U + (TIMER4TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Ctimer0M1ToTimer4Trigger                  = 33U + (TIMER4TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Ctimer0M2ToTimer4Trigger                  = 34U + (TIMER4TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Ctimer0M3ToTimer4Trigger                  = 35U + (TIMER4TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Ctimer1M1ToTimer4Trigger                  = 36U + (TIMER4TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Ctimer1M2ToTimer4Trigger                  = 37U + (TIMER4TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Ctimer1M3ToTimer4Trigger                  = 38U + (TIMER4TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Qdc0CmpFlag0ToTimer4Trigger               = 39U + (TIMER4TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Qdc0CmpFlag1ToTimer4Trigger               = 40U + (TIMER4TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Qdc0CmpFlag2ToTimer4Trigger               = 41U + (TIMER4TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Qdc0CmpFlag3ToTimer4Trigger               = 42U + (TIMER4TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Qdc0PosMatch0ToTimer4Trigger              = 43U + (TIMER4TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm0OutTrig0ToTimer4Trigger            = 44U + (TIMER4TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm1OutTrig0ToTimer4Trigger            = 45U + (TIMER4TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm2OutTrig0ToTimer4Trigger            = 46U + (TIMER4TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Lpi2c0MasterEndOfPacketToTimer4Trigger    = 48U + (TIMER4TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Lpi2c0SlaveEndOfPacketToTimer4Trigger     = 49U + (TIMER4TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Lpspi0EndOfFrameToTimer4Trigger           = 52U + (TIMER4TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Lpspi0ReceivedDataWordToTimer4Trigger     = 53U + (TIMER4TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Lpspi1EndOfFrameToTimer4Trigger           = 54U + (TIMER4TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Lpspi1ReceivedDataWordToTimer4Trigger     = 55U + (TIMER4TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Lpuart0ReceivedDataWordToTimer4Trigger    = 56U + (TIMER4TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Lpuart0TransmittedDataWordToTimer4Trigger = 57U + (TIMER4TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Lpuart0ReceiveLineIdleToTimer4Trigger     = 58U + (TIMER4TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Lpuart1ReceivedDataWordToTimer4Trigger    = 59U + (TIMER4TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Lpuart1TransmittedDataWordToTimer4Trigger = 60U + (TIMER4TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Lpuart1ReceiveLineIdleToTimer4Trigger     = 61U + (TIMER4TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Lpuart2ReceivedDataWordToTimer4Trigger    = 62U + (TIMER4TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Lpuart2TransmittedDataWordToTimer4Trigger = 63U + (TIMER4TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Lpuart2ReceiveLineIdleToTimer4Trigger     = 64U + (TIMER4TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Lpuart3ReceivedDataWordToTimer4Trigger    = 65U + (TIMER4TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Lpuart3TransmittedDataWordToTimer4Trigger = 66U + (TIMER4TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Lpuart3ReceiveLineIdleToTimer4Trigger     = 67U + (TIMER4TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Lpuart4ReceivedDataWordToTimer4Trigger    = 68U + (TIMER4TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Lpuart4TransmittedDataWordToTimer4Trigger = 69U + (TIMER4TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Lpuart4ReceiveLineIdleToTimer4Trigger     = 70U + (TIMER4TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out0ToTimer4Trigger                   = 71U + (TIMER4TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out1ToTimer4Trigger                   = 72U + (TIMER4TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out2ToTimer4Trigger                   = 73U + (TIMER4TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out3ToTimer4Trigger                   = 74U + (TIMER4TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Adc1Tcomp0ToTimer4Trigger                 = 75U + (TIMER4TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Adc1Tcomp1ToTimer4Trigger                 = 76U + (TIMER4TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Adc1Tcomp2ToTimer4Trigger                 = 77U + (TIMER4TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Adc1Tcomp3ToTimer4Trigger                 = 78U + (TIMER4TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Ctimer2M1ToTimer4Trigger                  = 79U + (TIMER4TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Ctimer2M2ToTimer4Trigger                  = 80U + (TIMER4TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Ctimer2M3ToTimer4Trigger                  = 81U + (TIMER4TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Ctimer3M1ToTimer4Trigger                  = 82U + (TIMER4TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Ctimer3M2ToTimer4Trigger                  = 83U + (TIMER4TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Ctimer3M3ToTimer4Trigger                  = 84U + (TIMER4TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Qdc1CmpFlag0ToTimer4Trigger               = 85U + (TIMER4TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Qdc1CmpFlag1ToTimer4Trigger               = 86U + (TIMER4TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Qdc1CmpFlag2ToTimer4Trigger               = 87U + (TIMER4TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Qdc1CmpFlag3ToTimer4Trigger               = 88U + (TIMER4TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Qdc1PosMatch0ToTimer4Trigger              = 89U + (TIMER4TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Pwm1Sm0OutTrig0ToTimer4Trigger            = 90U + (TIMER4TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Pwm1Sm1OutTrig0ToTimer4Trigger            = 91U + (TIMER4TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Pwm1Sm2OutTrig0ToTimer4Trigger            = 92U + (TIMER4TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Lpi2c2MasterEndOfPacketToTimer4Trigger    = 94U + (TIMER4TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Lpi2c2SlaveEndOfPacketToTimer4Trigger     = 95U + (TIMER4TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Lpi2c3MasterEndOfPacketToTimer4Trigger    = 96U + (TIMER4TRIGIN << PMUX_SHIFT),
    kINPUTMUX_Lpi2c3SlaveEndOfPacketToTimer4Trigger     = 97U + (TIMER4TRIGIN << PMUX_SHIFT),
        
    /*!< Selection for frequency measurement reference clock. */
    kINPUTMUX_ClkInToFreqmeasRef           = 1U + (FREQMEAS_REF_REG << PMUX_SHIFT),
    kINPUTMUX_FroOsc12MToFreqmeasRef       = 2u + (FREQMEAS_REF_REG << PMUX_SHIFT),
    kINPUTMUX_FroHfDivToFreqmeasRef        = 3u + (FREQMEAS_REF_REG << PMUX_SHIFT),
    kINPUTMUX_Clk16K1ToFreqmeasRef         = 5u + (FREQMEAS_REF_REG << PMUX_SHIFT),
    kINPUTMUX_SlowClkToFreqmeasRef         = 6u + (FREQMEAS_REF_REG << PMUX_SHIFT),
    kINPUTMUX_FreqmeClkIn0ToFreqmeasRef    = 7u + (FREQMEAS_REF_REG << PMUX_SHIFT),
    kINPUTMUX_FreqmeClkIn1ToFreqmeasRef    = 8u + (FREQMEAS_REF_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out0ToFreqmeasRef        = 9u + (FREQMEAS_REF_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out1ToFreqmeasRef        = 10u + (FREQMEAS_REF_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm0OutTrig0ToFreqmeasRef = 11u + (FREQMEAS_REF_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm0OutTrig1ToFreqmeasRef = 12u + (FREQMEAS_REF_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm1OutTrig0ToFreqmeasRef = 13u + (FREQMEAS_REF_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm1OutTrig1ToFreqmeasRef = 14u + (FREQMEAS_REF_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm2OutTrig0ToFreqmeasRef = 15u + (FREQMEAS_REF_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm2OutTrig1ToFreqmeasRef = 16u + (FREQMEAS_REF_REG << PMUX_SHIFT),

    /*!< Selection for frequency measurement target clock. */
    kINPUTMUX_ClkInToFreqmeasTar           = 1U + (FREQMEAS_TAR_REG << PMUX_SHIFT),
    kINPUTMUX_FroOsc12MToFreqmeasTar       = 2u + (FREQMEAS_TAR_REG << PMUX_SHIFT),
    kINPUTMUX_FroHfDivToFreqmeasTar        = 3u + (FREQMEAS_TAR_REG << PMUX_SHIFT),
    kINPUTMUX_Clk16K1ToFreqmeasTar         = 5u + (FREQMEAS_TAR_REG << PMUX_SHIFT),
    kINPUTMUX_SlowClkToFreqmeasTar         = 6u + (FREQMEAS_TAR_REG << PMUX_SHIFT),
    kINPUTMUX_FreqmeClkIn0ToFreqmeasTar    = 7u + (FREQMEAS_TAR_REG << PMUX_SHIFT),
    kINPUTMUX_FreqmeClkIn1ToFreqmeasTar    = 8u + (FREQMEAS_TAR_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out0ToFreqmeasTar        = 9u + (FREQMEAS_TAR_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out1ToFreqmeasTar        = 10u + (FREQMEAS_TAR_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm0OutTrig0ToFreqmeasTar = 11u + (FREQMEAS_TAR_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm0OutTrig1ToFreqmeasTar = 12u + (FREQMEAS_TAR_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm1OutTrig0ToFreqmeasTar = 13u + (FREQMEAS_TAR_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm1OutTrig1ToFreqmeasTar = 14u + (FREQMEAS_TAR_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm2OutTrig0ToFreqmeasTar = 15u + (FREQMEAS_TAR_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm2OutTrig1ToFreqmeasTar = 16u + (FREQMEAS_TAR_REG << PMUX_SHIFT),

    /*!< Cmp0 Trigger. */
    kINPUTMUX_Aoi0Out0ToCmp0Trigger           = 2U + (CMP0_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out1ToCmp0Trigger           = 3U + (CMP0_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out2ToCmp0Trigger           = 4U + (CMP0_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out3ToCmp0Trigger           = 5U + (CMP0_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Cmp1OutToCmp0Trigger            = 6U + (CMP0_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer0M0ToCmp0Trigger          = 8U + (CMP0_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer0M2ToCmp0Trigger          = 9U + (CMP0_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer1M0ToCmp0Trigger          = 10U + (CMP0_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer1M2ToCmp0Trigger          = 11U + (CMP0_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer2M0ToCmp0Trigger          = 12U + (CMP0_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer2M2ToCmp0Trigger          = 13U + (CMP0_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Lptmr0ToCmp0Trigger             = 14U + (CMP0_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Qdc0PosMatch0ToCmp0Trigger      = 16U + (CMP0_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm0OutTrig0ToCmp0Trigger    = 17U + (CMP0_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm0OutTrig1ToCmp0Trigger    = 18U + (CMP0_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm1OutTrig0ToCmp0Trigger    = 19U + (CMP0_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm1OutTrig1ToCmp0Trigger    = 20U + (CMP0_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm2OutTrig0ToCmp0Trigger    = 21U + (CMP0_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm2OutTrig1ToCmp0Trigger    = 22U + (CMP0_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio0PinEventTrig0ToCmp0Trigger = 25U + (CMP0_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio1PinEventTrig0ToCmp0Trigger = 26U + (CMP0_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio2PinEventTrig0ToCmp0Trigger = 27U + (CMP0_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio3PinEventTrig0ToCmp0Trigger = 28U + (CMP0_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio4PinEventTrig0ToCmp0Trigger = 29U + (CMP0_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_WuuToCmp0Trigger                = 30U + (CMP0_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out0ToCmp0Trigger           = 31U + (CMP0_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out1ToCmp0Trigger           = 32U + (CMP0_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out2ToCmp0Trigger           = 33U + (CMP0_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out3ToCmp0Trigger           = 34U + (CMP0_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer3M0ToCmp0Trigger          = 39U + (CMP0_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer3M1ToCmp0Trigger          = 40U + (CMP0_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer4M0ToCmp0Trigger          = 41U + (CMP0_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer4M1ToCmp0Trigger          = 42U + (CMP0_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Qdc1PosMatch0ToCmp0Trigger      = 47U + (CMP0_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm1Sm0OutTrig0ToCmp0Trigger    = 48U + (CMP0_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm1Sm0OutTrig1ToCmp0Trigger    = 49U + (CMP0_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm1Sm1OutTrig0ToCmp0Trigger    = 50U + (CMP0_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm1Sm1OutTrig1ToCmp0Trigger    = 51U + (CMP0_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm1Sm2OutTrig0ToCmp0Trigger    = 52U + (CMP0_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm1Sm2OutTrig1ToCmp0Trigger    = 53U + (CMP0_TRIG_REG << PMUX_SHIFT),

    /*!< Cmp1 Trigger. */
    kINPUTMUX_Aoi0Out0ToCmp1Trigger           = 2U + (CMP1_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out1ToCmp1Trigger           = 3U + (CMP1_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out2ToCmp1Trigger           = 4U + (CMP1_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out3ToCmp1Trigger           = 5U + (CMP1_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Cmp0OutToCmp1Trigger            = 6U + (CMP1_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer0M0ToCmp1Trigger          = 8U + (CMP1_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer0M2ToCmp1Trigger          = 9U + (CMP1_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer1M0ToCmp1Trigger          = 10U + (CMP1_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer1M2ToCmp1Trigger          = 11U + (CMP1_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer2M0ToCmp1Trigger          = 12U + (CMP1_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer2M2ToCmp1Trigger          = 13U + (CMP1_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Lptmr0ToCmp1Trigger             = 14U + (CMP1_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Qdc0PosMatch0ToCmp1Trigger      = 16U + (CMP1_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm0OutTrig0ToCmp1Trigger    = 17U + (CMP1_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm0OutTrig1ToCmp1Trigger    = 18U + (CMP1_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm1OutTrig0ToCmp1Trigger    = 19U + (CMP1_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm1OutTrig1ToCmp1Trigger    = 20U + (CMP1_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm2OutTrig0ToCmp1Trigger    = 21U + (CMP1_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm2OutTrig1ToCmp1Trigger    = 22U + (CMP1_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio0PinEventTrig0ToCmp1Trigger = 25U + (CMP1_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio1PinEventTrig0ToCmp1Trigger = 26U + (CMP1_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio2PinEventTrig0ToCmp1Trigger = 27U + (CMP1_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio3PinEventTrig0ToCmp1Trigger = 28U + (CMP1_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio4PinEventTrig0ToCmp1Trigger = 29U + (CMP1_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_WuuToCmp1Trigger                = 30U + (CMP1_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out0ToCmp1Trigger           = 31U + (CMP1_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out1ToCmp1Trigger           = 32U + (CMP1_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out2ToCmp1Trigger           = 33U + (CMP1_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out3ToCmp1Trigger           = 34U + (CMP1_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer3M0ToCmp1Trigger          = 39U + (CMP1_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer3M1ToCmp1Trigger          = 40U + (CMP1_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer4M0ToCmp1Trigger          = 41U + (CMP1_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer4M1ToCmp1Trigger          = 42U + (CMP1_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Qdc1PosMatch0ToCmp1Trigger      = 47U + (CMP1_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm1Sm0OutTrig0ToCmp1Trigger    = 48U + (CMP1_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm1Sm0OutTrig1ToCmp1Trigger    = 49U + (CMP1_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm1Sm1OutTrig0ToCmp1Trigger    = 50U + (CMP1_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm1Sm1OutTrig1ToCmp1Trigger    = 51U + (CMP1_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm1Sm2OutTrig0ToCmp1Trigger    = 52U + (CMP1_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm1Sm2OutTrig1ToCmp1Trigger    = 53U + (CMP1_TRIG_REG << PMUX_SHIFT),

    /*!< Adc0 Trigger. */
    kINPUTMUX_ArmTxevToAdc0Trigger            = 1U + (ADC0_TRIG0_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out0ToAdc0Trigger           = 2U + (ADC0_TRIG0_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out1ToAdc0Trigger           = 3U + (ADC0_TRIG0_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out2ToAdc0Trigger           = 4U + (ADC0_TRIG0_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out3ToAdc0Trigger           = 5U + (ADC0_TRIG0_REG << PMUX_SHIFT),
    kINPUTMUX_Cmp0OutToAdc0Trigger            = 6U + (ADC0_TRIG0_REG << PMUX_SHIFT),
    kINPUTMUX_Cmp1OutToAdc0Trigger            = 7U + (ADC0_TRIG0_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer0M0ToAdc0Trigger          = 9U + (ADC0_TRIG0_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer0M1ToAdc0Trigger          = 10U + (ADC0_TRIG0_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer1M0ToAdc0Trigger          = 11U + (ADC0_TRIG0_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer1M1ToAdc0Trigger          = 12U + (ADC0_TRIG0_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer2M0ToAdc0Trigger          = 13U + (ADC0_TRIG0_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer2M1ToAdc0Trigger          = 14U + (ADC0_TRIG0_REG << PMUX_SHIFT),
    kINPUTMUX_Lptmr0ToAdc0Trigger             = 15U + (ADC0_TRIG0_REG << PMUX_SHIFT),
    kINPUTMUX_Qdc0PosMatch0ToAdc0Trigger      = 17U + (ADC0_TRIG0_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm0OutTrig0ToAdc0Trigger    = 18U + (ADC0_TRIG0_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm0OutTrig1ToAdc0Trigger    = 19U + (ADC0_TRIG0_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm1OutTrig0ToAdc0Trigger    = 20U + (ADC0_TRIG0_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm1OutTrig1ToAdc0Trigger    = 21U + (ADC0_TRIG0_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm2OutTrig0ToAdc0Trigger    = 22U + (ADC0_TRIG0_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm2OutTrig1ToAdc0Trigger    = 23U + (ADC0_TRIG0_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio0PinEventTrig0ToAdc0Trigger = 26U + (ADC0_TRIG0_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio1PinEventTrig0ToAdc0Trigger = 27U + (ADC0_TRIG0_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio2PinEventTrig0ToAdc0Trigger = 28U + (ADC0_TRIG0_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio3PinEventTrig0ToAdc0Trigger = 29U + (ADC0_TRIG0_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio4PinEventTrig0ToAdc0Trigger = 30U + (ADC0_TRIG0_REG << PMUX_SHIFT),
    kINPUTMUX_WuuToAdc0Trigger                = 31U + (ADC0_TRIG0_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out0ToAdc0Trigger           = 33U + (ADC0_TRIG0_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out1ToAdc0Trigger           = 34U + (ADC0_TRIG0_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out2ToAdc0Trigger           = 35U + (ADC0_TRIG0_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out3ToAdc0Trigger           = 36U + (ADC0_TRIG0_REG << PMUX_SHIFT),
    kINPUTMUX_Adc1Tcomp0ToAdc0Trigger         = 37U + (ADC0_TRIG0_REG << PMUX_SHIFT),
    kINPUTMUX_Adc1Tcomp1ToAdc0Trigger         = 38U + (ADC0_TRIG0_REG << PMUX_SHIFT),
    kINPUTMUX_Adc1Tcomp2ToAdc0Trigger         = 39U + (ADC0_TRIG0_REG << PMUX_SHIFT),
    kINPUTMUX_Adc1Tcomp3ToAdc0Trigger         = 40U + (ADC0_TRIG0_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer3M0ToAdc0Trigger          = 41U + (ADC0_TRIG0_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer3M1ToAdc0Trigger          = 42U + (ADC0_TRIG0_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer4M0ToAdc0Trigger          = 43U + (ADC0_TRIG0_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer4M1ToAdc0Trigger          = 44U + (ADC0_TRIG0_REG << PMUX_SHIFT),
    kINPUTMUX_FlexioCh0ToAdc0Trigger          = 45U + (ADC0_TRIG0_REG << PMUX_SHIFT),
    kINPUTMUX_FlexioCh1ToAdc0Trigger          = 46U + (ADC0_TRIG0_REG << PMUX_SHIFT),
    kINPUTMUX_FlexioCh2ToAdc0Trigger          = 47U + (ADC0_TRIG0_REG << PMUX_SHIFT),
    kINPUTMUX_FlexioCh3ToAdc0Trigger          = 48U + (ADC0_TRIG0_REG << PMUX_SHIFT),
    kINPUTMUX_Qdc1PosMatch0ToAdc0Trigger      = 49U + (ADC0_TRIG0_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm1Sm0OutTrig0ToAdc0Trigger    = 50U + (ADC0_TRIG0_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm1Sm0OutTrig1ToAdc0Trigger    = 51U + (ADC0_TRIG0_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm1Sm1OutTrig0ToAdc0Trigger    = 52U + (ADC0_TRIG0_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm1Sm1OutTrig1ToAdc0Trigger    = 53U + (ADC0_TRIG0_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm1Sm2OutTrig0ToAdc0Trigger    = 54U + (ADC0_TRIG0_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm1Sm2OutTrig1ToAdc0Trigger    = 55U + (ADC0_TRIG0_REG << PMUX_SHIFT),

    /*!< Adc1 Trigger. */
    kINPUTMUX_ArmTxevToAdc1Trigger            = 1U + (ADC1_TRIG0_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out0ToAdc1Trigger           = 2U + (ADC1_TRIG0_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out1ToAdc1Trigger           = 3U + (ADC1_TRIG0_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out2ToAdc1Trigger           = 4U + (ADC1_TRIG0_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out3ToAdc1Trigger           = 5U + (ADC1_TRIG0_REG << PMUX_SHIFT),
    kINPUTMUX_Cmp0OutToAdc1Trigger            = 6U + (ADC1_TRIG0_REG << PMUX_SHIFT),
    kINPUTMUX_Cmp1OutToAdc1Trigger            = 7U + (ADC1_TRIG0_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer0M0ToAdc1Trigger          = 9U + (ADC1_TRIG0_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer0M1ToAdc1Trigger          = 10U + (ADC1_TRIG0_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer1M0ToAdc1Trigger          = 11U + (ADC1_TRIG0_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer1M1ToAdc1Trigger          = 12U + (ADC1_TRIG0_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer2M0ToAdc1Trigger          = 13U + (ADC1_TRIG0_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer2M1ToAdc1Trigger          = 14U + (ADC1_TRIG0_REG << PMUX_SHIFT),
    kINPUTMUX_Lptmr0ToAdc1Trigger             = 15U + (ADC1_TRIG0_REG << PMUX_SHIFT),
    kINPUTMUX_Qdc0PosMatch0ToAdc1Trigger      = 17U + (ADC1_TRIG0_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm0OutTrig0ToAdc1Trigger    = 18U + (ADC1_TRIG0_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm0OutTrig1ToAdc1Trigger    = 19U + (ADC1_TRIG0_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm1OutTrig0ToAdc1Trigger    = 20U + (ADC1_TRIG0_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm1OutTrig1ToAdc1Trigger    = 21U + (ADC1_TRIG0_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm2OutTrig0ToAdc1Trigger    = 22U + (ADC1_TRIG0_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm2OutTrig1ToAdc1Trigger    = 23U + (ADC1_TRIG0_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio0PinEventTrig0ToAdc1Trigger = 26U + (ADC1_TRIG0_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio1PinEventTrig0ToAdc1Trigger = 27U + (ADC1_TRIG0_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio2PinEventTrig0ToAdc1Trigger = 28U + (ADC1_TRIG0_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio3PinEventTrig0ToAdc1Trigger = 29U + (ADC1_TRIG0_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio4PinEventTrig0ToAdc1Trigger = 30U + (ADC1_TRIG0_REG << PMUX_SHIFT),
    kINPUTMUX_WuuToAdc1Trigger                = 31U + (ADC1_TRIG0_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out0ToAdc1Trigger           = 33U + (ADC1_TRIG0_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out1ToAdc1Trigger           = 34U + (ADC1_TRIG0_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out2ToAdc1Trigger           = 35U + (ADC1_TRIG0_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out3ToAdc1Trigger           = 36U + (ADC1_TRIG0_REG << PMUX_SHIFT),
    kINPUTMUX_Adc0Tcomp0ToAdc1Trigger         = 37U + (ADC1_TRIG0_REG << PMUX_SHIFT),
    kINPUTMUX_Adc0Tcomp1ToAdc1Trigger         = 38U + (ADC1_TRIG0_REG << PMUX_SHIFT),
    kINPUTMUX_Adc0Tcomp2ToAdc1Trigger         = 39U + (ADC1_TRIG0_REG << PMUX_SHIFT),
    kINPUTMUX_Adc0Tcomp3ToAdc1Trigger         = 40U + (ADC1_TRIG0_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer3M0ToAdc1Trigger          = 41U + (ADC1_TRIG0_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer3M1ToAdc1Trigger          = 42U + (ADC1_TRIG0_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer4M0ToAdc1Trigger          = 43U + (ADC1_TRIG0_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer4M1ToAdc1Trigger          = 44U + (ADC1_TRIG0_REG << PMUX_SHIFT),
    kINPUTMUX_FlexioCh0ToAdc1Trigger          = 45U + (ADC1_TRIG0_REG << PMUX_SHIFT),
    kINPUTMUX_FlexioCh1ToAdc1Trigger          = 46U + (ADC1_TRIG0_REG << PMUX_SHIFT),
    kINPUTMUX_FlexioCh2ToAdc1Trigger          = 47U + (ADC1_TRIG0_REG << PMUX_SHIFT),
    kINPUTMUX_FlexioCh3ToAdc1Trigger          = 48U + (ADC1_TRIG0_REG << PMUX_SHIFT),
    kINPUTMUX_Qdc1PosMatch0ToAdc1Trigger      = 49U + (ADC1_TRIG0_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm1Sm0OutTrig0ToAdc1Trigger    = 50U + (ADC1_TRIG0_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm1Sm0OutTrig1ToAdc1Trigger    = 51U + (ADC1_TRIG0_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm1Sm1OutTrig0ToAdc1Trigger    = 52U + (ADC1_TRIG0_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm1Sm1OutTrig1ToAdc1Trigger    = 53U + (ADC1_TRIG0_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm1Sm2OutTrig0ToAdc1Trigger    = 54U + (ADC1_TRIG0_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm1Sm2OutTrig1ToAdc1Trigger    = 55U + (ADC1_TRIG0_REG << PMUX_SHIFT),

   /*!< Dac0 Trigger. */
    kINPUTMUX_ArmTxevToDac0Trigger            = 1U + (DAC0_TRIG0_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out0ToDac0Trigger           = 2U + (DAC0_TRIG0_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out1ToDac0Trigger           = 3U + (DAC0_TRIG0_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out2ToDac0Trigger           = 4U + (DAC0_TRIG0_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out3ToDac0Trigger           = 5U + (DAC0_TRIG0_REG << PMUX_SHIFT),
    kINPUTMUX_Cmp0OutToDac0Trigger            = 6U + (DAC0_TRIG0_REG << PMUX_SHIFT),
    kINPUTMUX_Cmp1OutToDac0Trigger            = 7U + (DAC0_TRIG0_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer0M0ToDac0Trigger          = 9U + (DAC0_TRIG0_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer0M1ToDac0Trigger          = 10U + (DAC0_TRIG0_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer1M0ToDac0Trigger          = 11U + (DAC0_TRIG0_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer1M1ToDac0Trigger          = 12U + (DAC0_TRIG0_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer2M0ToDac0Trigger          = 13U + (DAC0_TRIG0_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer2M1ToDac0Trigger          = 14U + (DAC0_TRIG0_REG << PMUX_SHIFT),
    kINPUTMUX_Lptmr0ToDac0Trigger             = 15U + (DAC0_TRIG0_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm0OutTrig0ToDac0Trigger    = 18U + (DAC0_TRIG0_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm0OutTrig1ToDac0Trigger    = 19U + (DAC0_TRIG0_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm1OutTrig0ToDac0Trigger    = 20U + (DAC0_TRIG0_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm1OutTrig1ToDac0Trigger    = 21U + (DAC0_TRIG0_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm2OutTrig0ToDac0Trigger    = 22U + (DAC0_TRIG0_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm2OutTrig1ToDac0Trigger    = 23U + (DAC0_TRIG0_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio0PinEventTrig0ToDac0Trigger = 26U + (DAC0_TRIG0_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio1PinEventTrig0ToDac0Trigger = 27U + (DAC0_TRIG0_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio2PinEventTrig0ToDac0Trigger = 28U + (DAC0_TRIG0_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio3PinEventTrig0ToDac0Trigger = 29U + (DAC0_TRIG0_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio4PinEventTrig0ToDac0Trigger = 30U + (DAC0_TRIG0_REG << PMUX_SHIFT),
    kINPUTMUX_WuuToDac0Trigger                = 31U + (DAC0_TRIG0_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out0ToDac0Trigger           = 33U + (DAC0_TRIG0_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out1ToDac0Trigger           = 34U + (DAC0_TRIG0_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out2ToDac0Trigger           = 35U + (DAC0_TRIG0_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out3ToDac0Trigger           = 36U + (DAC0_TRIG0_REG << PMUX_SHIFT),
    kINPUTMUX_Adc1Tcomp0ToDac0Trigger         = 37U + (DAC0_TRIG0_REG << PMUX_SHIFT),
    kINPUTMUX_Adc1Tcomp1ToDac0Trigger         = 38U + (DAC0_TRIG0_REG << PMUX_SHIFT),
    kINPUTMUX_Adc1Tcomp2ToDac0Trigger         = 39U + (DAC0_TRIG0_REG << PMUX_SHIFT),
    kINPUTMUX_Adc1Tcomp3ToDac0Trigger         = 40U + (DAC0_TRIG0_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer3M0ToDac0Trigger          = 41U + (DAC0_TRIG0_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer3M1ToDac0Trigger          = 42U + (DAC0_TRIG0_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer4M0ToDac0Trigger          = 43U + (DAC0_TRIG0_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer4M1ToDac0Trigger          = 44U + (DAC0_TRIG0_REG << PMUX_SHIFT),

    /*!< Qdc0 Trigger. */
    kINPUTMUX_ArmTxevToQdc0Trigger            = 1U + (QDC0_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out0ToQdc0Trigger           = 2U + (QDC0_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out1ToQdc0Trigger           = 3U + (QDC0_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out2ToQdc0Trigger           = 4U + (QDC0_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out3ToQdc0Trigger           = 5U + (QDC0_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Cmp0OutToQdc0Trigger            = 6U + (QDC0_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Cmp1OutToQdc0Trigger            = 7U + (QDC0_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer0M2ToQdc0Trigger          = 9U + (QDC0_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer0M3ToQdc0Trigger          = 10U + (QDC0_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer1M2ToQdc0Trigger          = 11U + (QDC0_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer1M3ToQdc0Trigger          = 12U + (QDC0_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer2M2ToQdc0Trigger          = 13U + (QDC0_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer2M3ToQdc0Trigger          = 14U + (QDC0_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm0OutTrig0ToQdc0Trigger    = 16U + (QDC0_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm0OutTrig1ToQdc0Trigger    = 17U + (QDC0_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm1OutTrig0ToQdc0Trigger    = 18U + (QDC0_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm1OutTrig1ToQdc0Trigger    = 19U + (QDC0_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm2OutTrig0ToQdc0Trigger    = 20U + (QDC0_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm2OutTrig1ToQdc0Trigger    = 21U + (QDC0_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn0ToQdc0Trigger            = 24U + (QDC0_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn1ToQdc0Trigger            = 25U + (QDC0_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn2ToQdc0Trigger            = 26U + (QDC0_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn3ToQdc0Trigger            = 27U + (QDC0_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn4ToQdc0Trigger            = 28U + (QDC0_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn5ToQdc0Trigger            = 29U + (QDC0_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn6ToQdc0Trigger            = 30U + (QDC0_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn7ToQdc0Trigger            = 31U + (QDC0_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn8ToQdc0Trigger            = 32U + (QDC0_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn9ToQdc0Trigger            = 33U + (QDC0_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn10ToQdc0Trigger           = 34U + (QDC0_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn11ToQdc0Trigger           = 35U + (QDC0_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio0PinEventTrig0ToQdc0Trigger = 36U + (QDC0_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio1PinEventTrig0ToQdc0Trigger = 37U + (QDC0_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio2PinEventTrig0ToQdc0Trigger = 38U + (QDC0_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio3PinEventTrig0ToQdc0Trigger = 39U + (QDC0_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio4PinEventTrig0ToQdc0Trigger = 40U + (QDC0_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out0ToQdc0Trigger           = 41U + (QDC0_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out1ToQdc0Trigger           = 42U + (QDC0_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out2ToQdc0Trigger           = 43U + (QDC0_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out3ToQdc0Trigger           = 44U + (QDC0_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer3M2ToQdc0Trigger          = 49U + (QDC0_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer3M3ToQdc0Trigger          = 50U + (QDC0_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer4M2ToQdc0Trigger          = 51U + (QDC0_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer4M3ToQdc0Trigger          = 52U + (QDC0_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm1Sm0OutTrig0ToQdc0Trigger    = 62U + (QDC0_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm1Sm0OutTrig1ToQdc0Trigger    = 63U + (QDC0_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm1Sm1OutTrig0ToQdc0Trigger    = 64U + (QDC0_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm1Sm1OutTrig1ToQdc0Trigger    = 65U + (QDC0_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm1Sm2OutTrig0ToQdc0Trigger    = 66U + (QDC0_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm1Sm2OutTrig1ToQdc0Trigger    = 67U + (QDC0_TRIG_REG << PMUX_SHIFT),

    /*!< Qdc0 Home. */
    kINPUTMUX_ArmTxevToQdc0Home            = 1U + (QDC0_HOME_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out0ToQdc0Home           = 2U + (QDC0_HOME_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out1ToQdc0Home           = 3U + (QDC0_HOME_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out2ToQdc0Home           = 4U + (QDC0_HOME_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out3ToQdc0Home           = 5U + (QDC0_HOME_REG << PMUX_SHIFT),
    kINPUTMUX_Cmp0OutToQdc0Home            = 6U + (QDC0_HOME_REG << PMUX_SHIFT),
    kINPUTMUX_Cmp1OutToQdc0Home            = 7U + (QDC0_HOME_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer0M2ToQdc0Home          = 9U + (QDC0_HOME_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer0M3ToQdc0Home          = 10U + (QDC0_HOME_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer1M2ToQdc0Home          = 11U + (QDC0_HOME_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer1M3ToQdc0Home          = 12U + (QDC0_HOME_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer2M2ToQdc0Home          = 13U + (QDC0_HOME_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer2M3ToQdc0Home          = 14U + (QDC0_HOME_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm0OutTrig0ToQdc0Home    = 16U + (QDC0_HOME_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm0OutTrig1ToQdc0Home    = 17U + (QDC0_HOME_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm1OutTrig0ToQdc0Home    = 18U + (QDC0_HOME_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm1OutTrig1ToQdc0Home    = 19U + (QDC0_HOME_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm2OutTrig0ToQdc0Home    = 20U + (QDC0_HOME_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm2OutTrig1ToQdc0Home    = 21U + (QDC0_HOME_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn0ToQdc0Home            = 24U + (QDC0_HOME_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn1ToQdc0Home            = 25U + (QDC0_HOME_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn2ToQdc0Home            = 26U + (QDC0_HOME_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn3ToQdc0Home            = 27U + (QDC0_HOME_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn4ToQdc0Home            = 28U + (QDC0_HOME_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn5ToQdc0Home            = 29U + (QDC0_HOME_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn6ToQdc0Home            = 30U + (QDC0_HOME_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn7ToQdc0Home            = 31U + (QDC0_HOME_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn8ToQdc0Home            = 32U + (QDC0_HOME_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn9ToQdc0Home            = 33U + (QDC0_HOME_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn10ToQdc0Home           = 34U + (QDC0_HOME_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn11ToQdc0Home           = 35U + (QDC0_HOME_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio0PinEventTrig0ToQdc0Home = 36U + (QDC0_HOME_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio1PinEventTrig0ToQdc0Home = 37U + (QDC0_HOME_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio2PinEventTrig0ToQdc0Home = 38U + (QDC0_HOME_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio3PinEventTrig0ToQdc0Home = 39U + (QDC0_HOME_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio4PinEventTrig0ToQdc0Home = 40U + (QDC0_HOME_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out0ToQdc0Home           = 41U + (QDC0_HOME_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out1ToQdc0Home           = 42U + (QDC0_HOME_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out2ToQdc0Home           = 43U + (QDC0_HOME_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out3ToQdc0Home           = 44U + (QDC0_HOME_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer3M2ToQdc0Home          = 49U + (QDC0_HOME_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer3M3ToQdc0Home          = 50U + (QDC0_HOME_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer4M2ToQdc0Home          = 51U + (QDC0_HOME_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer4M3ToQdc0Home          = 52U + (QDC0_HOME_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm1Sm0OutTrig0ToQdc0Home    = 62U + (QDC0_HOME_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm1Sm0OutTrig1ToQdc0Home    = 63U + (QDC0_HOME_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm1Sm1OutTrig0ToQdc0Home    = 64U + (QDC0_HOME_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm1Sm1OutTrig1ToQdc0Home    = 65U + (QDC0_HOME_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm1Sm2OutTrig0ToQdc0Home    = 66U + (QDC0_HOME_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm1Sm2OutTrig1ToQdc0Home    = 67U + (QDC0_HOME_REG << PMUX_SHIFT),

    /*!< Qdc0 Index. */
    kINPUTMUX_ArmTxevToQdc0Index            = 1U + (QDC0_INDEX_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out0ToQdc0Index           = 2U + (QDC0_INDEX_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out1ToQdc0Index           = 3U + (QDC0_INDEX_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out2ToQdc0Index           = 4U + (QDC0_INDEX_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out3ToQdc0Index           = 5U + (QDC0_INDEX_REG << PMUX_SHIFT),
    kINPUTMUX_Cmp0OutToQdc0Index            = 6U + (QDC0_INDEX_REG << PMUX_SHIFT),
    kINPUTMUX_Cmp1OutToQdc0Index            = 7U + (QDC0_INDEX_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer0M2ToQdc0Index          = 9U + (QDC0_INDEX_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer0M3ToQdc0Index          = 10U + (QDC0_INDEX_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer1M2ToQdc0Index          = 11U + (QDC0_INDEX_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer1M3ToQdc0Index          = 12U + (QDC0_INDEX_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer2M2ToQdc0Index          = 13U + (QDC0_INDEX_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer2M3ToQdc0Index          = 14U + (QDC0_INDEX_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm0OutTrig0ToQdc0Index    = 16U + (QDC0_INDEX_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm0OutTrig1ToQdc0Index    = 17U + (QDC0_INDEX_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm1OutTrig0ToQdc0Index    = 18U + (QDC0_INDEX_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm1OutTrig1ToQdc0Index    = 19U + (QDC0_INDEX_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm2OutTrig0ToQdc0Index    = 20U + (QDC0_INDEX_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm2OutTrig1ToQdc0Index    = 21U + (QDC0_INDEX_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn0ToQdc0Index            = 24U + (QDC0_INDEX_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn1ToQdc0Index            = 25U + (QDC0_INDEX_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn2ToQdc0Index            = 26U + (QDC0_INDEX_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn3ToQdc0Index            = 27U + (QDC0_INDEX_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn4ToQdc0Index            = 28U + (QDC0_INDEX_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn5ToQdc0Index            = 29U + (QDC0_INDEX_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn6ToQdc0Index            = 30U + (QDC0_INDEX_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn7ToQdc0Index            = 31U + (QDC0_INDEX_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn8ToQdc0Index            = 32U + (QDC0_INDEX_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn9ToQdc0Index            = 33U + (QDC0_INDEX_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn10ToQdc0Index           = 34U + (QDC0_INDEX_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn11ToQdc0Index           = 35U + (QDC0_INDEX_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio0PinEventTrig0ToQdc0Index = 36U + (QDC0_INDEX_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio1PinEventTrig0ToQdc0Index = 37U + (QDC0_INDEX_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio2PinEventTrig0ToQdc0Index = 38U + (QDC0_INDEX_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio3PinEventTrig0ToQdc0Index = 39U + (QDC0_INDEX_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio4PinEventTrig0ToQdc0Index = 40U + (QDC0_INDEX_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out0ToQdc0Index           = 41U + (QDC0_INDEX_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out1ToQdc0Index           = 42U + (QDC0_INDEX_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out2ToQdc0Index           = 43U + (QDC0_INDEX_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out3ToQdc0Index           = 44U + (QDC0_INDEX_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer3M2ToQdc0Index          = 49U + (QDC0_INDEX_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer3M3ToQdc0Index          = 50U + (QDC0_INDEX_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer4M2ToQdc0Index          = 51U + (QDC0_INDEX_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer4M3ToQdc0Index          = 52U + (QDC0_INDEX_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm1Sm0OutTrig0ToQdc0Index    = 62U + (QDC0_INDEX_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm1Sm0OutTrig1ToQdc0Index    = 63U + (QDC0_INDEX_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm1Sm1OutTrig0ToQdc0Index    = 64U + (QDC0_INDEX_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm1Sm1OutTrig1ToQdc0Index    = 65U + (QDC0_INDEX_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm1Sm2OutTrig0ToQdc0Index    = 66U + (QDC0_INDEX_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm1Sm2OutTrig1ToQdc0Index    = 67U + (QDC0_INDEX_REG << PMUX_SHIFT),

    /*!< Qdc0 Phaseb. */
    kINPUTMUX_ArmTxevToQdc0Phaseb            = 1U + (QDC0_PHASEB_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out0ToQdc0Phaseb           = 2U + (QDC0_PHASEB_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out1ToQdc0Phaseb           = 3U + (QDC0_PHASEB_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out2ToQdc0Phaseb           = 4U + (QDC0_PHASEB_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out3ToQdc0Phaseb           = 5U + (QDC0_PHASEB_REG << PMUX_SHIFT),
    kINPUTMUX_Cmp0OutToQdc0Phaseb            = 6U + (QDC0_PHASEB_REG << PMUX_SHIFT),
    kINPUTMUX_Cmp1OutToQdc0Phaseb            = 7U + (QDC0_PHASEB_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer0M2ToQdc0Phaseb          = 9U + (QDC0_PHASEB_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer0M3ToQdc0Phaseb          = 10U + (QDC0_PHASEB_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer1M2ToQdc0Phaseb          = 11U + (QDC0_PHASEB_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer1M3ToQdc0Phaseb          = 12U + (QDC0_PHASEB_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer2M2ToQdc0Phaseb          = 13U + (QDC0_PHASEB_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer2M3ToQdc0Phaseb          = 14U + (QDC0_PHASEB_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm0OutTrig0ToQdc0Phaseb    = 16U + (QDC0_PHASEB_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm0OutTrig1ToQdc0Phaseb    = 17U + (QDC0_PHASEB_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm1OutTrig0ToQdc0Phaseb    = 18U + (QDC0_PHASEB_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm1OutTrig1ToQdc0Phaseb    = 19U + (QDC0_PHASEB_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm2OutTrig0ToQdc0Phaseb    = 20U + (QDC0_PHASEB_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm2OutTrig1ToQdc0Phaseb    = 21U + (QDC0_PHASEB_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn0ToQdc0Phaseb            = 24U + (QDC0_PHASEB_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn1ToQdc0Phaseb            = 25U + (QDC0_PHASEB_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn2ToQdc0Phaseb            = 26U + (QDC0_PHASEB_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn3ToQdc0Phaseb            = 27U + (QDC0_PHASEB_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn4ToQdc0Phaseb            = 28U + (QDC0_PHASEB_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn5ToQdc0Phaseb            = 29U + (QDC0_PHASEB_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn6ToQdc0Phaseb            = 30U + (QDC0_PHASEB_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn7ToQdc0Phaseb            = 31U + (QDC0_PHASEB_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn8ToQdc0Phaseb            = 32U + (QDC0_PHASEB_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn9ToQdc0Phaseb            = 33U + (QDC0_PHASEB_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn10ToQdc0Phaseb           = 34U + (QDC0_PHASEB_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn11ToQdc0Phaseb           = 35U + (QDC0_PHASEB_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio0PinEventTrig0ToQdc0Phaseb = 36U + (QDC0_PHASEB_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio1PinEventTrig0ToQdc0Phaseb = 37U + (QDC0_PHASEB_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio2PinEventTrig0ToQdc0Phaseb = 38U + (QDC0_PHASEB_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio3PinEventTrig0ToQdc0Phaseb = 39U + (QDC0_PHASEB_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio4PinEventTrig0ToQdc0Phaseb = 40U + (QDC0_PHASEB_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out0ToQdc0Phaseb           = 41U + (QDC0_PHASEB_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out1ToQdc0Phaseb           = 42U + (QDC0_PHASEB_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out2ToQdc0Phaseb           = 43U + (QDC0_PHASEB_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out3ToQdc0Phaseb           = 44U + (QDC0_PHASEB_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer3M2ToQdc0Phaseb          = 49U + (QDC0_PHASEB_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer3M3ToQdc0Phaseb          = 50U + (QDC0_PHASEB_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer4M2ToQdc0Phaseb          = 51U + (QDC0_PHASEB_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer4M3ToQdc0Phaseb          = 52U + (QDC0_PHASEB_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm1Sm0OutTrig0ToQdc0Phaseb    = 62U + (QDC0_PHASEB_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm1Sm0OutTrig1ToQdc0Phaseb    = 63U + (QDC0_PHASEB_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm1Sm1OutTrig0ToQdc0Phaseb    = 64U + (QDC0_PHASEB_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm1Sm1OutTrig1ToQdc0Phaseb    = 65U + (QDC0_PHASEB_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm1Sm2OutTrig0ToQdc0Phaseb    = 66U + (QDC0_PHASEB_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm1Sm2OutTrig1ToQdc0Phaseb    = 67U + (QDC0_PHASEB_REG << PMUX_SHIFT),

    /*!< Qdc0 Phasea. */
    kINPUTMUX_ArmTxevToQdc0Phasea            = 1U + (QDC0_PHASEA_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out0ToQdc0Phasea           = 2U + (QDC0_PHASEA_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out1ToQdc0Phasea           = 3U + (QDC0_PHASEA_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out2ToQdc0Phasea           = 4U + (QDC0_PHASEA_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out3ToQdc0Phasea           = 5U + (QDC0_PHASEA_REG << PMUX_SHIFT),
    kINPUTMUX_Cmp0OutToQdc0Phasea            = 6U + (QDC0_PHASEA_REG << PMUX_SHIFT),
    kINPUTMUX_Cmp1OutToQdc0Phasea            = 7U + (QDC0_PHASEA_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer0M2ToQdc0Phasea          = 9U + (QDC0_PHASEA_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer0M3ToQdc0Phasea          = 10U + (QDC0_PHASEA_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer1M2ToQdc0Phasea          = 11U + (QDC0_PHASEA_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer1M3ToQdc0Phasea          = 12U + (QDC0_PHASEA_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer2M2ToQdc0Phasea          = 13U + (QDC0_PHASEA_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer2M3ToQdc0Phasea          = 14U + (QDC0_PHASEA_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm0OutTrig0ToQdc0Phasea    = 16U + (QDC0_PHASEA_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm0OutTrig1ToQdc0Phasea    = 17U + (QDC0_PHASEA_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm1OutTrig0ToQdc0Phasea    = 18U + (QDC0_PHASEA_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm1OutTrig1ToQdc0Phasea    = 19U + (QDC0_PHASEA_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm2OutTrig0ToQdc0Phasea    = 20U + (QDC0_PHASEA_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm2OutTrig1ToQdc0Phasea    = 21U + (QDC0_PHASEA_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn0ToQdc0Phasea            = 24U + (QDC0_PHASEA_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn1ToQdc0Phasea            = 25U + (QDC0_PHASEA_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn2ToQdc0Phasea            = 26U + (QDC0_PHASEA_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn3ToQdc0Phasea            = 27U + (QDC0_PHASEA_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn4ToQdc0Phasea            = 28U + (QDC0_PHASEA_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn5ToQdc0Phasea            = 29U + (QDC0_PHASEA_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn6ToQdc0Phasea            = 30U + (QDC0_PHASEA_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn7ToQdc0Phasea            = 31U + (QDC0_PHASEA_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn8ToQdc0Phasea            = 32U + (QDC0_PHASEA_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn9ToQdc0Phasea            = 33U + (QDC0_PHASEA_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn10ToQdc0Phasea           = 34U + (QDC0_PHASEA_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn11ToQdc0Phasea           = 35U + (QDC0_PHASEA_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio0PinEventTrig0ToQdc0Phasea = 36U + (QDC0_PHASEA_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio1PinEventTrig0ToQdc0Phasea = 37U + (QDC0_PHASEA_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio2PinEventTrig0ToQdc0Phasea = 38U + (QDC0_PHASEA_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio3PinEventTrig0ToQdc0Phasea = 39U + (QDC0_PHASEA_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio4PinEventTrig0ToQdc0Phasea = 40U + (QDC0_PHASEA_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out0ToQdc0Phasea           = 41U + (QDC0_PHASEA_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out1ToQdc0Phasea           = 42U + (QDC0_PHASEA_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out2ToQdc0Phasea           = 43U + (QDC0_PHASEA_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out3ToQdc0Phasea           = 44U + (QDC0_PHASEA_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer3M2ToQdc0Phasea          = 49U + (QDC0_PHASEA_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer3M3ToQdc0Phasea          = 50U + (QDC0_PHASEA_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer4M2ToQdc0Phasea          = 51U + (QDC0_PHASEA_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer4M3ToQdc0Phasea          = 52U + (QDC0_PHASEA_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm1Sm0OutTrig0ToQdc0Phasea    = 62U + (QDC0_PHASEA_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm1Sm0OutTrig1ToQdc0Phasea    = 63U + (QDC0_PHASEA_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm1Sm1OutTrig0ToQdc0Phasea    = 64U + (QDC0_PHASEA_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm1Sm1OutTrig1ToQdc0Phasea    = 65U + (QDC0_PHASEA_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm1Sm2OutTrig0ToQdc0Phasea    = 66U + (QDC0_PHASEA_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm1Sm2OutTrig1ToQdc0Phasea    = 67U + (QDC0_PHASEA_REG << PMUX_SHIFT),

    /*!< Qdc0 Icap1. */
    kINPUTMUX_ArmTxevToQdc0Icap1            = 1U + (QDC0_ICAP1_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out0ToQdc0Icap1           = 2U + (QDC0_ICAP1_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out1ToQdc0Icap1           = 3U + (QDC0_ICAP1_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out2ToQdc0Icap1           = 4U + (QDC0_ICAP1_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out3ToQdc0Icap1           = 5U + (QDC0_ICAP1_REG << PMUX_SHIFT),
    kINPUTMUX_Cmp0OutToQdc0Icap1            = 6U + (QDC0_ICAP1_REG << PMUX_SHIFT),
    kINPUTMUX_Cmp1OutToQdc0Icap1            = 7U + (QDC0_ICAP1_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer0M2ToQdc0Icap1          = 9U + (QDC0_ICAP1_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer0M3ToQdc0Icap1          = 10U + (QDC0_ICAP1_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer1M2ToQdc0Icap1          = 11U + (QDC0_ICAP1_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer1M3ToQdc0Icap1          = 12U + (QDC0_ICAP1_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer2M2ToQdc0Icap1          = 13U + (QDC0_ICAP1_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer2M3ToQdc0Icap1          = 14U + (QDC0_ICAP1_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm0OutTrig0ToQdc0Icap1    = 16U + (QDC0_ICAP1_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm0OutTrig1ToQdc0Icap1    = 17U + (QDC0_ICAP1_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm1OutTrig0ToQdc0Icap1    = 18U + (QDC0_ICAP1_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm1OutTrig1ToQdc0Icap1    = 19U + (QDC0_ICAP1_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm2OutTrig0ToQdc0Icap1    = 20U + (QDC0_ICAP1_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm2OutTrig1ToQdc0Icap1    = 21U + (QDC0_ICAP1_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn0ToQdc0Icap1            = 24U + (QDC0_ICAP1_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn1ToQdc0Icap1            = 25U + (QDC0_ICAP1_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn2ToQdc0Icap1            = 26U + (QDC0_ICAP1_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn3ToQdc0Icap1            = 27U + (QDC0_ICAP1_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn4ToQdc0Icap1            = 28U + (QDC0_ICAP1_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn5ToQdc0Icap1            = 29U + (QDC0_ICAP1_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn6ToQdc0Icap1            = 30U + (QDC0_ICAP1_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn7ToQdc0Icap1            = 31U + (QDC0_ICAP1_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn8ToQdc0Icap1            = 32U + (QDC0_ICAP1_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn9ToQdc0Icap1            = 33U + (QDC0_ICAP1_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn10ToQdc0Icap1           = 34U + (QDC0_ICAP1_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn11ToQdc0Icap1           = 35U + (QDC0_ICAP1_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio0PinEventTrig0ToQdc0Icap1 = 36U + (QDC0_ICAP1_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio1PinEventTrig0ToQdc0Icap1 = 37U + (QDC0_ICAP1_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio2PinEventTrig0ToQdc0Icap1 = 38U + (QDC0_ICAP1_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio3PinEventTrig0ToQdc0Icap1 = 39U + (QDC0_ICAP1_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio4PinEventTrig0ToQdc0Icap1 = 40U + (QDC0_ICAP1_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out0ToQdc0Icap1           = 41U + (QDC0_ICAP1_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out1ToQdc0Icap1           = 42U + (QDC0_ICAP1_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out2ToQdc0Icap1           = 43U + (QDC0_ICAP1_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out3ToQdc0Icap1           = 44U + (QDC0_ICAP1_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer3M2ToQdc0Icap1          = 49U + (QDC0_ICAP1_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer3M3ToQdc0Icap1          = 50U + (QDC0_ICAP1_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer4M2ToQdc0Icap1          = 51U + (QDC0_ICAP1_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer4M3ToQdc0Icap1          = 52U + (QDC0_ICAP1_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm1Sm0OutTrig0ToQdc0Icap1    = 62U + (QDC0_ICAP1_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm1Sm0OutTrig1ToQdc0Icap1    = 63U + (QDC0_ICAP1_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm1Sm1OutTrig0ToQdc0Icap1    = 64U + (QDC0_ICAP1_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm1Sm1OutTrig1ToQdc0Icap1    = 65U + (QDC0_ICAP1_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm1Sm2OutTrig0ToQdc0Icap1    = 66U + (QDC0_ICAP1_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm1Sm2OutTrig1ToQdc0Icap1    = 67U + (QDC0_ICAP1_REG << PMUX_SHIFT),

    /*!< Qdc1 Trigger. */
    kINPUTMUX_ArmTxevToQdc1Trigger            = 1U + (QDC1_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out0ToQdc1Trigger           = 2U + (QDC1_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out1ToQdc1Trigger           = 3U + (QDC1_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out2ToQdc1Trigger           = 4U + (QDC1_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out3ToQdc1Trigger           = 5U + (QDC1_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Cmp0OutToQdc1Trigger            = 6U + (QDC1_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Cmp1OutToQdc1Trigger            = 7U + (QDC1_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer0M2ToQdc1Trigger          = 9U + (QDC1_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer0M3ToQdc1Trigger          = 10U + (QDC1_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer1M2ToQdc1Trigger          = 11U + (QDC1_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer1M3ToQdc1Trigger          = 12U + (QDC1_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer2M2ToQdc1Trigger          = 13U + (QDC1_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer2M3ToQdc1Trigger          = 14U + (QDC1_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm0OutTrig0ToQdc1Trigger    = 16U + (QDC1_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm0OutTrig1ToQdc1Trigger    = 17U + (QDC1_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm1OutTrig0ToQdc1Trigger    = 18U + (QDC1_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm1OutTrig1ToQdc1Trigger    = 19U + (QDC1_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm2OutTrig0ToQdc1Trigger    = 20U + (QDC1_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm2OutTrig1ToQdc1Trigger    = 21U + (QDC1_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn0ToQdc1Trigger            = 24U + (QDC1_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn1ToQdc1Trigger            = 25U + (QDC1_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn2ToQdc1Trigger            = 26U + (QDC1_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn3ToQdc1Trigger            = 27U + (QDC1_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn4ToQdc1Trigger            = 28U + (QDC1_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn5ToQdc1Trigger            = 29U + (QDC1_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn6ToQdc1Trigger            = 30U + (QDC1_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn7ToQdc1Trigger            = 31U + (QDC1_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn8ToQdc1Trigger            = 32U + (QDC1_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn9ToQdc1Trigger            = 33U + (QDC1_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn10ToQdc1Trigger           = 34U + (QDC1_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn11ToQdc1Trigger           = 35U + (QDC1_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio0PinEventTrig0ToQdc1Trigger = 36U + (QDC1_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio1PinEventTrig0ToQdc1Trigger = 37U + (QDC1_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio2PinEventTrig0ToQdc1Trigger = 38U + (QDC1_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio3PinEventTrig0ToQdc1Trigger = 39U + (QDC1_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio4PinEventTrig0ToQdc1Trigger = 40U + (QDC1_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out0ToQdc1Trigger           = 41U + (QDC1_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out1ToQdc1Trigger           = 42U + (QDC1_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out2ToQdc1Trigger           = 43U + (QDC1_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out3ToQdc1Trigger           = 44U + (QDC1_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer3M2ToQdc1Trigger          = 49U + (QDC1_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer3M3ToQdc1Trigger          = 50U + (QDC1_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer4M2ToQdc1Trigger          = 51U + (QDC1_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer4M3ToQdc1Trigger          = 52U + (QDC1_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm1Sm0OutTrig0ToQdc1Trigger    = 62U + (QDC1_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm1Sm0OutTrig1ToQdc1Trigger    = 63U + (QDC1_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm1Sm1OutTrig0ToQdc1Trigger    = 64U + (QDC1_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm1Sm1OutTrig1ToQdc1Trigger    = 65U + (QDC1_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm1Sm2OutTrig0ToQdc1Trigger    = 66U + (QDC1_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm1Sm2OutTrig1ToQdc1Trigger    = 67U + (QDC1_TRIG_REG << PMUX_SHIFT),

    /*!< Qdc1 Home. */
    kINPUTMUX_ArmTxevToQdc1Home            = 1U + (QDC1_HOME_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out0ToQdc1Home           = 2U + (QDC1_HOME_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out1ToQdc1Home           = 3U + (QDC1_HOME_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out2ToQdc1Home           = 4U + (QDC1_HOME_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out3ToQdc1Home           = 5U + (QDC1_HOME_REG << PMUX_SHIFT),
    kINPUTMUX_Cmp0OutToQdc1Home            = 6U + (QDC1_HOME_REG << PMUX_SHIFT),
    kINPUTMUX_Cmp1OutToQdc1Home            = 7U + (QDC1_HOME_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer0M2ToQdc1Home          = 9U + (QDC1_HOME_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer0M3ToQdc1Home          = 10U + (QDC1_HOME_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer1M2ToQdc1Home          = 11U + (QDC1_HOME_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer1M3ToQdc1Home          = 12U + (QDC1_HOME_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer2M2ToQdc1Home          = 13U + (QDC1_HOME_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer2M3ToQdc1Home          = 14U + (QDC1_HOME_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm0OutTrig0ToQdc1Home    = 16U + (QDC1_HOME_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm0OutTrig1ToQdc1Home    = 17U + (QDC1_HOME_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm1OutTrig0ToQdc1Home    = 18U + (QDC1_HOME_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm1OutTrig1ToQdc1Home    = 19U + (QDC1_HOME_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm2OutTrig0ToQdc1Home    = 20U + (QDC1_HOME_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm2OutTrig1ToQdc1Home    = 21U + (QDC1_HOME_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn0ToQdc1Home            = 24U + (QDC1_HOME_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn1ToQdc1Home            = 25U + (QDC1_HOME_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn2ToQdc1Home            = 26U + (QDC1_HOME_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn3ToQdc1Home            = 27U + (QDC1_HOME_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn4ToQdc1Home            = 28U + (QDC1_HOME_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn5ToQdc1Home            = 29U + (QDC1_HOME_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn6ToQdc1Home            = 30U + (QDC1_HOME_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn7ToQdc1Home            = 31U + (QDC1_HOME_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn8ToQdc1Home            = 32U + (QDC1_HOME_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn9ToQdc1Home            = 33U + (QDC1_HOME_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn10ToQdc1Home           = 34U + (QDC1_HOME_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn11ToQdc1Home           = 35U + (QDC1_HOME_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio0PinEventTrig0ToQdc1Home = 36U + (QDC1_HOME_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio1PinEventTrig0ToQdc1Home = 37U + (QDC1_HOME_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio2PinEventTrig0ToQdc1Home = 38U + (QDC1_HOME_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio3PinEventTrig0ToQdc1Home = 39U + (QDC1_HOME_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio4PinEventTrig0ToQdc1Home = 40U + (QDC1_HOME_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out0ToQdc1Home           = 41U + (QDC1_HOME_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out1ToQdc1Home           = 42U + (QDC1_HOME_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out2ToQdc1Home           = 43U + (QDC1_HOME_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out3ToQdc1Home           = 44U + (QDC1_HOME_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer3M2ToQdc1Home          = 49U + (QDC1_HOME_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer3M3ToQdc1Home          = 50U + (QDC1_HOME_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer4M2ToQdc1Home          = 51U + (QDC1_HOME_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer4M3ToQdc1Home          = 52U + (QDC1_HOME_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm1Sm0OutTrig0ToQdc1Home    = 62U + (QDC1_HOME_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm1Sm0OutTrig1ToQdc1Home    = 63U + (QDC1_HOME_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm1Sm1OutTrig0ToQdc1Home    = 64U + (QDC1_HOME_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm1Sm1OutTrig1ToQdc1Home    = 65U + (QDC1_HOME_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm1Sm2OutTrig0ToQdc1Home    = 66U + (QDC1_HOME_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm1Sm2OutTrig1ToQdc1Home    = 67U + (QDC1_HOME_REG << PMUX_SHIFT),

    /*!< Qdc1 Index. */
    kINPUTMUX_ArmTxevToQdc1Index            = 1U + (QDC1_INDEX_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out0ToQdc1Index           = 2U + (QDC1_INDEX_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out1ToQdc1Index           = 3U + (QDC1_INDEX_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out2ToQdc1Index           = 4U + (QDC1_INDEX_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out3ToQdc1Index           = 5U + (QDC1_INDEX_REG << PMUX_SHIFT),
    kINPUTMUX_Cmp0OutToQdc1Index            = 6U + (QDC1_INDEX_REG << PMUX_SHIFT),
    kINPUTMUX_Cmp1OutToQdc1Index            = 7U + (QDC1_INDEX_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer0M2ToQdc1Index          = 9U + (QDC1_INDEX_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer0M3ToQdc1Index          = 10U + (QDC1_INDEX_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer1M2ToQdc1Index          = 11U + (QDC1_INDEX_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer1M3ToQdc1Index          = 12U + (QDC1_INDEX_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer2M2ToQdc1Index          = 13U + (QDC1_INDEX_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer2M3ToQdc1Index          = 14U + (QDC1_INDEX_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm0OutTrig0ToQdc1Index    = 16U + (QDC1_INDEX_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm0OutTrig1ToQdc1Index    = 17U + (QDC1_INDEX_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm1OutTrig0ToQdc1Index    = 18U + (QDC1_INDEX_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm1OutTrig1ToQdc1Index    = 19U + (QDC1_INDEX_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm2OutTrig0ToQdc1Index    = 20U + (QDC1_INDEX_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm2OutTrig1ToQdc1Index    = 21U + (QDC1_INDEX_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn0ToQdc1Index            = 24U + (QDC1_INDEX_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn1ToQdc1Index            = 25U + (QDC1_INDEX_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn2ToQdc1Index            = 26U + (QDC1_INDEX_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn3ToQdc1Index            = 27U + (QDC1_INDEX_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn4ToQdc1Index            = 28U + (QDC1_INDEX_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn5ToQdc1Index            = 29U + (QDC1_INDEX_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn6ToQdc1Index            = 30U + (QDC1_INDEX_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn7ToQdc1Index            = 31U + (QDC1_INDEX_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn8ToQdc1Index            = 32U + (QDC1_INDEX_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn9ToQdc1Index            = 33U + (QDC1_INDEX_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn10ToQdc1Index           = 34U + (QDC1_INDEX_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn11ToQdc1Index           = 35U + (QDC1_INDEX_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio0PinEventTrig0ToQdc1Index = 36U + (QDC1_INDEX_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio1PinEventTrig0ToQdc1Index = 37U + (QDC1_INDEX_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio2PinEventTrig0ToQdc1Index = 38U + (QDC1_INDEX_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio3PinEventTrig0ToQdc1Index = 39U + (QDC1_INDEX_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio4PinEventTrig0ToQdc1Index = 40U + (QDC1_INDEX_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out0ToQdc1Index           = 41U + (QDC1_INDEX_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out1ToQdc1Index           = 42U + (QDC1_INDEX_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out2ToQdc1Index           = 43U + (QDC1_INDEX_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out3ToQdc1Index           = 44U + (QDC1_INDEX_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer3M2ToQdc1Index          = 49U + (QDC1_INDEX_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer3M3ToQdc1Index          = 50U + (QDC1_INDEX_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer4M2ToQdc1Index          = 51U + (QDC1_INDEX_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer4M3ToQdc1Index          = 52U + (QDC1_INDEX_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm1Sm0OutTrig0ToQdc1Index    = 62U + (QDC1_INDEX_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm1Sm0OutTrig1ToQdc1Index    = 63U + (QDC1_INDEX_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm1Sm1OutTrig0ToQdc1Index    = 64U + (QDC1_INDEX_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm1Sm1OutTrig1ToQdc1Index    = 65U + (QDC1_INDEX_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm1Sm2OutTrig0ToQdc1Index    = 66U + (QDC1_INDEX_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm1Sm2OutTrig1ToQdc1Index    = 67U + (QDC1_INDEX_REG << PMUX_SHIFT),

    /*!< Qdc1 Phaseb. */
    kINPUTMUX_ArmTxevToQdc1Phaseb            = 1U + (QDC1_PHASEB_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out0ToQdc1Phaseb           = 2U + (QDC1_PHASEB_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out1ToQdc1Phaseb           = 3U + (QDC1_PHASEB_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out2ToQdc1Phaseb           = 4U + (QDC1_PHASEB_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out3ToQdc1Phaseb           = 5U + (QDC1_PHASEB_REG << PMUX_SHIFT),
    kINPUTMUX_Cmp0OutToQdc1Phaseb            = 6U + (QDC1_PHASEB_REG << PMUX_SHIFT),
    kINPUTMUX_Cmp1OutToQdc1Phaseb            = 7U + (QDC1_PHASEB_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer0M2ToQdc1Phaseb          = 9U + (QDC1_PHASEB_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer0M3ToQdc1Phaseb          = 10U + (QDC1_PHASEB_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer1M2ToQdc1Phaseb          = 11U + (QDC1_PHASEB_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer1M3ToQdc1Phaseb          = 12U + (QDC1_PHASEB_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer2M2ToQdc1Phaseb          = 13U + (QDC1_PHASEB_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer2M3ToQdc1Phaseb          = 14U + (QDC1_PHASEB_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm0OutTrig0ToQdc1Phaseb    = 16U + (QDC1_PHASEB_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm0OutTrig1ToQdc1Phaseb    = 17U + (QDC1_PHASEB_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm1OutTrig0ToQdc1Phaseb    = 18U + (QDC1_PHASEB_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm1OutTrig1ToQdc1Phaseb    = 19U + (QDC1_PHASEB_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm2OutTrig0ToQdc1Phaseb    = 20U + (QDC1_PHASEB_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm2OutTrig1ToQdc1Phaseb    = 21U + (QDC1_PHASEB_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn0ToQdc1Phaseb            = 24U + (QDC1_PHASEB_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn1ToQdc1Phaseb            = 25U + (QDC1_PHASEB_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn2ToQdc1Phaseb            = 26U + (QDC1_PHASEB_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn3ToQdc1Phaseb            = 27U + (QDC1_PHASEB_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn4ToQdc1Phaseb            = 28U + (QDC1_PHASEB_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn5ToQdc1Phaseb            = 29U + (QDC1_PHASEB_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn6ToQdc1Phaseb            = 30U + (QDC1_PHASEB_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn7ToQdc1Phaseb            = 31U + (QDC1_PHASEB_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn8ToQdc1Phaseb            = 32U + (QDC1_PHASEB_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn9ToQdc1Phaseb            = 33U + (QDC1_PHASEB_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn10ToQdc1Phaseb           = 34U + (QDC1_PHASEB_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn11ToQdc1Phaseb           = 35U + (QDC1_PHASEB_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio0PinEventTrig0ToQdc1Phaseb = 36U + (QDC1_PHASEB_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio1PinEventTrig0ToQdc1Phaseb = 37U + (QDC1_PHASEB_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio2PinEventTrig0ToQdc1Phaseb = 38U + (QDC1_PHASEB_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio3PinEventTrig0ToQdc1Phaseb = 39U + (QDC1_PHASEB_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio4PinEventTrig0ToQdc1Phaseb = 40U + (QDC1_PHASEB_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out0ToQdc1Phaseb           = 41U + (QDC1_PHASEB_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out1ToQdc1Phaseb           = 42U + (QDC1_PHASEB_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out2ToQdc1Phaseb           = 43U + (QDC1_PHASEB_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out3ToQdc1Phaseb           = 44U + (QDC1_PHASEB_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer3M2ToQdc1Phaseb          = 49U + (QDC1_PHASEB_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer3M3ToQdc1Phaseb          = 50U + (QDC1_PHASEB_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer4M2ToQdc1Phaseb          = 51U + (QDC1_PHASEB_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer4M3ToQdc1Phaseb          = 52U + (QDC1_PHASEB_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm1Sm0OutTrig0ToQdc1Phaseb    = 62U + (QDC1_PHASEB_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm1Sm0OutTrig1ToQdc1Phaseb    = 63U + (QDC1_PHASEB_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm1Sm1OutTrig0ToQdc1Phaseb    = 64U + (QDC1_PHASEB_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm1Sm1OutTrig1ToQdc1Phaseb    = 65U + (QDC1_PHASEB_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm1Sm2OutTrig0ToQdc1Phaseb    = 66U + (QDC1_PHASEB_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm1Sm2OutTrig1ToQdc1Phaseb    = 67U + (QDC1_PHASEB_REG << PMUX_SHIFT),

    /*!< Qdc1 Phasea. */
    kINPUTMUX_ArmTxevToQdc1Phasea            = 1U + (QDC1_PHASEA_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out0ToQdc1Phasea           = 2U + (QDC1_PHASEA_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out1ToQdc1Phasea           = 3U + (QDC1_PHASEA_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out2ToQdc1Phasea           = 4U + (QDC1_PHASEA_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out3ToQdc1Phasea           = 5U + (QDC1_PHASEA_REG << PMUX_SHIFT),
    kINPUTMUX_Cmp0OutToQdc1Phasea            = 6U + (QDC1_PHASEA_REG << PMUX_SHIFT),
    kINPUTMUX_Cmp1OutToQdc1Phasea            = 7U + (QDC1_PHASEA_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer0M2ToQdc1Phasea          = 9U + (QDC1_PHASEA_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer0M3ToQdc1Phasea          = 10U + (QDC1_PHASEA_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer1M2ToQdc1Phasea          = 11U + (QDC1_PHASEA_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer1M3ToQdc1Phasea          = 12U + (QDC1_PHASEA_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer2M2ToQdc1Phasea          = 13U + (QDC1_PHASEA_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer2M3ToQdc1Phasea          = 14U + (QDC1_PHASEA_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm0OutTrig0ToQdc1Phasea    = 16U + (QDC1_PHASEA_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm0OutTrig1ToQdc1Phasea    = 17U + (QDC1_PHASEA_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm1OutTrig0ToQdc1Phasea    = 18U + (QDC1_PHASEA_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm1OutTrig1ToQdc1Phasea    = 19U + (QDC1_PHASEA_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm2OutTrig0ToQdc1Phasea    = 20U + (QDC1_PHASEA_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm2OutTrig1ToQdc1Phasea    = 21U + (QDC1_PHASEA_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn0ToQdc1Phasea            = 24U + (QDC1_PHASEA_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn1ToQdc1Phasea            = 25U + (QDC1_PHASEA_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn2ToQdc1Phasea            = 26U + (QDC1_PHASEA_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn3ToQdc1Phasea            = 27U + (QDC1_PHASEA_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn4ToQdc1Phasea            = 28U + (QDC1_PHASEA_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn5ToQdc1Phasea            = 29U + (QDC1_PHASEA_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn6ToQdc1Phasea            = 30U + (QDC1_PHASEA_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn7ToQdc1Phasea            = 31U + (QDC1_PHASEA_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn8ToQdc1Phasea            = 32U + (QDC1_PHASEA_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn9ToQdc1Phasea            = 33U + (QDC1_PHASEA_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn10ToQdc1Phasea           = 34U + (QDC1_PHASEA_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn11ToQdc1Phasea           = 35U + (QDC1_PHASEA_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio0PinEventTrig0ToQdc1Phasea = 36U + (QDC1_PHASEA_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio1PinEventTrig0ToQdc1Phasea = 37U + (QDC1_PHASEA_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio2PinEventTrig0ToQdc1Phasea = 38U + (QDC1_PHASEA_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio3PinEventTrig0ToQdc1Phasea = 39U + (QDC1_PHASEA_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio4PinEventTrig0ToQdc1Phasea = 40U + (QDC1_PHASEA_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out0ToQdc1Phasea           = 41U + (QDC1_PHASEA_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out1ToQdc1Phasea           = 42U + (QDC1_PHASEA_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out2ToQdc1Phasea           = 43U + (QDC1_PHASEA_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out3ToQdc1Phasea           = 44U + (QDC1_PHASEA_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer3M2ToQdc1Phasea          = 49U + (QDC1_PHASEA_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer3M3ToQdc1Phasea          = 50U + (QDC1_PHASEA_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer4M2ToQdc1Phasea          = 51U + (QDC1_PHASEA_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer4M3ToQdc1Phasea          = 52U + (QDC1_PHASEA_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm1Sm0OutTrig0ToQdc1Phasea    = 62U + (QDC1_PHASEA_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm1Sm0OutTrig1ToQdc1Phasea    = 63U + (QDC1_PHASEA_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm1Sm1OutTrig0ToQdc1Phasea    = 64U + (QDC1_PHASEA_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm1Sm1OutTrig1ToQdc1Phasea    = 65U + (QDC1_PHASEA_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm1Sm2OutTrig0ToQdc1Phasea    = 66U + (QDC1_PHASEA_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm1Sm2OutTrig1ToQdc1Phasea    = 67U + (QDC1_PHASEA_REG << PMUX_SHIFT),

    /*!< Qdc1 Icap1. */
    kINPUTMUX_ArmTxevToQdc1Icap1            = 1U + (QDC1_ICAP1_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out0ToQdc1Icap1           = 2U + (QDC1_ICAP1_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out1ToQdc1Icap1           = 3U + (QDC1_ICAP1_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out2ToQdc1Icap1           = 4U + (QDC1_ICAP1_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out3ToQdc1Icap1           = 5U + (QDC1_ICAP1_REG << PMUX_SHIFT),
    kINPUTMUX_Cmp0OutToQdc1Icap1            = 6U + (QDC1_ICAP1_REG << PMUX_SHIFT),
    kINPUTMUX_Cmp1OutToQdc1Icap1            = 7U + (QDC1_ICAP1_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer0M2ToQdc1Icap1          = 9U + (QDC1_ICAP1_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer0M3ToQdc1Icap1          = 10U + (QDC1_ICAP1_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer1M2ToQdc1Icap1          = 11U + (QDC1_ICAP1_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer1M3ToQdc1Icap1          = 12U + (QDC1_ICAP1_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer2M2ToQdc1Icap1          = 13U + (QDC1_ICAP1_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer2M3ToQdc1Icap1          = 14U + (QDC1_ICAP1_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm0OutTrig0ToQdc1Icap1    = 16U + (QDC1_ICAP1_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm0OutTrig1ToQdc1Icap1    = 17U + (QDC1_ICAP1_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm1OutTrig0ToQdc1Icap1    = 18U + (QDC1_ICAP1_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm1OutTrig1ToQdc1Icap1    = 19U + (QDC1_ICAP1_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm2OutTrig0ToQdc1Icap1    = 20U + (QDC1_ICAP1_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm2OutTrig1ToQdc1Icap1    = 21U + (QDC1_ICAP1_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn0ToQdc1Icap1            = 24U + (QDC1_ICAP1_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn1ToQdc1Icap1            = 25U + (QDC1_ICAP1_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn2ToQdc1Icap1            = 26U + (QDC1_ICAP1_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn3ToQdc1Icap1            = 27U + (QDC1_ICAP1_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn4ToQdc1Icap1            = 28U + (QDC1_ICAP1_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn5ToQdc1Icap1            = 29U + (QDC1_ICAP1_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn6ToQdc1Icap1            = 30U + (QDC1_ICAP1_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn7ToQdc1Icap1            = 31U + (QDC1_ICAP1_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn8ToQdc1Icap1            = 32U + (QDC1_ICAP1_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn9ToQdc1Icap1            = 33U + (QDC1_ICAP1_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn10ToQdc1Icap1           = 34U + (QDC1_ICAP1_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn11ToQdc1Icap1           = 35U + (QDC1_ICAP1_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio0PinEventTrig0ToQdc1Icap1 = 36U + (QDC1_ICAP1_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio1PinEventTrig0ToQdc1Icap1 = 37U + (QDC1_ICAP1_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio2PinEventTrig0ToQdc1Icap1 = 38U + (QDC1_ICAP1_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio3PinEventTrig0ToQdc1Icap1 = 39U + (QDC1_ICAP1_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio4PinEventTrig0ToQdc1Icap1 = 40U + (QDC1_ICAP1_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out0ToQdc1Icap1           = 41U + (QDC1_ICAP1_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out1ToQdc1Icap1           = 42U + (QDC1_ICAP1_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out2ToQdc1Icap1           = 43U + (QDC1_ICAP1_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out3ToQdc1Icap1           = 44U + (QDC1_ICAP1_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer3M2ToQdc1Icap1          = 49U + (QDC1_ICAP1_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer3M3ToQdc1Icap1          = 50U + (QDC1_ICAP1_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer4M2ToQdc1Icap1          = 51U + (QDC1_ICAP1_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer4M3ToQdc1Icap1          = 52U + (QDC1_ICAP1_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm1Sm0OutTrig0ToQdc1Icap1    = 62U + (QDC1_ICAP1_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm1Sm0OutTrig1ToQdc1Icap1    = 63U + (QDC1_ICAP1_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm1Sm1OutTrig0ToQdc1Icap1    = 64U + (QDC1_ICAP1_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm1Sm1OutTrig1ToQdc1Icap1    = 65U + (QDC1_ICAP1_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm1Sm2OutTrig0ToQdc1Icap1    = 66U + (QDC1_ICAP1_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm1Sm2OutTrig1ToQdc1Icap1    = 67U + (QDC1_ICAP1_REG << PMUX_SHIFT),

    /*!< FlexPWM0_SM0_EXTA0 input trigger connections. */
    kINPUTMUX_ArmTxevToFlexPwm0Sm0Exta0            = 1U + (FlexPWM0_SM0_EXTA0_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out0ToFlexPwm0Sm0Exta0           = 2U + (FlexPWM0_SM0_EXTA0_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out1ToFlexPwm0Sm0Exta0           = 3U + (FlexPWM0_SM0_EXTA0_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out2ToFlexPwm0Sm0Exta0           = 4U + (FlexPWM0_SM0_EXTA0_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out3ToFlexPwm0Sm0Exta0           = 5U + (FlexPWM0_SM0_EXTA0_REG << PMUX_SHIFT),
    kINPUTMUX_Cmp0OutToFlexPwm0Sm0Exta0            = 6U + (FlexPWM0_SM0_EXTA0_REG << PMUX_SHIFT),
    kINPUTMUX_Cmp1OutToFlexPwm0Sm0Exta0            = 7U + (FlexPWM0_SM0_EXTA0_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer0M2ToFlexPwm0Sm0Exta0          = 9U + (FlexPWM0_SM0_EXTA0_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer0M3ToFlexPwm0Sm0Exta0          = 10U + (FlexPWM0_SM0_EXTA0_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer1M2ToFlexPwm0Sm0Exta0          = 11U + (FlexPWM0_SM0_EXTA0_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer1M3ToFlexPwm0Sm0Exta0          = 12U + (FlexPWM0_SM0_EXTA0_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer2M2ToFlexPwm0Sm0Exta0          = 13U + (FlexPWM0_SM0_EXTA0_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer2M3ToFlexPwm0Sm0Exta0          = 14U + (FlexPWM0_SM0_EXTA0_REG << PMUX_SHIFT),
    kINPUTMUX_Qdc0CmpFlag0ToFlexPwm0Sm0Exta0       = 15U + (FlexPWM0_SM0_EXTA0_REG << PMUX_SHIFT),
    kINPUTMUX_Qdc0CmpFlag1ToFlexPwm0Sm0Exta0       = 16U + (FlexPWM0_SM0_EXTA0_REG << PMUX_SHIFT),
    kINPUTMUX_Qdc0CmpFlag2ToFlexPwm0Sm0Exta0       = 17U + (FlexPWM0_SM0_EXTA0_REG << PMUX_SHIFT),
    kINPUTMUX_Qdc0CmpFlag3ToFlexPwm0Sm0Exta0       = 18U + (FlexPWM0_SM0_EXTA0_REG << PMUX_SHIFT),
    kINPUTMUX_Qdc0PosMatch0ToFlexPwm0Sm0Exta0      = 19U + (FlexPWM0_SM0_EXTA0_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn0ToFlexPwm0Sm0Exta0            = 20U + (FlexPWM0_SM0_EXTA0_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn1ToFlexPwm0Sm0Exta0            = 21U + (FlexPWM0_SM0_EXTA0_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn2ToFlexPwm0Sm0Exta0            = 22U + (FlexPWM0_SM0_EXTA0_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn3ToFlexPwm0Sm0Exta0            = 23U + (FlexPWM0_SM0_EXTA0_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn4ToFlexPwm0Sm0Exta0            = 24U + (FlexPWM0_SM0_EXTA0_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn5ToFlexPwm0Sm0Exta0            = 25U + (FlexPWM0_SM0_EXTA0_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn6ToFlexPwm0Sm0Exta0            = 26U + (FlexPWM0_SM0_EXTA0_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn7ToFlexPwm0Sm0Exta0            = 27U + (FlexPWM0_SM0_EXTA0_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn8ToFlexPwm0Sm0Exta0            = 28U + (FlexPWM0_SM0_EXTA0_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn9ToFlexPwm0Sm0Exta0            = 29U + (FlexPWM0_SM0_EXTA0_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn10ToFlexPwm0Sm0Exta0           = 30U + (FlexPWM0_SM0_EXTA0_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn11ToFlexPwm0Sm0Exta0           = 31U + (FlexPWM0_SM0_EXTA0_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio0PinEventTrig0ToFlexPwm0Sm0Exta0 = 32U + (FlexPWM0_SM0_EXTA0_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio1PinEventTrig0ToFlexPwm0Sm0Exta0 = 33U + (FlexPWM0_SM0_EXTA0_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio2PinEventTrig0ToFlexPwm0Sm0Exta0 = 34U + (FlexPWM0_SM0_EXTA0_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio3PinEventTrig0ToFlexPwm0Sm0Exta0 = 35U + (FlexPWM0_SM0_EXTA0_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio4PinEventTrig0ToFlexPwm0Sm0Exta0 = 36U + (FlexPWM0_SM0_EXTA0_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out0ToFlexPwm0Sm0Exta0           = 37U + (FlexPWM0_SM0_EXTA0_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out1ToFlexPwm0Sm0Exta0           = 38U + (FlexPWM0_SM0_EXTA0_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out2ToFlexPwm0Sm0Exta0           = 39U + (FlexPWM0_SM0_EXTA0_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out3ToFlexPwm0Sm0Exta0           = 40U + (FlexPWM0_SM0_EXTA0_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer3M2ToFlexPwm0Sm0Exta0          = 45U + (FlexPWM0_SM0_EXTA0_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer3M3ToFlexPwm0Sm0Exta0          = 46U + (FlexPWM0_SM0_EXTA0_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer4M2ToFlexPwm0Sm0Exta0          = 47U + (FlexPWM0_SM0_EXTA0_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer4M3ToFlexPwm0Sm0Exta0          = 48U + (FlexPWM0_SM0_EXTA0_REG << PMUX_SHIFT),
    kINPUTMUX_Qdc1CmpFlag0ToFlexPwm0Sm0Exta0       = 49U + (FlexPWM0_SM0_EXTA0_REG << PMUX_SHIFT),
    kINPUTMUX_Qdc1CmpFlag1ToFlexPwm0Sm0Exta0       = 50U + (FlexPWM0_SM0_EXTA0_REG << PMUX_SHIFT),
    kINPUTMUX_Qdc1CmpFlag2ToFlexPwm0Sm0Exta0       = 51U + (FlexPWM0_SM0_EXTA0_REG << PMUX_SHIFT),
    kINPUTMUX_Qdc1CmpFlag3ToFlexPwm0Sm0Exta0       = 52U + (FlexPWM0_SM0_EXTA0_REG << PMUX_SHIFT),
    kINPUTMUX_Qdc1PosMatch0ToFlexPwm0Sm0Exta0      = 53U + (FlexPWM0_SM0_EXTA0_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm1Sm0OutTrig0ToFlexPwm0Sm0Exta0    = 54U + (FlexPWM0_SM0_EXTA0_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm1Sm0OutTrig1ToFlexPwm0Sm0Exta0    = 55U + (FlexPWM0_SM0_EXTA0_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm1Sm1OutTrig0ToFlexPwm0Sm0Exta0    = 56U + (FlexPWM0_SM0_EXTA0_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm1Sm1OutTrig1ToFlexPwm0Sm0Exta0    = 57U + (FlexPWM0_SM0_EXTA0_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm1Sm2OutTrig0ToFlexPwm0Sm0Exta0    = 58U + (FlexPWM0_SM0_EXTA0_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm1Sm2OutTrig1ToFlexPwm0Sm0Exta0    = 59U + (FlexPWM0_SM0_EXTA0_REG << PMUX_SHIFT),

    /*!< FlexPWM0_SM1_EXTA1 input trigger connections. */
    kINPUTMUX_ArmTxevToFlexPwm0Sm1Exta1            = 1U + (FlexPWM0_SM1_EXTA1_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out0ToFlexPwm0Sm1Exta1           = 2U + (FlexPWM0_SM1_EXTA1_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out1ToFlexPwm0Sm1Exta1           = 3U + (FlexPWM0_SM1_EXTA1_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out2ToFlexPwm0Sm1Exta1           = 4U + (FlexPWM0_SM1_EXTA1_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out3ToFlexPwm0Sm1Exta1           = 5U + (FlexPWM0_SM1_EXTA1_REG << PMUX_SHIFT),
    kINPUTMUX_Cmp0OutToFlexPwm0Sm1Exta1            = 6U + (FlexPWM0_SM1_EXTA1_REG << PMUX_SHIFT),
    kINPUTMUX_Cmp1OutToFlexPwm0Sm1Exta1            = 7U + (FlexPWM0_SM1_EXTA1_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer0M2ToFlexPwm0Sm1Exta1          = 9U + (FlexPWM0_SM1_EXTA1_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer0M3ToFlexPwm0Sm1Exta1          = 10U + (FlexPWM0_SM1_EXTA1_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer1M2ToFlexPwm0Sm1Exta1          = 11U + (FlexPWM0_SM1_EXTA1_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer1M3ToFlexPwm0Sm1Exta1          = 12U + (FlexPWM0_SM1_EXTA1_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer2M2ToFlexPwm0Sm1Exta1          = 13U + (FlexPWM0_SM1_EXTA1_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer2M3ToFlexPwm0Sm1Exta1          = 14U + (FlexPWM0_SM1_EXTA1_REG << PMUX_SHIFT),
    kINPUTMUX_Qdc0CmpFlag0ToFlexPwm0Sm1Exta1       = 15U + (FlexPWM0_SM1_EXTA1_REG << PMUX_SHIFT),
    kINPUTMUX_Qdc0CmpFlag1ToFlexPwm0Sm1Exta1       = 16U + (FlexPWM0_SM1_EXTA1_REG << PMUX_SHIFT),
    kINPUTMUX_Qdc0CmpFlag2ToFlexPwm0Sm1Exta1       = 17U + (FlexPWM0_SM1_EXTA1_REG << PMUX_SHIFT),
    kINPUTMUX_Qdc0CmpFlag3ToFlexPwm0Sm1Exta1       = 18U + (FlexPWM0_SM1_EXTA1_REG << PMUX_SHIFT),
    kINPUTMUX_Qdc0PosMatch0ToFlexPwm0Sm1Exta1      = 19U + (FlexPWM0_SM1_EXTA1_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn0ToFlexPwm0Sm1Exta1            = 20U + (FlexPWM0_SM1_EXTA1_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn1ToFlexPwm0Sm1Exta1            = 21U + (FlexPWM0_SM1_EXTA1_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn2ToFlexPwm0Sm1Exta1            = 22U + (FlexPWM0_SM1_EXTA1_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn3ToFlexPwm0Sm1Exta1            = 23U + (FlexPWM0_SM1_EXTA1_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn4ToFlexPwm0Sm1Exta1            = 24U + (FlexPWM0_SM1_EXTA1_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn5ToFlexPwm0Sm1Exta1            = 25U + (FlexPWM0_SM1_EXTA1_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn6ToFlexPwm0Sm1Exta1            = 26U + (FlexPWM0_SM1_EXTA1_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn7ToFlexPwm0Sm1Exta1            = 27U + (FlexPWM0_SM1_EXTA1_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn8ToFlexPwm0Sm1Exta1            = 28U + (FlexPWM0_SM1_EXTA1_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn9ToFlexPwm0Sm1Exta1            = 29U + (FlexPWM0_SM1_EXTA1_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn10ToFlexPwm0Sm1Exta1           = 30U + (FlexPWM0_SM1_EXTA1_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn11ToFlexPwm0Sm1Exta1           = 31U + (FlexPWM0_SM1_EXTA1_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio0PinEventTrig0ToFlexPwm0Sm1Exta1 = 32U + (FlexPWM0_SM1_EXTA1_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio1PinEventTrig0ToFlexPwm0Sm1Exta1 = 33U + (FlexPWM0_SM1_EXTA1_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio2PinEventTrig0ToFlexPwm0Sm1Exta1 = 34U + (FlexPWM0_SM1_EXTA1_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio3PinEventTrig0ToFlexPwm0Sm1Exta1 = 35U + (FlexPWM0_SM1_EXTA1_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio4PinEventTrig0ToFlexPwm0Sm1Exta1 = 36U + (FlexPWM0_SM1_EXTA1_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out0ToFlexPwm0Sm1Exta1           = 37U + (FlexPWM0_SM1_EXTA1_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out1ToFlexPwm0Sm1Exta1           = 38U + (FlexPWM0_SM1_EXTA1_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out2ToFlexPwm0Sm1Exta1           = 39U + (FlexPWM0_SM1_EXTA1_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out3ToFlexPwm0Sm1Exta1           = 40U + (FlexPWM0_SM1_EXTA1_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer3M2ToFlexPwm0Sm1Exta1          = 45U + (FlexPWM0_SM1_EXTA1_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer3M3ToFlexPwm0Sm1Exta1          = 46U + (FlexPWM0_SM1_EXTA1_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer4M2ToFlexPwm0Sm1Exta1          = 47U + (FlexPWM0_SM1_EXTA1_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer4M3ToFlexPwm0Sm1Exta1          = 48U + (FlexPWM0_SM1_EXTA1_REG << PMUX_SHIFT),
    kINPUTMUX_Qdc1CmpFlag0ToFlexPwm0Sm1Exta1       = 49U + (FlexPWM0_SM1_EXTA1_REG << PMUX_SHIFT),
    kINPUTMUX_Qdc1CmpFlag1ToFlexPwm0Sm1Exta1       = 50U + (FlexPWM0_SM1_EXTA1_REG << PMUX_SHIFT),
    kINPUTMUX_Qdc1CmpFlag2ToFlexPwm0Sm1Exta1       = 51U + (FlexPWM0_SM1_EXTA1_REG << PMUX_SHIFT),
    kINPUTMUX_Qdc1CmpFlag3ToFlexPwm0Sm1Exta1       = 52U + (FlexPWM0_SM1_EXTA1_REG << PMUX_SHIFT),
    kINPUTMUX_Qdc1PosMatch0ToFlexPwm0Sm1Exta1      = 53U + (FlexPWM0_SM1_EXTA1_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm1Sm0OutTrig0ToFlexPwm0Sm1Exta1    = 54U + (FlexPWM0_SM1_EXTA1_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm1Sm0OutTrig1ToFlexPwm0Sm1Exta1    = 55U + (FlexPWM0_SM1_EXTA1_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm1Sm1OutTrig0ToFlexPwm0Sm1Exta1    = 56U + (FlexPWM0_SM1_EXTA1_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm1Sm1OutTrig1ToFlexPwm0Sm1Exta1    = 57U + (FlexPWM0_SM1_EXTA1_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm1Sm2OutTrig0ToFlexPwm0Sm1Exta1    = 58U + (FlexPWM0_SM1_EXTA1_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm1Sm2OutTrig1ToFlexPwm0Sm1Exta1    = 59U + (FlexPWM0_SM1_EXTA1_REG << PMUX_SHIFT),

    /*!< FlexPWM0_SM2_EXTA2 input trigger connections. */
    kINPUTMUX_ArmTxevToFlexPwm0Sm2Exta2            = 1U + (FlexPWM0_SM2_EXTA2_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out0ToFlexPwm0Sm2Exta2           = 2U + (FlexPWM0_SM2_EXTA2_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out1ToFlexPwm0Sm2Exta2           = 3U + (FlexPWM0_SM2_EXTA2_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out2ToFlexPwm0Sm2Exta2           = 4U + (FlexPWM0_SM2_EXTA2_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out3ToFlexPwm0Sm2Exta2           = 5U + (FlexPWM0_SM2_EXTA2_REG << PMUX_SHIFT),
    kINPUTMUX_Cmp0OutToFlexPwm0Sm2Exta2            = 6U + (FlexPWM0_SM2_EXTA2_REG << PMUX_SHIFT),
    kINPUTMUX_Cmp1OutToFlexPwm0Sm2Exta2            = 7U + (FlexPWM0_SM2_EXTA2_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer0M2ToFlexPwm0Sm2Exta2          = 9U + (FlexPWM0_SM2_EXTA2_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer0M3ToFlexPwm0Sm2Exta2          = 10U + (FlexPWM0_SM2_EXTA2_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer1M2ToFlexPwm0Sm2Exta2          = 11U + (FlexPWM0_SM2_EXTA2_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer1M3ToFlexPwm0Sm2Exta2          = 12U + (FlexPWM0_SM2_EXTA2_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer2M2ToFlexPwm0Sm2Exta2          = 13U + (FlexPWM0_SM2_EXTA2_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer2M3ToFlexPwm0Sm2Exta2          = 14U + (FlexPWM0_SM2_EXTA2_REG << PMUX_SHIFT),
    kINPUTMUX_Qdc0CmpFlag0ToFlexPwm0Sm2Exta2       = 15U + (FlexPWM0_SM2_EXTA2_REG << PMUX_SHIFT),
    kINPUTMUX_Qdc0CmpFlag1ToFlexPwm0Sm2Exta2       = 16U + (FlexPWM0_SM2_EXTA2_REG << PMUX_SHIFT),
    kINPUTMUX_Qdc0CmpFlag2ToFlexPwm0Sm2Exta2       = 17U + (FlexPWM0_SM2_EXTA2_REG << PMUX_SHIFT),
    kINPUTMUX_Qdc0CmpFlag3ToFlexPwm0Sm2Exta2       = 18U + (FlexPWM0_SM2_EXTA2_REG << PMUX_SHIFT),
    kINPUTMUX_Qdc0PosMatch0ToFlexPwm0Sm2Exta2      = 19U + (FlexPWM0_SM2_EXTA2_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn0ToFlexPwm0Sm2Exta2            = 20U + (FlexPWM0_SM2_EXTA2_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn1ToFlexPwm0Sm2Exta2            = 21U + (FlexPWM0_SM2_EXTA2_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn2ToFlexPwm0Sm2Exta2            = 22U + (FlexPWM0_SM2_EXTA2_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn3ToFlexPwm0Sm2Exta2            = 23U + (FlexPWM0_SM2_EXTA2_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn4ToFlexPwm0Sm2Exta2            = 24U + (FlexPWM0_SM2_EXTA2_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn5ToFlexPwm0Sm2Exta2            = 25U + (FlexPWM0_SM2_EXTA2_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn6ToFlexPwm0Sm2Exta2            = 26U + (FlexPWM0_SM2_EXTA2_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn7ToFlexPwm0Sm2Exta2            = 27U + (FlexPWM0_SM2_EXTA2_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn8ToFlexPwm0Sm2Exta2            = 28U + (FlexPWM0_SM2_EXTA2_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn9ToFlexPwm0Sm2Exta2            = 29U + (FlexPWM0_SM2_EXTA2_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn10ToFlexPwm0Sm2Exta2           = 30U + (FlexPWM0_SM2_EXTA2_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn11ToFlexPwm0Sm2Exta2           = 31U + (FlexPWM0_SM2_EXTA2_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio0PinEventTrig0ToFlexPwm0Sm2Exta2 = 32U + (FlexPWM0_SM2_EXTA2_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio1PinEventTrig0ToFlexPwm0Sm2Exta2 = 33U + (FlexPWM0_SM2_EXTA2_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio2PinEventTrig0ToFlexPwm0Sm2Exta2 = 34U + (FlexPWM0_SM2_EXTA2_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio3PinEventTrig0ToFlexPwm0Sm2Exta2 = 35U + (FlexPWM0_SM2_EXTA2_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio4PinEventTrig0ToFlexPwm0Sm2Exta2 = 36U + (FlexPWM0_SM2_EXTA2_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out0ToFlexPwm0Sm2Exta2           = 37U + (FlexPWM0_SM2_EXTA2_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out1ToFlexPwm0Sm2Exta2           = 38U + (FlexPWM0_SM2_EXTA2_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out2ToFlexPwm0Sm2Exta2           = 39U + (FlexPWM0_SM2_EXTA2_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out3ToFlexPwm0Sm2Exta2           = 40U + (FlexPWM0_SM2_EXTA2_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer3M2ToFlexPwm0Sm2Exta2          = 45U + (FlexPWM0_SM2_EXTA2_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer3M3ToFlexPwm0Sm2Exta2          = 46U + (FlexPWM0_SM2_EXTA2_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer4M2ToFlexPwm0Sm2Exta2          = 47U + (FlexPWM0_SM2_EXTA2_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer4M3ToFlexPwm0Sm2Exta2          = 48U + (FlexPWM0_SM2_EXTA2_REG << PMUX_SHIFT),
    kINPUTMUX_Qdc1CmpFlag0ToFlexPwm0Sm2Exta2       = 49U + (FlexPWM0_SM2_EXTA2_REG << PMUX_SHIFT),
    kINPUTMUX_Qdc1CmpFlag1ToFlexPwm0Sm2Exta2       = 50U + (FlexPWM0_SM2_EXTA2_REG << PMUX_SHIFT),
    kINPUTMUX_Qdc1CmpFlag2ToFlexPwm0Sm2Exta2       = 51U + (FlexPWM0_SM2_EXTA2_REG << PMUX_SHIFT),
    kINPUTMUX_Qdc1CmpFlag3ToFlexPwm0Sm2Exta2       = 52U + (FlexPWM0_SM2_EXTA2_REG << PMUX_SHIFT),
    kINPUTMUX_Qdc1PosMatch0ToFlexPwm0Sm2Exta2      = 53U + (FlexPWM0_SM2_EXTA2_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm1Sm0OutTrig0ToFlexPwm0Sm2Exta2    = 54U + (FlexPWM0_SM2_EXTA2_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm1Sm0OutTrig1ToFlexPwm0Sm2Exta2    = 55U + (FlexPWM0_SM2_EXTA2_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm1Sm1OutTrig0ToFlexPwm0Sm2Exta2    = 56U + (FlexPWM0_SM2_EXTA2_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm1Sm1OutTrig1ToFlexPwm0Sm2Exta2    = 57U + (FlexPWM0_SM2_EXTA2_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm1Sm2OutTrig0ToFlexPwm0Sm2Exta2    = 58U + (FlexPWM0_SM2_EXTA2_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm1Sm2OutTrig1ToFlexPwm0Sm2Exta2    = 59U + (FlexPWM0_SM2_EXTA2_REG << PMUX_SHIFT),

    /*!< FlexPWM0_SM0_EXTSYNC0 input trigger connections. */
    kINPUTMUX_ArmTxevToFlexPwm0Sm0Extsync0            = 1U + (FlexPWM0_SM0_EXTSYNC0_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out0ToFlexPwm0Sm0Extsync0           = 2U + (FlexPWM0_SM0_EXTSYNC0_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out1ToFlexPwm0Sm0Extsync0           = 3U + (FlexPWM0_SM0_EXTSYNC0_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out2ToFlexPwm0Sm0Extsync0           = 4U + (FlexPWM0_SM0_EXTSYNC0_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out3ToFlexPwm0Sm0Extsync0           = 5U + (FlexPWM0_SM0_EXTSYNC0_REG << PMUX_SHIFT),
    kINPUTMUX_Cmp0OutToFlexPwm0Sm0Extsync0            = 6U + (FlexPWM0_SM0_EXTSYNC0_REG << PMUX_SHIFT),
    kINPUTMUX_Cmp1OutToFlexPwm0Sm0Extsync0            = 7U + (FlexPWM0_SM0_EXTSYNC0_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer0M2ToFlexPwm0Sm0Extsync0          = 9U + (FlexPWM0_SM0_EXTSYNC0_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer0M3ToFlexPwm0Sm0Extsync0          = 10U + (FlexPWM0_SM0_EXTSYNC0_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer1M2ToFlexPwm0Sm0Extsync0          = 11U + (FlexPWM0_SM0_EXTSYNC0_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer1M3ToFlexPwm0Sm0Extsync0          = 12U + (FlexPWM0_SM0_EXTSYNC0_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer2M2ToFlexPwm0Sm0Extsync0          = 13U + (FlexPWM0_SM0_EXTSYNC0_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer2M3ToFlexPwm0Sm0Extsync0          = 14U + (FlexPWM0_SM0_EXTSYNC0_REG << PMUX_SHIFT),
    kINPUTMUX_Qdc0CmpFlag0ToFlexPwm0Sm0Extsync0       = 15U + (FlexPWM0_SM0_EXTSYNC0_REG << PMUX_SHIFT),
    kINPUTMUX_Qdc0CmpFlag1ToFlexPwm0Sm0Extsync0       = 16U + (FlexPWM0_SM0_EXTSYNC0_REG << PMUX_SHIFT),
    kINPUTMUX_Qdc0CmpFlag2ToFlexPwm0Sm0Extsync0       = 17U + (FlexPWM0_SM0_EXTSYNC0_REG << PMUX_SHIFT),
    kINPUTMUX_Qdc0CmpFlag3ToFlexPwm0Sm0Extsync0       = 18U + (FlexPWM0_SM0_EXTSYNC0_REG << PMUX_SHIFT),
    kINPUTMUX_Qdc0PosMatch0ToFlexPwm0Sm0Extsync0      = 19U + (FlexPWM0_SM0_EXTSYNC0_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn0ToFlexPwm0Sm0Extsync0            = 20U + (FlexPWM0_SM0_EXTSYNC0_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn1ToFlexPwm0Sm0Extsync0            = 21U + (FlexPWM0_SM0_EXTSYNC0_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn2ToFlexPwm0Sm0Extsync0            = 22U + (FlexPWM0_SM0_EXTSYNC0_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn3ToFlexPwm0Sm0Extsync0            = 23U + (FlexPWM0_SM0_EXTSYNC0_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn4ToFlexPwm0Sm0Extsync0            = 24U + (FlexPWM0_SM0_EXTSYNC0_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn5ToFlexPwm0Sm0Extsync0            = 25U + (FlexPWM0_SM0_EXTSYNC0_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn6ToFlexPwm0Sm0Extsync0            = 26U + (FlexPWM0_SM0_EXTSYNC0_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn7ToFlexPwm0Sm0Extsync0            = 27U + (FlexPWM0_SM0_EXTSYNC0_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn8ToFlexPwm0Sm0Extsync0            = 28U + (FlexPWM0_SM0_EXTSYNC0_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn9ToFlexPwm0Sm0Extsync0            = 29U + (FlexPWM0_SM0_EXTSYNC0_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn10ToFlexPwm0Sm0Extsync0           = 30U + (FlexPWM0_SM0_EXTSYNC0_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn11ToFlexPwm0Sm0Extsync0           = 31U + (FlexPWM0_SM0_EXTSYNC0_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio0PinEventTrig0ToFlexPwm0Sm0Extsync0 = 32U + (FlexPWM0_SM0_EXTSYNC0_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio1PinEventTrig0ToFlexPwm0Sm0Extsync0 = 33U + (FlexPWM0_SM0_EXTSYNC0_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio2PinEventTrig0ToFlexPwm0Sm0Extsync0 = 34U + (FlexPWM0_SM0_EXTSYNC0_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio3PinEventTrig0ToFlexPwm0Sm0Extsync0 = 35U + (FlexPWM0_SM0_EXTSYNC0_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio4PinEventTrig0ToFlexPwm0Sm0Extsync0 = 36U + (FlexPWM0_SM0_EXTSYNC0_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out0ToFlexPwm0Sm0Extsync0           = 37U + (FlexPWM0_SM0_EXTSYNC0_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out1ToFlexPwm0Sm0Extsync0           = 38U + (FlexPWM0_SM0_EXTSYNC0_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out2ToFlexPwm0Sm0Extsync0           = 39U + (FlexPWM0_SM0_EXTSYNC0_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out3ToFlexPwm0Sm0Extsync0           = 40U + (FlexPWM0_SM0_EXTSYNC0_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer3M2ToFlexPwm0Sm0Extsync0          = 45U + (FlexPWM0_SM0_EXTSYNC0_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer3M3ToFlexPwm0Sm0Extsync0          = 46U + (FlexPWM0_SM0_EXTSYNC0_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer4M2ToFlexPwm0Sm0Extsync0          = 47U + (FlexPWM0_SM0_EXTSYNC0_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer4M3ToFlexPwm0Sm0Extsync0          = 48U + (FlexPWM0_SM0_EXTSYNC0_REG << PMUX_SHIFT),
    kINPUTMUX_Qdc1CmpFlag0ToFlexPwm0Sm0Extsync0       = 49U + (FlexPWM0_SM0_EXTSYNC0_REG << PMUX_SHIFT),
    kINPUTMUX_Qdc1CmpFlag1ToFlexPwm0Sm0Extsync0       = 50U + (FlexPWM0_SM0_EXTSYNC0_REG << PMUX_SHIFT),
    kINPUTMUX_Qdc1CmpFlag2ToFlexPwm0Sm0Extsync0       = 51U + (FlexPWM0_SM0_EXTSYNC0_REG << PMUX_SHIFT),
    kINPUTMUX_Qdc1CmpFlag3ToFlexPwm0Sm0Extsync0       = 52U + (FlexPWM0_SM0_EXTSYNC0_REG << PMUX_SHIFT),
    kINPUTMUX_Qdc1PosMatch0ToFlexPwm0Sm0Extsync0      = 53U + (FlexPWM0_SM0_EXTSYNC0_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm1Sm0OutTrig0ToFlexPwm0Sm0Extsync0    = 54U + (FlexPWM0_SM0_EXTSYNC0_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm1Sm0OutTrig1ToFlexPwm0Sm0Extsync0    = 55U + (FlexPWM0_SM0_EXTSYNC0_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm1Sm1OutTrig0ToFlexPwm0Sm0Extsync0    = 56U + (FlexPWM0_SM0_EXTSYNC0_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm1Sm1OutTrig1ToFlexPwm0Sm0Extsync0    = 57U + (FlexPWM0_SM0_EXTSYNC0_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm1Sm2OutTrig0ToFlexPwm0Sm0Extsync0    = 58U + (FlexPWM0_SM0_EXTSYNC0_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm1Sm2OutTrig1ToFlexPwm0Sm0Extsync0    = 59U + (FlexPWM0_SM0_EXTSYNC0_REG << PMUX_SHIFT),

    /*!< FlexPWM0_SM1_EXTSYNC1 input trigger connections. */
    kINPUTMUX_ArmTxevToFlexPwm0Sm1Extsync1            = 1U + (FlexPWM0_SM1_EXTSYNC1_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out0ToFlexPwm0Sm1Extsync1           = 2U + (FlexPWM0_SM1_EXTSYNC1_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out1ToFlexPwm0Sm1Extsync1           = 3U + (FlexPWM0_SM1_EXTSYNC1_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out2ToFlexPwm0Sm1Extsync1           = 4U + (FlexPWM0_SM1_EXTSYNC1_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out3ToFlexPwm0Sm1Extsync1           = 5U + (FlexPWM0_SM1_EXTSYNC1_REG << PMUX_SHIFT),
    kINPUTMUX_Cmp0OutToFlexPwm0Sm1Extsync1            = 6U + (FlexPWM0_SM1_EXTSYNC1_REG << PMUX_SHIFT),
    kINPUTMUX_Cmp1OutToFlexPwm0Sm1Extsync1            = 7U + (FlexPWM0_SM1_EXTSYNC1_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer0M2ToFlexPwm0Sm1Extsync1          = 9U + (FlexPWM0_SM1_EXTSYNC1_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer0M3ToFlexPwm0Sm1Extsync1          = 10U + (FlexPWM0_SM1_EXTSYNC1_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer1M2ToFlexPwm0Sm1Extsync1          = 11U + (FlexPWM0_SM1_EXTSYNC1_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer1M3ToFlexPwm0Sm1Extsync1          = 12U + (FlexPWM0_SM1_EXTSYNC1_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer2M2ToFlexPwm0Sm1Extsync1          = 13U + (FlexPWM0_SM1_EXTSYNC1_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer2M3ToFlexPwm0Sm1Extsync1          = 14U + (FlexPWM0_SM1_EXTSYNC1_REG << PMUX_SHIFT),
    kINPUTMUX_Qdc0CmpFlag0ToFlexPwm0Sm1Extsync1       = 15U + (FlexPWM0_SM1_EXTSYNC1_REG << PMUX_SHIFT),
    kINPUTMUX_Qdc0CmpFlag1ToFlexPwm0Sm1Extsync1       = 16U + (FlexPWM0_SM1_EXTSYNC1_REG << PMUX_SHIFT),
    kINPUTMUX_Qdc0CmpFlag2ToFlexPwm0Sm1Extsync1       = 17U + (FlexPWM0_SM1_EXTSYNC1_REG << PMUX_SHIFT),
    kINPUTMUX_Qdc0CmpFlag3ToFlexPwm0Sm1Extsync1       = 18U + (FlexPWM0_SM1_EXTSYNC1_REG << PMUX_SHIFT),
    kINPUTMUX_Qdc0PosMatch0ToFlexPwm0Sm1Extsync1      = 19U + (FlexPWM0_SM1_EXTSYNC1_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn0ToFlexPwm0Sm1Extsync1            = 20U + (FlexPWM0_SM1_EXTSYNC1_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn1ToFlexPwm0Sm1Extsync1            = 21U + (FlexPWM0_SM1_EXTSYNC1_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn2ToFlexPwm0Sm1Extsync1            = 22U + (FlexPWM0_SM1_EXTSYNC1_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn3ToFlexPwm0Sm1Extsync1            = 23U + (FlexPWM0_SM1_EXTSYNC1_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn4ToFlexPwm0Sm1Extsync1            = 24U + (FlexPWM0_SM1_EXTSYNC1_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn5ToFlexPwm0Sm1Extsync1            = 25U + (FlexPWM0_SM1_EXTSYNC1_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn6ToFlexPwm0Sm1Extsync1            = 26U + (FlexPWM0_SM1_EXTSYNC1_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn7ToFlexPwm0Sm1Extsync1            = 27U + (FlexPWM0_SM1_EXTSYNC1_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn8ToFlexPwm0Sm1Extsync1            = 28U + (FlexPWM0_SM1_EXTSYNC1_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn9ToFlexPwm0Sm1Extsync1            = 29U + (FlexPWM0_SM1_EXTSYNC1_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn10ToFlexPwm0Sm1Extsync1           = 30U + (FlexPWM0_SM1_EXTSYNC1_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn11ToFlexPwm0Sm1Extsync1           = 31U + (FlexPWM0_SM1_EXTSYNC1_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio0PinEventTrig0ToFlexPwm0Sm1Extsync1 = 32U + (FlexPWM0_SM1_EXTSYNC1_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio1PinEventTrig0ToFlexPwm0Sm1Extsync1 = 33U + (FlexPWM0_SM1_EXTSYNC1_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio2PinEventTrig0ToFlexPwm0Sm1Extsync1 = 34U + (FlexPWM0_SM1_EXTSYNC1_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio3PinEventTrig0ToFlexPwm0Sm1Extsync1 = 35U + (FlexPWM0_SM1_EXTSYNC1_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio4PinEventTrig0ToFlexPwm0Sm1Extsync1 = 36U + (FlexPWM0_SM1_EXTSYNC1_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out0ToFlexPwm0Sm1Extsync1           = 37U + (FlexPWM0_SM1_EXTSYNC1_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out1ToFlexPwm0Sm1Extsync1           = 38U + (FlexPWM0_SM1_EXTSYNC1_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out2ToFlexPwm0Sm1Extsync1           = 39U + (FlexPWM0_SM1_EXTSYNC1_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out3ToFlexPwm0Sm1Extsync1           = 40U + (FlexPWM0_SM1_EXTSYNC1_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer3M2ToFlexPwm0Sm1Extsync1          = 45U + (FlexPWM0_SM1_EXTSYNC1_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer3M3ToFlexPwm0Sm1Extsync1          = 46U + (FlexPWM0_SM1_EXTSYNC1_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer4M2ToFlexPwm0Sm1Extsync1          = 47U + (FlexPWM0_SM1_EXTSYNC1_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer4M3ToFlexPwm0Sm1Extsync1          = 48U + (FlexPWM0_SM1_EXTSYNC1_REG << PMUX_SHIFT),
    kINPUTMUX_Qdc1CmpFlag0ToFlexPwm0Sm1Extsync1       = 49U + (FlexPWM0_SM1_EXTSYNC1_REG << PMUX_SHIFT),
    kINPUTMUX_Qdc1CmpFlag1ToFlexPwm0Sm1Extsync1       = 50U + (FlexPWM0_SM1_EXTSYNC1_REG << PMUX_SHIFT),
    kINPUTMUX_Qdc1CmpFlag2ToFlexPwm0Sm1Extsync1       = 51U + (FlexPWM0_SM1_EXTSYNC1_REG << PMUX_SHIFT),
    kINPUTMUX_Qdc1CmpFlag3ToFlexPwm0Sm1Extsync1       = 52U + (FlexPWM0_SM1_EXTSYNC1_REG << PMUX_SHIFT),
    kINPUTMUX_Qdc1PosMatch0ToFlexPwm0Sm1Extsync1      = 53U + (FlexPWM0_SM1_EXTSYNC1_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm1Sm0OutTrig0ToFlexPwm0Sm1Extsync1    = 54U + (FlexPWM0_SM1_EXTSYNC1_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm1Sm0OutTrig1ToFlexPwm0Sm1Extsync1    = 55U + (FlexPWM0_SM1_EXTSYNC1_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm1Sm1OutTrig0ToFlexPwm0Sm1Extsync1    = 56U + (FlexPWM0_SM1_EXTSYNC1_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm1Sm1OutTrig1ToFlexPwm0Sm1Extsync1    = 57U + (FlexPWM0_SM1_EXTSYNC1_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm1Sm2OutTrig0ToFlexPwm0Sm1Extsync1    = 58U + (FlexPWM0_SM1_EXTSYNC1_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm1Sm2OutTrig1ToFlexPwm0Sm1Extsync1    = 59U + (FlexPWM0_SM1_EXTSYNC1_REG << PMUX_SHIFT),

    /*!< FlexPWM0_SM2_EXTSYNC2 input trigger connections. */
    kINPUTMUX_ArmTxevToFlexPwm0Sm2Extsync2            = 1U + (FlexPWM0_SM2_EXTSYNC2_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out0ToFlexPwm0Sm2Extsync2           = 2U + (FlexPWM0_SM2_EXTSYNC2_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out1ToFlexPwm0Sm2Extsync2           = 3U + (FlexPWM0_SM2_EXTSYNC2_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out2ToFlexPwm0Sm2Extsync2           = 4U + (FlexPWM0_SM2_EXTSYNC2_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out3ToFlexPwm0Sm2Extsync2           = 5U + (FlexPWM0_SM2_EXTSYNC2_REG << PMUX_SHIFT),
    kINPUTMUX_Cmp0OutToFlexPwm0Sm2Extsync2            = 6U + (FlexPWM0_SM2_EXTSYNC2_REG << PMUX_SHIFT),
    kINPUTMUX_Cmp1OutToFlexPwm0Sm2Extsync2            = 7U + (FlexPWM0_SM2_EXTSYNC2_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer0M2ToFlexPwm0Sm2Extsync2          = 9U + (FlexPWM0_SM2_EXTSYNC2_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer0M3ToFlexPwm0Sm2Extsync2          = 10U + (FlexPWM0_SM2_EXTSYNC2_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer1M2ToFlexPwm0Sm2Extsync2          = 11U + (FlexPWM0_SM2_EXTSYNC2_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer1M3ToFlexPwm0Sm2Extsync2          = 12U + (FlexPWM0_SM2_EXTSYNC2_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer2M2ToFlexPwm0Sm2Extsync2          = 13U + (FlexPWM0_SM2_EXTSYNC2_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer2M3ToFlexPwm0Sm2Extsync2          = 14U + (FlexPWM0_SM2_EXTSYNC2_REG << PMUX_SHIFT),
    kINPUTMUX_Qdc0CmpFlag0ToFlexPwm0Sm2Extsync2       = 15U + (FlexPWM0_SM2_EXTSYNC2_REG << PMUX_SHIFT),
    kINPUTMUX_Qdc0CmpFlag1ToFlexPwm0Sm2Extsync2       = 16U + (FlexPWM0_SM2_EXTSYNC2_REG << PMUX_SHIFT),
    kINPUTMUX_Qdc0CmpFlag2ToFlexPwm0Sm2Extsync2       = 17U + (FlexPWM0_SM2_EXTSYNC2_REG << PMUX_SHIFT),
    kINPUTMUX_Qdc0CmpFlag3ToFlexPwm0Sm2Extsync2       = 18U + (FlexPWM0_SM2_EXTSYNC2_REG << PMUX_SHIFT),
    kINPUTMUX_Qdc0PosMatch0ToFlexPwm0Sm2Extsync2      = 19U + (FlexPWM0_SM2_EXTSYNC2_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn0ToFlexPwm0Sm2Extsync2            = 20U + (FlexPWM0_SM2_EXTSYNC2_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn1ToFlexPwm0Sm2Extsync2            = 21U + (FlexPWM0_SM2_EXTSYNC2_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn2ToFlexPwm0Sm2Extsync2            = 22U + (FlexPWM0_SM2_EXTSYNC2_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn3ToFlexPwm0Sm2Extsync2            = 23U + (FlexPWM0_SM2_EXTSYNC2_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn4ToFlexPwm0Sm2Extsync2            = 24U + (FlexPWM0_SM2_EXTSYNC2_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn5ToFlexPwm0Sm2Extsync2            = 25U + (FlexPWM0_SM2_EXTSYNC2_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn6ToFlexPwm0Sm2Extsync2            = 26U + (FlexPWM0_SM2_EXTSYNC2_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn7ToFlexPwm0Sm2Extsync2            = 27U + (FlexPWM0_SM2_EXTSYNC2_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn8ToFlexPwm0Sm2Extsync2            = 28U + (FlexPWM0_SM2_EXTSYNC2_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn9ToFlexPwm0Sm2Extsync2            = 29U + (FlexPWM0_SM2_EXTSYNC2_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn10ToFlexPwm0Sm2Extsync2           = 30U + (FlexPWM0_SM2_EXTSYNC2_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn11ToFlexPwm0Sm2Extsync2           = 31U + (FlexPWM0_SM2_EXTSYNC2_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio0PinEventTrig0ToFlexPwm0Sm2Extsync2 = 32U + (FlexPWM0_SM2_EXTSYNC2_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio1PinEventTrig0ToFlexPwm0Sm2Extsync2 = 33U + (FlexPWM0_SM2_EXTSYNC2_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio2PinEventTrig0ToFlexPwm0Sm2Extsync2 = 34U + (FlexPWM0_SM2_EXTSYNC2_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio3PinEventTrig0ToFlexPwm0Sm2Extsync2 = 35U + (FlexPWM0_SM2_EXTSYNC2_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio4PinEventTrig0ToFlexPwm0Sm2Extsync2 = 36U + (FlexPWM0_SM2_EXTSYNC2_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out0ToFlexPwm0Sm2Extsync2           = 37U + (FlexPWM0_SM2_EXTSYNC2_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out1ToFlexPwm0Sm2Extsync2           = 38U + (FlexPWM0_SM2_EXTSYNC2_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out2ToFlexPwm0Sm2Extsync2           = 39U + (FlexPWM0_SM2_EXTSYNC2_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out3ToFlexPwm0Sm2Extsync2           = 40U + (FlexPWM0_SM2_EXTSYNC2_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer3M2ToFlexPwm0Sm2Extsync2          = 45U + (FlexPWM0_SM2_EXTSYNC2_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer3M3ToFlexPwm0Sm2Extsync2          = 46U + (FlexPWM0_SM2_EXTSYNC2_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer4M2ToFlexPwm0Sm2Extsync2          = 47U + (FlexPWM0_SM2_EXTSYNC2_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer4M3ToFlexPwm0Sm2Extsync2          = 48U + (FlexPWM0_SM2_EXTSYNC2_REG << PMUX_SHIFT),
    kINPUTMUX_Qdc1CmpFlag0ToFlexPwm0Sm2Extsync2       = 49U + (FlexPWM0_SM2_EXTSYNC2_REG << PMUX_SHIFT),
    kINPUTMUX_Qdc1CmpFlag1ToFlexPwm0Sm2Extsync2       = 50U + (FlexPWM0_SM2_EXTSYNC2_REG << PMUX_SHIFT),
    kINPUTMUX_Qdc1CmpFlag2ToFlexPwm0Sm2Extsync2       = 51U + (FlexPWM0_SM2_EXTSYNC2_REG << PMUX_SHIFT),
    kINPUTMUX_Qdc1CmpFlag3ToFlexPwm0Sm2Extsync2       = 52U + (FlexPWM0_SM2_EXTSYNC2_REG << PMUX_SHIFT),
    kINPUTMUX_Qdc1PosMatch0ToFlexPwm0Sm2Extsync2      = 53U + (FlexPWM0_SM2_EXTSYNC2_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm1Sm0OutTrig0ToFlexPwm0Sm2Extsync2    = 54U + (FlexPWM0_SM2_EXTSYNC2_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm1Sm0OutTrig1ToFlexPwm0Sm2Extsync2    = 55U + (FlexPWM0_SM2_EXTSYNC2_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm1Sm1OutTrig0ToFlexPwm0Sm2Extsync2    = 56U + (FlexPWM0_SM2_EXTSYNC2_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm1Sm1OutTrig1ToFlexPwm0Sm2Extsync2    = 57U + (FlexPWM0_SM2_EXTSYNC2_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm1Sm2OutTrig0ToFlexPwm0Sm2Extsync2    = 58U + (FlexPWM0_SM2_EXTSYNC2_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm1Sm2OutTrig1ToFlexPwm0Sm2Extsync2    = 59U + (FlexPWM0_SM2_EXTSYNC2_REG << PMUX_SHIFT),

    /*!< FlexPWM0_FAULT input trigger connections. */
    kINPUTMUX_ArmTxevToFlexPwm0Fault            = 1U + (FlexPWM0_FAULT_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out0ToFlexPwm0Fault           = 2U + (FlexPWM0_FAULT_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out1ToFlexPwm0Fault           = 3U + (FlexPWM0_FAULT_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out2ToFlexPwm0Fault           = 4U + (FlexPWM0_FAULT_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out3ToFlexPwm0Fault           = 5U + (FlexPWM0_FAULT_REG << PMUX_SHIFT),
    kINPUTMUX_Cmp0OutToFlexPwm0Fault            = 6U + (FlexPWM0_FAULT_REG << PMUX_SHIFT),
    kINPUTMUX_Cmp1OutToFlexPwm0Fault            = 7U + (FlexPWM0_FAULT_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer0M2ToFlexPwm0Fault          = 9U + (FlexPWM0_FAULT_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer0M3ToFlexPwm0Fault          = 10U + (FlexPWM0_FAULT_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer1M2ToFlexPwm0Fault          = 11U + (FlexPWM0_FAULT_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer1M3ToFlexPwm0Fault          = 12U + (FlexPWM0_FAULT_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer2M2ToFlexPwm0Fault          = 13U + (FlexPWM0_FAULT_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer2M3ToFlexPwm0Fault          = 14U + (FlexPWM0_FAULT_REG << PMUX_SHIFT),
    kINPUTMUX_Qdc0CmpFlag0ToFlexPwm0Fault       = 15U + (FlexPWM0_FAULT_REG << PMUX_SHIFT),
    kINPUTMUX_Qdc0CmpFlag1ToFlexPwm0Fault       = 16U + (FlexPWM0_FAULT_REG << PMUX_SHIFT),
    kINPUTMUX_Qdc0CmpFlag2ToFlexPwm0Fault       = 17U + (FlexPWM0_FAULT_REG << PMUX_SHIFT),
    kINPUTMUX_Qdc0CmpFlag3ToFlexPwm0Fault       = 18U + (FlexPWM0_FAULT_REG << PMUX_SHIFT),
    kINPUTMUX_Qdc0PosMatch0ToFlexPwm0Fault      = 19U + (FlexPWM0_FAULT_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn0ToFlexPwm0Fault            = 20U + (FlexPWM0_FAULT_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn1ToFlexPwm0Fault            = 21U + (FlexPWM0_FAULT_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn2ToFlexPwm0Fault            = 22U + (FlexPWM0_FAULT_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn3ToFlexPwm0Fault            = 23U + (FlexPWM0_FAULT_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn4ToFlexPwm0Fault            = 24U + (FlexPWM0_FAULT_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn5ToFlexPwm0Fault            = 25U + (FlexPWM0_FAULT_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn6ToFlexPwm0Fault            = 26U + (FlexPWM0_FAULT_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn7ToFlexPwm0Fault            = 27U + (FlexPWM0_FAULT_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn8ToFlexPwm0Fault            = 28U + (FlexPWM0_FAULT_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn9ToFlexPwm0Fault            = 29U + (FlexPWM0_FAULT_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn10ToFlexPwm0Fault           = 30U + (FlexPWM0_FAULT_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn11ToFlexPwm0Fault           = 31U + (FlexPWM0_FAULT_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio0PinEventTrig0ToFlexPwm0Fault = 32U + (FlexPWM0_FAULT_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio1PinEventTrig0ToFlexPwm0Fault = 33U + (FlexPWM0_FAULT_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio2PinEventTrig0ToFlexPwm0Fault = 34U + (FlexPWM0_FAULT_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio3PinEventTrig0ToFlexPwm0Fault = 35U + (FlexPWM0_FAULT_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio4PinEventTrig0ToFlexPwm0Fault = 36U + (FlexPWM0_FAULT_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out0ToFlexPwm0Fault           = 37U + (FlexPWM0_FAULT_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out1ToFlexPwm0Fault           = 38U + (FlexPWM0_FAULT_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out2ToFlexPwm0Fault           = 39U + (FlexPWM0_FAULT_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out3ToFlexPwm0Fault           = 40U + (FlexPWM0_FAULT_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer3M2ToFlexPwm0Fault          = 45U + (FlexPWM0_FAULT_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer3M3ToFlexPwm0Fault          = 46U + (FlexPWM0_FAULT_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer4M2ToFlexPwm0Fault          = 47U + (FlexPWM0_FAULT_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer4M3ToFlexPwm0Fault          = 48U + (FlexPWM0_FAULT_REG << PMUX_SHIFT),
    kINPUTMUX_Qdc1CmpFlag0ToFlexPwm0Fault       = 49U + (FlexPWM0_FAULT_REG << PMUX_SHIFT),
    kINPUTMUX_Qdc1CmpFlag1ToFlexPwm0Fault       = 50U + (FlexPWM0_FAULT_REG << PMUX_SHIFT),
    kINPUTMUX_Qdc1CmpFlag2ToFlexPwm0Fault       = 51U + (FlexPWM0_FAULT_REG << PMUX_SHIFT),
    kINPUTMUX_Qdc1CmpFlag3ToFlexPwm0Fault       = 52U + (FlexPWM0_FAULT_REG << PMUX_SHIFT),
    kINPUTMUX_Qdc1PosMatch0ToFlexPwm0Fault      = 53U + (FlexPWM0_FAULT_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm1Sm0OutTrig0ToFlexPwm0Fault    = 54U + (FlexPWM0_FAULT_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm1Sm0OutTrig1ToFlexPwm0Fault    = 55U + (FlexPWM0_FAULT_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm1Sm1OutTrig0ToFlexPwm0Fault    = 56U + (FlexPWM0_FAULT_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm1Sm1OutTrig1ToFlexPwm0Fault    = 57U + (FlexPWM0_FAULT_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm1Sm2OutTrig0ToFlexPwm0Fault    = 58U + (FlexPWM0_FAULT_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm1Sm2OutTrig1ToFlexPwm0Fault    = 59U + (FlexPWM0_FAULT_REG << PMUX_SHIFT),

    /*!< FlexPWM0_FORCE input trigger connections. */
    kINPUTMUX_ArmTxevToFlexPwm0Force            = 1U + (FlexPWM0_FORCE_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out0ToFlexPwm0Force           = 2U + (FlexPWM0_FORCE_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out1ToFlexPwm0Force           = 3U + (FlexPWM0_FORCE_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out2ToFlexPwm0Force           = 4U + (FlexPWM0_FORCE_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out3ToFlexPwm0Force           = 5U + (FlexPWM0_FORCE_REG << PMUX_SHIFT),
    kINPUTMUX_Cmp0OutToFlexPwm0Force            = 6U + (FlexPWM0_FORCE_REG << PMUX_SHIFT),
    kINPUTMUX_Cmp1OutToFlexPwm0Force            = 7U + (FlexPWM0_FORCE_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer0M2ToFlexPwm0Force          = 9U + (FlexPWM0_FORCE_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer0M3ToFlexPwm0Force          = 10U + (FlexPWM0_FORCE_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer1M2ToFlexPwm0Force          = 11U + (FlexPWM0_FORCE_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer1M3ToFlexPwm0Force          = 12U + (FlexPWM0_FORCE_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer2M2ToFlexPwm0Force          = 13U + (FlexPWM0_FORCE_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer2M3ToFlexPwm0Force          = 14U + (FlexPWM0_FORCE_REG << PMUX_SHIFT),
    kINPUTMUX_Qdc0CmpFlag0ToFlexPwm0Force       = 15U + (FlexPWM0_FORCE_REG << PMUX_SHIFT),
    kINPUTMUX_Qdc0CmpFlag1ToFlexPwm0Force       = 16U + (FlexPWM0_FORCE_REG << PMUX_SHIFT),
    kINPUTMUX_Qdc0CmpFlag2ToFlexPwm0Force       = 17U + (FlexPWM0_FORCE_REG << PMUX_SHIFT),
    kINPUTMUX_Qdc0CmpFlag3ToFlexPwm0Force       = 18U + (FlexPWM0_FORCE_REG << PMUX_SHIFT),
    kINPUTMUX_Qdc0PosMatch0ToFlexPwm0Force      = 19U + (FlexPWM0_FORCE_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn0ToFlexPwm0Force            = 20U + (FlexPWM0_FORCE_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn1ToFlexPwm0Force            = 21U + (FlexPWM0_FORCE_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn2ToFlexPwm0Force            = 22U + (FlexPWM0_FORCE_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn3ToFlexPwm0Force            = 23U + (FlexPWM0_FORCE_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn4ToFlexPwm0Force            = 24U + (FlexPWM0_FORCE_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn5ToFlexPwm0Force            = 25U + (FlexPWM0_FORCE_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn6ToFlexPwm0Force            = 26U + (FlexPWM0_FORCE_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn7ToFlexPwm0Force            = 27U + (FlexPWM0_FORCE_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn8ToFlexPwm0Force            = 28U + (FlexPWM0_FORCE_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn9ToFlexPwm0Force            = 29U + (FlexPWM0_FORCE_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn10ToFlexPwm0Force           = 30U + (FlexPWM0_FORCE_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn11ToFlexPwm0Force           = 31U + (FlexPWM0_FORCE_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio0PinEventTrig0ToFlexPwm0Force = 32U + (FlexPWM0_FORCE_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio1PinEventTrig0ToFlexPwm0Force = 33U + (FlexPWM0_FORCE_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio2PinEventTrig0ToFlexPwm0Force = 34U + (FlexPWM0_FORCE_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio3PinEventTrig0ToFlexPwm0Force = 35U + (FlexPWM0_FORCE_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio4PinEventTrig0ToFlexPwm0Force = 36U + (FlexPWM0_FORCE_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out0ToFlexPwm0Force           = 37U + (FlexPWM0_FORCE_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out1ToFlexPwm0Force           = 38U + (FlexPWM0_FORCE_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out2ToFlexPwm0Force           = 39U + (FlexPWM0_FORCE_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out3ToFlexPwm0Force           = 40U + (FlexPWM0_FORCE_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer3M2ToFlexPwm0Force          = 45U + (FlexPWM0_FORCE_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer3M3ToFlexPwm0Force          = 46U + (FlexPWM0_FORCE_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer4M2ToFlexPwm0Force          = 47U + (FlexPWM0_FORCE_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer4M3ToFlexPwm0Force          = 48U + (FlexPWM0_FORCE_REG << PMUX_SHIFT),
    kINPUTMUX_Qdc1CmpFlag0ToFlexPwm0Force       = 49U + (FlexPWM0_FORCE_REG << PMUX_SHIFT),
    kINPUTMUX_Qdc1CmpFlag1ToFlexPwm0Force       = 50U + (FlexPWM0_FORCE_REG << PMUX_SHIFT),
    kINPUTMUX_Qdc1CmpFlag2ToFlexPwm0Force       = 51U + (FlexPWM0_FORCE_REG << PMUX_SHIFT),
    kINPUTMUX_Qdc1CmpFlag3ToFlexPwm0Force       = 52U + (FlexPWM0_FORCE_REG << PMUX_SHIFT),
    kINPUTMUX_Qdc1PosMatch0ToFlexPwm0Force      = 53U + (FlexPWM0_FORCE_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm1Sm0OutTrig0ToFlexPwm0Force    = 54U + (FlexPWM0_FORCE_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm1Sm0OutTrig1ToFlexPwm0Force    = 55U + (FlexPWM0_FORCE_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm1Sm1OutTrig0ToFlexPwm0Force    = 56U + (FlexPWM0_FORCE_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm1Sm1OutTrig1ToFlexPwm0Force    = 57U + (FlexPWM0_FORCE_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm1Sm2OutTrig0ToFlexPwm0Force    = 58U + (FlexPWM0_FORCE_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm1Sm2OutTrig1ToFlexPwm0Force    = 59U + (FlexPWM0_FORCE_REG << PMUX_SHIFT),

    /*!< FlexPWM1_SM0_EXTA0 input trigger connections. */
    kINPUTMUX_ArmTxevToFlexPwm1Sm0Exta0            = 1U + (FlexPWM1_SM0_EXTA0_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out0ToFlexPwm1Sm0Exta0           = 2U + (FlexPWM1_SM0_EXTA0_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out1ToFlexPwm1Sm0Exta0           = 3U + (FlexPWM1_SM0_EXTA0_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out2ToFlexPwm1Sm0Exta0           = 4U + (FlexPWM1_SM0_EXTA0_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out3ToFlexPwm1Sm0Exta0           = 5U + (FlexPWM1_SM0_EXTA0_REG << PMUX_SHIFT),
    kINPUTMUX_Cmp0OutToFlexPwm1Sm0Exta0            = 6U + (FlexPWM1_SM0_EXTA0_REG << PMUX_SHIFT),
    kINPUTMUX_Cmp1OutToFlexPwm1Sm0Exta0            = 7U + (FlexPWM1_SM0_EXTA0_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer0M2ToFlexPwm1Sm0Exta0          = 9U + (FlexPWM1_SM0_EXTA0_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer0M3ToFlexPwm1Sm0Exta0          = 10U + (FlexPWM1_SM0_EXTA0_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer1M2ToFlexPwm1Sm0Exta0          = 11U + (FlexPWM1_SM0_EXTA0_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer1M3ToFlexPwm1Sm0Exta0          = 12U + (FlexPWM1_SM0_EXTA0_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer2M2ToFlexPwm1Sm0Exta0          = 13U + (FlexPWM1_SM0_EXTA0_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer2M3ToFlexPwm1Sm0Exta0          = 14U + (FlexPWM1_SM0_EXTA0_REG << PMUX_SHIFT),
    kINPUTMUX_Qdc0CmpFlag0ToFlexPwm1Sm0Exta0       = 15U + (FlexPWM1_SM0_EXTA0_REG << PMUX_SHIFT),
    kINPUTMUX_Qdc0CmpFlag1ToFlexPwm1Sm0Exta0       = 16U + (FlexPWM1_SM0_EXTA0_REG << PMUX_SHIFT),
    kINPUTMUX_Qdc0CmpFlag2ToFlexPwm1Sm0Exta0       = 17U + (FlexPWM1_SM0_EXTA0_REG << PMUX_SHIFT),
    kINPUTMUX_Qdc0CmpFlag3ToFlexPwm1Sm0Exta0       = 18U + (FlexPWM1_SM0_EXTA0_REG << PMUX_SHIFT),
    kINPUTMUX_Qdc0PosMatch0ToFlexPwm1Sm0Exta0      = 19U + (FlexPWM1_SM0_EXTA0_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn0ToFlexPwm1Sm0Exta0            = 20U + (FlexPWM1_SM0_EXTA0_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn1ToFlexPwm1Sm0Exta0            = 21U + (FlexPWM1_SM0_EXTA0_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn2ToFlexPwm1Sm0Exta0            = 22U + (FlexPWM1_SM0_EXTA0_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn3ToFlexPwm1Sm0Exta0            = 23U + (FlexPWM1_SM0_EXTA0_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn4ToFlexPwm1Sm0Exta0            = 24U + (FlexPWM1_SM0_EXTA0_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn5ToFlexPwm1Sm0Exta0            = 25U + (FlexPWM1_SM0_EXTA0_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn6ToFlexPwm1Sm0Exta0            = 26U + (FlexPWM1_SM0_EXTA0_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn7ToFlexPwm1Sm0Exta0            = 27U + (FlexPWM1_SM0_EXTA0_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn8ToFlexPwm1Sm0Exta0            = 28U + (FlexPWM1_SM0_EXTA0_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn9ToFlexPwm1Sm0Exta0            = 29U + (FlexPWM1_SM0_EXTA0_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn10ToFlexPwm1Sm0Exta0           = 30U + (FlexPWM1_SM0_EXTA0_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn11ToFlexPwm1Sm0Exta0           = 31U + (FlexPWM1_SM0_EXTA0_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio0PinEventTrig0ToFlexPwm1Sm0Exta0 = 32U + (FlexPWM1_SM0_EXTA0_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio1PinEventTrig0ToFlexPwm1Sm0Exta0 = 33U + (FlexPWM1_SM0_EXTA0_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio2PinEventTrig0ToFlexPwm1Sm0Exta0 = 34U + (FlexPWM1_SM0_EXTA0_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio3PinEventTrig0ToFlexPwm1Sm0Exta0 = 35U + (FlexPWM1_SM0_EXTA0_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio4PinEventTrig0ToFlexPwm1Sm0Exta0 = 36U + (FlexPWM1_SM0_EXTA0_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out0ToFlexPwm1Sm0Exta0           = 37U + (FlexPWM1_SM0_EXTA0_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out1ToFlexPwm1Sm0Exta0           = 38U + (FlexPWM1_SM0_EXTA0_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out2ToFlexPwm1Sm0Exta0           = 39U + (FlexPWM1_SM0_EXTA0_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out3ToFlexPwm1Sm0Exta0           = 40U + (FlexPWM1_SM0_EXTA0_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer3M2ToFlexPwm1Sm0Exta0          = 45U + (FlexPWM1_SM0_EXTA0_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer3M3ToFlexPwm1Sm0Exta0          = 46U + (FlexPWM1_SM0_EXTA0_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer4M2ToFlexPwm1Sm0Exta0          = 47U + (FlexPWM1_SM0_EXTA0_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer4M3ToFlexPwm1Sm0Exta0          = 48U + (FlexPWM1_SM0_EXTA0_REG << PMUX_SHIFT),
    kINPUTMUX_Qdc1CmpFlag0ToFlexPwm1Sm0Exta0       = 49U + (FlexPWM1_SM0_EXTA0_REG << PMUX_SHIFT),
    kINPUTMUX_Qdc1CmpFlag1ToFlexPwm1Sm0Exta0       = 50U + (FlexPWM1_SM0_EXTA0_REG << PMUX_SHIFT),
    kINPUTMUX_Qdc1CmpFlag2ToFlexPwm1Sm0Exta0       = 51U + (FlexPWM1_SM0_EXTA0_REG << PMUX_SHIFT),
    kINPUTMUX_Qdc1CmpFlag3ToFlexPwm1Sm0Exta0       = 52U + (FlexPWM1_SM0_EXTA0_REG << PMUX_SHIFT),
    kINPUTMUX_Qdc1PosMatch0ToFlexPwm1Sm0Exta0      = 53U + (FlexPWM1_SM0_EXTA0_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm0OutTrig0ToFlexPwm1Sm0Exta0    = 54U + (FlexPWM1_SM0_EXTA0_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm0OutTrig1ToFlexPwm1Sm0Exta0    = 55U + (FlexPWM1_SM0_EXTA0_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm1OutTrig0ToFlexPwm1Sm0Exta0    = 56U + (FlexPWM1_SM0_EXTA0_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm1OutTrig1ToFlexPwm1Sm0Exta0    = 57U + (FlexPWM1_SM0_EXTA0_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm2OutTrig0ToFlexPwm1Sm0Exta0    = 58U + (FlexPWM1_SM0_EXTA0_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm2OutTrig1ToFlexPwm1Sm0Exta0    = 59U + (FlexPWM1_SM0_EXTA0_REG << PMUX_SHIFT),

    /*!< FlexPWM1_SM1_EXTA1 input trigger connections. */
    kINPUTMUX_ArmTxevToFlexPwm1Sm1Exta1            = 1U + (FlexPWM1_SM1_EXTA1_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out0ToFlexPwm1Sm1Exta1           = 2U + (FlexPWM1_SM1_EXTA1_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out1ToFlexPwm1Sm1Exta1           = 3U + (FlexPWM1_SM1_EXTA1_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out2ToFlexPwm1Sm1Exta1           = 4U + (FlexPWM1_SM1_EXTA1_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out3ToFlexPwm1Sm1Exta1           = 5U + (FlexPWM1_SM1_EXTA1_REG << PMUX_SHIFT),
    kINPUTMUX_Cmp0OutToFlexPwm1Sm1Exta1            = 6U + (FlexPWM1_SM1_EXTA1_REG << PMUX_SHIFT),
    kINPUTMUX_Cmp1OutToFlexPwm1Sm1Exta1            = 7U + (FlexPWM1_SM1_EXTA1_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer0M2ToFlexPwm1Sm1Exta1          = 9U + (FlexPWM1_SM1_EXTA1_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer0M3ToFlexPwm1Sm1Exta1          = 10U + (FlexPWM1_SM1_EXTA1_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer1M2ToFlexPwm1Sm1Exta1          = 11U + (FlexPWM1_SM1_EXTA1_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer1M3ToFlexPwm1Sm1Exta1          = 12U + (FlexPWM1_SM1_EXTA1_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer2M2ToFlexPwm1Sm1Exta1          = 13U + (FlexPWM1_SM1_EXTA1_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer2M3ToFlexPwm1Sm1Exta1          = 14U + (FlexPWM1_SM1_EXTA1_REG << PMUX_SHIFT),
    kINPUTMUX_Qdc0CmpFlag0ToFlexPwm1Sm1Exta1       = 15U + (FlexPWM1_SM1_EXTA1_REG << PMUX_SHIFT),
    kINPUTMUX_Qdc0CmpFlag1ToFlexPwm1Sm1Exta1       = 16U + (FlexPWM1_SM1_EXTA1_REG << PMUX_SHIFT),
    kINPUTMUX_Qdc0CmpFlag2ToFlexPwm1Sm1Exta1       = 17U + (FlexPWM1_SM1_EXTA1_REG << PMUX_SHIFT),
    kINPUTMUX_Qdc0CmpFlag3ToFlexPwm1Sm1Exta1       = 18U + (FlexPWM1_SM1_EXTA1_REG << PMUX_SHIFT),
    kINPUTMUX_Qdc0PosMatch0ToFlexPwm1Sm1Exta1      = 19U + (FlexPWM1_SM1_EXTA1_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn0ToFlexPwm1Sm1Exta1            = 20U + (FlexPWM1_SM1_EXTA1_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn1ToFlexPwm1Sm1Exta1            = 21U + (FlexPWM1_SM1_EXTA1_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn2ToFlexPwm1Sm1Exta1            = 22U + (FlexPWM1_SM1_EXTA1_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn3ToFlexPwm1Sm1Exta1            = 23U + (FlexPWM1_SM1_EXTA1_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn4ToFlexPwm1Sm1Exta1            = 24U + (FlexPWM1_SM1_EXTA1_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn5ToFlexPwm1Sm1Exta1            = 25U + (FlexPWM1_SM1_EXTA1_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn6ToFlexPwm1Sm1Exta1            = 26U + (FlexPWM1_SM1_EXTA1_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn7ToFlexPwm1Sm1Exta1            = 27U + (FlexPWM1_SM1_EXTA1_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn8ToFlexPwm1Sm1Exta1            = 28U + (FlexPWM1_SM1_EXTA1_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn9ToFlexPwm1Sm1Exta1            = 29U + (FlexPWM1_SM1_EXTA1_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn10ToFlexPwm1Sm1Exta1           = 30U + (FlexPWM1_SM1_EXTA1_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn11ToFlexPwm1Sm1Exta1           = 31U + (FlexPWM1_SM1_EXTA1_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio0PinEventTrig0ToFlexPwm1Sm1Exta1 = 32U + (FlexPWM1_SM1_EXTA1_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio1PinEventTrig0ToFlexPwm1Sm1Exta1 = 33U + (FlexPWM1_SM1_EXTA1_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio2PinEventTrig0ToFlexPwm1Sm1Exta1 = 34U + (FlexPWM1_SM1_EXTA1_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio3PinEventTrig0ToFlexPwm1Sm1Exta1 = 35U + (FlexPWM1_SM1_EXTA1_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio4PinEventTrig0ToFlexPwm1Sm1Exta1 = 36U + (FlexPWM1_SM1_EXTA1_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out0ToFlexPwm1Sm1Exta1           = 37U + (FlexPWM1_SM1_EXTA1_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out1ToFlexPwm1Sm1Exta1           = 38U + (FlexPWM1_SM1_EXTA1_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out2ToFlexPwm1Sm1Exta1           = 39U + (FlexPWM1_SM1_EXTA1_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out3ToFlexPwm1Sm1Exta1           = 40U + (FlexPWM1_SM1_EXTA1_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer3M2ToFlexPwm1Sm1Exta1          = 45U + (FlexPWM1_SM1_EXTA1_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer3M3ToFlexPwm1Sm1Exta1          = 46U + (FlexPWM1_SM1_EXTA1_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer4M2ToFlexPwm1Sm1Exta1          = 47U + (FlexPWM1_SM1_EXTA1_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer4M3ToFlexPwm1Sm1Exta1          = 48U + (FlexPWM1_SM1_EXTA1_REG << PMUX_SHIFT),
    kINPUTMUX_Qdc1CmpFlag0ToFlexPwm1Sm1Exta1       = 49U + (FlexPWM1_SM1_EXTA1_REG << PMUX_SHIFT),
    kINPUTMUX_Qdc1CmpFlag1ToFlexPwm1Sm1Exta1       = 50U + (FlexPWM1_SM1_EXTA1_REG << PMUX_SHIFT),
    kINPUTMUX_Qdc1CmpFlag2ToFlexPwm1Sm1Exta1       = 51U + (FlexPWM1_SM1_EXTA1_REG << PMUX_SHIFT),
    kINPUTMUX_Qdc1CmpFlag3ToFlexPwm1Sm1Exta1       = 52U + (FlexPWM1_SM1_EXTA1_REG << PMUX_SHIFT),
    kINPUTMUX_Qdc1PosMatch0ToFlexPwm1Sm1Exta1      = 53U + (FlexPWM1_SM1_EXTA1_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm0OutTrig0ToFlexPwm1Sm1Exta1    = 54U + (FlexPWM1_SM1_EXTA1_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm0OutTrig1ToFlexPwm1Sm1Exta1    = 55U + (FlexPWM1_SM1_EXTA1_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm1OutTrig0ToFlexPwm1Sm1Exta1    = 56U + (FlexPWM1_SM1_EXTA1_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm1OutTrig1ToFlexPwm1Sm1Exta1    = 57U + (FlexPWM1_SM1_EXTA1_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm2OutTrig0ToFlexPwm1Sm1Exta1    = 58U + (FlexPWM1_SM1_EXTA1_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm2OutTrig1ToFlexPwm1Sm1Exta1    = 59U + (FlexPWM1_SM1_EXTA1_REG << PMUX_SHIFT),

    /*!< FlexPWM1_SM2_EXTA2 input trigger connections. */
    kINPUTMUX_ArmTxevToFlexPwm1Sm2Exta2            = 1U + (FlexPWM1_SM2_EXTA2_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out0ToFlexPwm1Sm2Exta2           = 2U + (FlexPWM1_SM2_EXTA2_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out1ToFlexPwm1Sm2Exta2           = 3U + (FlexPWM1_SM2_EXTA2_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out2ToFlexPwm1Sm2Exta2           = 4U + (FlexPWM1_SM2_EXTA2_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out3ToFlexPwm1Sm2Exta2           = 5U + (FlexPWM1_SM2_EXTA2_REG << PMUX_SHIFT),
    kINPUTMUX_Cmp0OutToFlexPwm1Sm2Exta2            = 6U + (FlexPWM1_SM2_EXTA2_REG << PMUX_SHIFT),
    kINPUTMUX_Cmp1OutToFlexPwm1Sm2Exta2            = 7U + (FlexPWM1_SM2_EXTA2_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer0M2ToFlexPwm1Sm2Exta2          = 9U + (FlexPWM1_SM2_EXTA2_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer0M3ToFlexPwm1Sm2Exta2          = 10U + (FlexPWM1_SM2_EXTA2_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer1M2ToFlexPwm1Sm2Exta2          = 11U + (FlexPWM1_SM2_EXTA2_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer1M3ToFlexPwm1Sm2Exta2          = 12U + (FlexPWM1_SM2_EXTA2_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer2M2ToFlexPwm1Sm2Exta2          = 13U + (FlexPWM1_SM2_EXTA2_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer2M3ToFlexPwm1Sm2Exta2          = 14U + (FlexPWM1_SM2_EXTA2_REG << PMUX_SHIFT),
    kINPUTMUX_Qdc0CmpFlag0ToFlexPwm1Sm2Exta2       = 15U + (FlexPWM1_SM2_EXTA2_REG << PMUX_SHIFT),
    kINPUTMUX_Qdc0CmpFlag1ToFlexPwm1Sm2Exta2       = 16U + (FlexPWM1_SM2_EXTA2_REG << PMUX_SHIFT),
    kINPUTMUX_Qdc0CmpFlag2ToFlexPwm1Sm2Exta2       = 17U + (FlexPWM1_SM2_EXTA2_REG << PMUX_SHIFT),
    kINPUTMUX_Qdc0CmpFlag3ToFlexPwm1Sm2Exta2       = 18U + (FlexPWM1_SM2_EXTA2_REG << PMUX_SHIFT),
    kINPUTMUX_Qdc0PosMatch0ToFlexPwm1Sm2Exta2      = 19U + (FlexPWM1_SM2_EXTA2_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn0ToFlexPwm1Sm2Exta2            = 20U + (FlexPWM1_SM2_EXTA2_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn1ToFlexPwm1Sm2Exta2            = 21U + (FlexPWM1_SM2_EXTA2_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn2ToFlexPwm1Sm2Exta2            = 22U + (FlexPWM1_SM2_EXTA2_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn3ToFlexPwm1Sm2Exta2            = 23U + (FlexPWM1_SM2_EXTA2_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn4ToFlexPwm1Sm2Exta2            = 24U + (FlexPWM1_SM2_EXTA2_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn5ToFlexPwm1Sm2Exta2            = 25U + (FlexPWM1_SM2_EXTA2_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn6ToFlexPwm1Sm2Exta2            = 26U + (FlexPWM1_SM2_EXTA2_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn7ToFlexPwm1Sm2Exta2            = 27U + (FlexPWM1_SM2_EXTA2_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn8ToFlexPwm1Sm2Exta2            = 28U + (FlexPWM1_SM2_EXTA2_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn9ToFlexPwm1Sm2Exta2            = 29U + (FlexPWM1_SM2_EXTA2_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn10ToFlexPwm1Sm2Exta2           = 30U + (FlexPWM1_SM2_EXTA2_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn11ToFlexPwm1Sm2Exta2           = 31U + (FlexPWM1_SM2_EXTA2_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio0PinEventTrig0ToFlexPwm1Sm2Exta2 = 32U + (FlexPWM1_SM2_EXTA2_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio1PinEventTrig0ToFlexPwm1Sm2Exta2 = 33U + (FlexPWM1_SM2_EXTA2_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio2PinEventTrig0ToFlexPwm1Sm2Exta2 = 34U + (FlexPWM1_SM2_EXTA2_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio3PinEventTrig0ToFlexPwm1Sm2Exta2 = 35U + (FlexPWM1_SM2_EXTA2_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio4PinEventTrig0ToFlexPwm1Sm2Exta2 = 36U + (FlexPWM1_SM2_EXTA2_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out0ToFlexPwm1Sm2Exta2           = 37U + (FlexPWM1_SM2_EXTA2_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out1ToFlexPwm1Sm2Exta2           = 38U + (FlexPWM1_SM2_EXTA2_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out2ToFlexPwm1Sm2Exta2           = 39U + (FlexPWM1_SM2_EXTA2_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out3ToFlexPwm1Sm2Exta2           = 40U + (FlexPWM1_SM2_EXTA2_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer3M2ToFlexPwm1Sm2Exta2          = 45U + (FlexPWM1_SM2_EXTA2_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer3M3ToFlexPwm1Sm2Exta2          = 46U + (FlexPWM1_SM2_EXTA2_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer4M2ToFlexPwm1Sm2Exta2          = 47U + (FlexPWM1_SM2_EXTA2_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer4M3ToFlexPwm1Sm2Exta2          = 48U + (FlexPWM1_SM2_EXTA2_REG << PMUX_SHIFT),
    kINPUTMUX_Qdc1CmpFlag0ToFlexPwm1Sm2Exta2       = 49U + (FlexPWM1_SM2_EXTA2_REG << PMUX_SHIFT),
    kINPUTMUX_Qdc1CmpFlag1ToFlexPwm1Sm2Exta2       = 50U + (FlexPWM1_SM2_EXTA2_REG << PMUX_SHIFT),
    kINPUTMUX_Qdc1CmpFlag2ToFlexPwm1Sm2Exta2       = 51U + (FlexPWM1_SM2_EXTA2_REG << PMUX_SHIFT),
    kINPUTMUX_Qdc1CmpFlag3ToFlexPwm1Sm2Exta2       = 52U + (FlexPWM1_SM2_EXTA2_REG << PMUX_SHIFT),
    kINPUTMUX_Qdc1PosMatch0ToFlexPwm1Sm2Exta2      = 53U + (FlexPWM1_SM2_EXTA2_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm0OutTrig0ToFlexPwm1Sm2Exta2    = 54U + (FlexPWM1_SM2_EXTA2_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm0OutTrig1ToFlexPwm1Sm2Exta2    = 55U + (FlexPWM1_SM2_EXTA2_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm1OutTrig0ToFlexPwm1Sm2Exta2    = 56U + (FlexPWM1_SM2_EXTA2_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm1OutTrig1ToFlexPwm1Sm2Exta2    = 57U + (FlexPWM1_SM2_EXTA2_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm2OutTrig0ToFlexPwm1Sm2Exta2    = 58U + (FlexPWM1_SM2_EXTA2_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm2OutTrig1ToFlexPwm1Sm2Exta2    = 59U + (FlexPWM1_SM2_EXTA2_REG << PMUX_SHIFT),

    /*!< FlexPWM1_SM0_EXTSYNC0 input trigger connections. */
    kINPUTMUX_ArmTxevToFlexPwm1Sm0Extsync0            = 1U + (FlexPWM1_SM0_EXTSYNC0_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out0ToFlexPwm1Sm0Extsync0           = 2U + (FlexPWM1_SM0_EXTSYNC0_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out1ToFlexPwm1Sm0Extsync0           = 3U + (FlexPWM1_SM0_EXTSYNC0_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out2ToFlexPwm1Sm0Extsync0           = 4U + (FlexPWM1_SM0_EXTSYNC0_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out3ToFlexPwm1Sm0Extsync0           = 5U + (FlexPWM1_SM0_EXTSYNC0_REG << PMUX_SHIFT),
    kINPUTMUX_Cmp0OutToFlexPwm1Sm0Extsync0            = 6U + (FlexPWM1_SM0_EXTSYNC0_REG << PMUX_SHIFT),
    kINPUTMUX_Cmp1OutToFlexPwm1Sm0Extsync0            = 7U + (FlexPWM1_SM0_EXTSYNC0_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer0M2ToFlexPwm1Sm0Extsync0          = 9U + (FlexPWM1_SM0_EXTSYNC0_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer0M3ToFlexPwm1Sm0Extsync0          = 10U + (FlexPWM1_SM0_EXTSYNC0_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer1M2ToFlexPwm1Sm0Extsync0          = 11U + (FlexPWM1_SM0_EXTSYNC0_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer1M3ToFlexPwm1Sm0Extsync0          = 12U + (FlexPWM1_SM0_EXTSYNC0_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer2M2ToFlexPwm1Sm0Extsync0          = 13U + (FlexPWM1_SM0_EXTSYNC0_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer2M3ToFlexPwm1Sm0Extsync0          = 14U + (FlexPWM1_SM0_EXTSYNC0_REG << PMUX_SHIFT),
    kINPUTMUX_Qdc0CmpFlag0ToFlexPwm1Sm0Extsync0       = 15U + (FlexPWM1_SM0_EXTSYNC0_REG << PMUX_SHIFT),
    kINPUTMUX_Qdc0CmpFlag1ToFlexPwm1Sm0Extsync0       = 16U + (FlexPWM1_SM0_EXTSYNC0_REG << PMUX_SHIFT),
    kINPUTMUX_Qdc0CmpFlag2ToFlexPwm1Sm0Extsync0       = 17U + (FlexPWM1_SM0_EXTSYNC0_REG << PMUX_SHIFT),
    kINPUTMUX_Qdc0CmpFlag3ToFlexPwm1Sm0Extsync0       = 18U + (FlexPWM1_SM0_EXTSYNC0_REG << PMUX_SHIFT),
    kINPUTMUX_Qdc0PosMatch0ToFlexPwm1Sm0Extsync0      = 19U + (FlexPWM1_SM0_EXTSYNC0_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn0ToFlexPwm1Sm0Extsync0            = 20U + (FlexPWM1_SM0_EXTSYNC0_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn1ToFlexPwm1Sm0Extsync0            = 21U + (FlexPWM1_SM0_EXTSYNC0_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn2ToFlexPwm1Sm0Extsync0            = 22U + (FlexPWM1_SM0_EXTSYNC0_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn3ToFlexPwm1Sm0Extsync0            = 23U + (FlexPWM1_SM0_EXTSYNC0_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn4ToFlexPwm1Sm0Extsync0            = 24U + (FlexPWM1_SM0_EXTSYNC0_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn5ToFlexPwm1Sm0Extsync0            = 25U + (FlexPWM1_SM0_EXTSYNC0_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn6ToFlexPwm1Sm0Extsync0            = 26U + (FlexPWM1_SM0_EXTSYNC0_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn7ToFlexPwm1Sm0Extsync0            = 27U + (FlexPWM1_SM0_EXTSYNC0_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn8ToFlexPwm1Sm0Extsync0            = 28U + (FlexPWM1_SM0_EXTSYNC0_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn9ToFlexPwm1Sm0Extsync0            = 29U + (FlexPWM1_SM0_EXTSYNC0_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn10ToFlexPwm1Sm0Extsync0           = 30U + (FlexPWM1_SM0_EXTSYNC0_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn11ToFlexPwm1Sm0Extsync0           = 31U + (FlexPWM1_SM0_EXTSYNC0_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio0PinEventTrig0ToFlexPwm1Sm0Extsync0 = 32U + (FlexPWM1_SM0_EXTSYNC0_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio1PinEventTrig0ToFlexPwm1Sm0Extsync0 = 33U + (FlexPWM1_SM0_EXTSYNC0_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio2PinEventTrig0ToFlexPwm1Sm0Extsync0 = 34U + (FlexPWM1_SM0_EXTSYNC0_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio3PinEventTrig0ToFlexPwm1Sm0Extsync0 = 35U + (FlexPWM1_SM0_EXTSYNC0_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio4PinEventTrig0ToFlexPwm1Sm0Extsync0 = 36U + (FlexPWM1_SM0_EXTSYNC0_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out0ToFlexPwm1Sm0Extsync0           = 37U + (FlexPWM1_SM0_EXTSYNC0_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out1ToFlexPwm1Sm0Extsync0           = 38U + (FlexPWM1_SM0_EXTSYNC0_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out2ToFlexPwm1Sm0Extsync0           = 39U + (FlexPWM1_SM0_EXTSYNC0_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out3ToFlexPwm1Sm0Extsync0           = 40U + (FlexPWM1_SM0_EXTSYNC0_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer3M2ToFlexPwm1Sm0Extsync0          = 45U + (FlexPWM1_SM0_EXTSYNC0_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer3M3ToFlexPwm1Sm0Extsync0          = 46U + (FlexPWM1_SM0_EXTSYNC0_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer4M2ToFlexPwm1Sm0Extsync0          = 47U + (FlexPWM1_SM0_EXTSYNC0_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer4M3ToFlexPwm1Sm0Extsync0          = 48U + (FlexPWM1_SM0_EXTSYNC0_REG << PMUX_SHIFT),
    kINPUTMUX_Qdc1CmpFlag0ToFlexPwm1Sm0Extsync0       = 49U + (FlexPWM1_SM0_EXTSYNC0_REG << PMUX_SHIFT),
    kINPUTMUX_Qdc1CmpFlag1ToFlexPwm1Sm0Extsync0       = 50U + (FlexPWM1_SM0_EXTSYNC0_REG << PMUX_SHIFT),
    kINPUTMUX_Qdc1CmpFlag2ToFlexPwm1Sm0Extsync0       = 51U + (FlexPWM1_SM0_EXTSYNC0_REG << PMUX_SHIFT),
    kINPUTMUX_Qdc1CmpFlag3ToFlexPwm1Sm0Extsync0       = 52U + (FlexPWM1_SM0_EXTSYNC0_REG << PMUX_SHIFT),
    kINPUTMUX_Qdc1PosMatch0ToFlexPwm1Sm0Extsync0      = 53U + (FlexPWM1_SM0_EXTSYNC0_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm0OutTrig0ToFlexPwm1Sm0Extsync0    = 54U + (FlexPWM1_SM0_EXTSYNC0_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm0OutTrig1ToFlexPwm1Sm0Extsync0    = 55U + (FlexPWM1_SM0_EXTSYNC0_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm1OutTrig0ToFlexPwm1Sm0Extsync0    = 56U + (FlexPWM1_SM0_EXTSYNC0_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm1OutTrig1ToFlexPwm1Sm0Extsync0    = 57U + (FlexPWM1_SM0_EXTSYNC0_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm2OutTrig0ToFlexPwm1Sm0Extsync0    = 58U + (FlexPWM1_SM0_EXTSYNC0_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm2OutTrig1ToFlexPwm1Sm0Extsync0    = 59U + (FlexPWM1_SM0_EXTSYNC0_REG << PMUX_SHIFT),

    /*!< FlexPWM1_SM1_EXTSYNC1 input trigger connections. */
    kINPUTMUX_ArmTxevToFlexPwm1Sm1Extsync1            = 1U + (FlexPWM1_SM1_EXTSYNC1_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out0ToFlexPwm1Sm1Extsync1           = 2U + (FlexPWM1_SM1_EXTSYNC1_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out1ToFlexPwm1Sm1Extsync1           = 3U + (FlexPWM1_SM1_EXTSYNC1_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out2ToFlexPwm1Sm1Extsync1           = 4U + (FlexPWM1_SM1_EXTSYNC1_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out3ToFlexPwm1Sm1Extsync1           = 5U + (FlexPWM1_SM1_EXTSYNC1_REG << PMUX_SHIFT),
    kINPUTMUX_Cmp0OutToFlexPwm1Sm1Extsync1            = 6U + (FlexPWM1_SM1_EXTSYNC1_REG << PMUX_SHIFT),
    kINPUTMUX_Cmp1OutToFlexPwm1Sm1Extsync1            = 7U + (FlexPWM1_SM1_EXTSYNC1_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer0M2ToFlexPwm1Sm1Extsync1          = 9U + (FlexPWM1_SM1_EXTSYNC1_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer0M3ToFlexPwm1Sm1Extsync1          = 10U + (FlexPWM1_SM1_EXTSYNC1_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer1M2ToFlexPwm1Sm1Extsync1          = 11U + (FlexPWM1_SM1_EXTSYNC1_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer1M3ToFlexPwm1Sm1Extsync1          = 12U + (FlexPWM1_SM1_EXTSYNC1_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer2M2ToFlexPwm1Sm1Extsync1          = 13U + (FlexPWM1_SM1_EXTSYNC1_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer2M3ToFlexPwm1Sm1Extsync1          = 14U + (FlexPWM1_SM1_EXTSYNC1_REG << PMUX_SHIFT),
    kINPUTMUX_Qdc0CmpFlag0ToFlexPwm1Sm1Extsync1       = 15U + (FlexPWM1_SM1_EXTSYNC1_REG << PMUX_SHIFT),
    kINPUTMUX_Qdc0CmpFlag1ToFlexPwm1Sm1Extsync1       = 16U + (FlexPWM1_SM1_EXTSYNC1_REG << PMUX_SHIFT),
    kINPUTMUX_Qdc0CmpFlag2ToFlexPwm1Sm1Extsync1       = 17U + (FlexPWM1_SM1_EXTSYNC1_REG << PMUX_SHIFT),
    kINPUTMUX_Qdc0CmpFlag3ToFlexPwm1Sm1Extsync1       = 18U + (FlexPWM1_SM1_EXTSYNC1_REG << PMUX_SHIFT),
    kINPUTMUX_Qdc0PosMatch0ToFlexPwm1Sm1Extsync1      = 19U + (FlexPWM1_SM1_EXTSYNC1_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn0ToFlexPwm1Sm1Extsync1            = 20U + (FlexPWM1_SM1_EXTSYNC1_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn1ToFlexPwm1Sm1Extsync1            = 21U + (FlexPWM1_SM1_EXTSYNC1_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn2ToFlexPwm1Sm1Extsync1            = 22U + (FlexPWM1_SM1_EXTSYNC1_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn3ToFlexPwm1Sm1Extsync1            = 23U + (FlexPWM1_SM1_EXTSYNC1_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn4ToFlexPwm1Sm1Extsync1            = 24U + (FlexPWM1_SM1_EXTSYNC1_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn5ToFlexPwm1Sm1Extsync1            = 25U + (FlexPWM1_SM1_EXTSYNC1_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn6ToFlexPwm1Sm1Extsync1            = 26U + (FlexPWM1_SM1_EXTSYNC1_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn7ToFlexPwm1Sm1Extsync1            = 27U + (FlexPWM1_SM1_EXTSYNC1_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn8ToFlexPwm1Sm1Extsync1            = 28U + (FlexPWM1_SM1_EXTSYNC1_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn9ToFlexPwm1Sm1Extsync1            = 29U + (FlexPWM1_SM1_EXTSYNC1_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn10ToFlexPwm1Sm1Extsync1           = 30U + (FlexPWM1_SM1_EXTSYNC1_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn11ToFlexPwm1Sm1Extsync1           = 31U + (FlexPWM1_SM1_EXTSYNC1_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio0PinEventTrig0ToFlexPwm1Sm1Extsync1 = 32U + (FlexPWM1_SM1_EXTSYNC1_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio1PinEventTrig0ToFlexPwm1Sm1Extsync1 = 33U + (FlexPWM1_SM1_EXTSYNC1_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio2PinEventTrig0ToFlexPwm1Sm1Extsync1 = 34U + (FlexPWM1_SM1_EXTSYNC1_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio3PinEventTrig0ToFlexPwm1Sm1Extsync1 = 35U + (FlexPWM1_SM1_EXTSYNC1_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio4PinEventTrig0ToFlexPwm1Sm1Extsync1 = 36U + (FlexPWM1_SM1_EXTSYNC1_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out0ToFlexPwm1Sm1Extsync1           = 37U + (FlexPWM1_SM1_EXTSYNC1_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out1ToFlexPwm1Sm1Extsync1           = 38U + (FlexPWM1_SM1_EXTSYNC1_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out2ToFlexPwm1Sm1Extsync1           = 39U + (FlexPWM1_SM1_EXTSYNC1_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out3ToFlexPwm1Sm1Extsync1           = 40U + (FlexPWM1_SM1_EXTSYNC1_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer3M2ToFlexPwm1Sm1Extsync1          = 45U + (FlexPWM1_SM1_EXTSYNC1_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer3M3ToFlexPwm1Sm1Extsync1          = 46U + (FlexPWM1_SM1_EXTSYNC1_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer4M2ToFlexPwm1Sm1Extsync1          = 47U + (FlexPWM1_SM1_EXTSYNC1_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer4M3ToFlexPwm1Sm1Extsync1          = 48U + (FlexPWM1_SM1_EXTSYNC1_REG << PMUX_SHIFT),
    kINPUTMUX_Qdc1CmpFlag0ToFlexPwm1Sm1Extsync1       = 49U + (FlexPWM1_SM1_EXTSYNC1_REG << PMUX_SHIFT),
    kINPUTMUX_Qdc1CmpFlag1ToFlexPwm1Sm1Extsync1       = 50U + (FlexPWM1_SM1_EXTSYNC1_REG << PMUX_SHIFT),
    kINPUTMUX_Qdc1CmpFlag2ToFlexPwm1Sm1Extsync1       = 51U + (FlexPWM1_SM1_EXTSYNC1_REG << PMUX_SHIFT),
    kINPUTMUX_Qdc1CmpFlag3ToFlexPwm1Sm1Extsync1       = 52U + (FlexPWM1_SM1_EXTSYNC1_REG << PMUX_SHIFT),
    kINPUTMUX_Qdc1PosMatch0ToFlexPwm1Sm1Extsync1      = 53U + (FlexPWM1_SM1_EXTSYNC1_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm0OutTrig0ToFlexPwm1Sm1Extsync1    = 54U + (FlexPWM1_SM1_EXTSYNC1_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm0OutTrig1ToFlexPwm1Sm1Extsync1    = 55U + (FlexPWM1_SM1_EXTSYNC1_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm1OutTrig0ToFlexPwm1Sm1Extsync1    = 56U + (FlexPWM1_SM1_EXTSYNC1_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm1OutTrig1ToFlexPwm1Sm1Extsync1    = 57U + (FlexPWM1_SM1_EXTSYNC1_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm2OutTrig0ToFlexPwm1Sm1Extsync1    = 58U + (FlexPWM1_SM1_EXTSYNC1_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm2OutTrig1ToFlexPwm1Sm1Extsync1    = 59U + (FlexPWM1_SM1_EXTSYNC1_REG << PMUX_SHIFT),

    /*!< FlexPWM1_SM2_EXTSYNC2 input trigger connections. */
    kINPUTMUX_ArmTxevToFlexPwm1Sm2Extsync2            = 1U + (FlexPWM1_SM2_EXTSYNC2_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out0ToFlexPwm1Sm2Extsync2           = 2U + (FlexPWM1_SM2_EXTSYNC2_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out1ToFlexPwm1Sm2Extsync2           = 3U + (FlexPWM1_SM2_EXTSYNC2_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out2ToFlexPwm1Sm2Extsync2           = 4U + (FlexPWM1_SM2_EXTSYNC2_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out3ToFlexPwm1Sm2Extsync2           = 5U + (FlexPWM1_SM2_EXTSYNC2_REG << PMUX_SHIFT),
    kINPUTMUX_Cmp0OutToFlexPwm1Sm2Extsync2            = 6U + (FlexPWM1_SM2_EXTSYNC2_REG << PMUX_SHIFT),
    kINPUTMUX_Cmp1OutToFlexPwm1Sm2Extsync2            = 7U + (FlexPWM1_SM2_EXTSYNC2_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer0M2ToFlexPwm1Sm2Extsync2          = 9U + (FlexPWM1_SM2_EXTSYNC2_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer0M3ToFlexPwm1Sm2Extsync2          = 10U + (FlexPWM1_SM2_EXTSYNC2_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer1M2ToFlexPwm1Sm2Extsync2          = 11U + (FlexPWM1_SM2_EXTSYNC2_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer1M3ToFlexPwm1Sm2Extsync2          = 12U + (FlexPWM1_SM2_EXTSYNC2_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer2M2ToFlexPwm1Sm2Extsync2          = 13U + (FlexPWM1_SM2_EXTSYNC2_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer2M3ToFlexPwm1Sm2Extsync2          = 14U + (FlexPWM1_SM2_EXTSYNC2_REG << PMUX_SHIFT),
    kINPUTMUX_Qdc0CmpFlag0ToFlexPwm1Sm2Extsync2       = 15U + (FlexPWM1_SM2_EXTSYNC2_REG << PMUX_SHIFT),
    kINPUTMUX_Qdc0CmpFlag1ToFlexPwm1Sm2Extsync2       = 16U + (FlexPWM1_SM2_EXTSYNC2_REG << PMUX_SHIFT),
    kINPUTMUX_Qdc0CmpFlag2ToFlexPwm1Sm2Extsync2       = 17U + (FlexPWM1_SM2_EXTSYNC2_REG << PMUX_SHIFT),
    kINPUTMUX_Qdc0CmpFlag3ToFlexPwm1Sm2Extsync2       = 18U + (FlexPWM1_SM2_EXTSYNC2_REG << PMUX_SHIFT),
    kINPUTMUX_Qdc0PosMatch0ToFlexPwm1Sm2Extsync2      = 19U + (FlexPWM1_SM2_EXTSYNC2_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn0ToFlexPwm1Sm2Extsync2            = 20U + (FlexPWM1_SM2_EXTSYNC2_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn1ToFlexPwm1Sm2Extsync2            = 21U + (FlexPWM1_SM2_EXTSYNC2_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn2ToFlexPwm1Sm2Extsync2            = 22U + (FlexPWM1_SM2_EXTSYNC2_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn3ToFlexPwm1Sm2Extsync2            = 23U + (FlexPWM1_SM2_EXTSYNC2_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn4ToFlexPwm1Sm2Extsync2            = 24U + (FlexPWM1_SM2_EXTSYNC2_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn5ToFlexPwm1Sm2Extsync2            = 25U + (FlexPWM1_SM2_EXTSYNC2_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn6ToFlexPwm1Sm2Extsync2            = 26U + (FlexPWM1_SM2_EXTSYNC2_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn7ToFlexPwm1Sm2Extsync2            = 27U + (FlexPWM1_SM2_EXTSYNC2_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn8ToFlexPwm1Sm2Extsync2            = 28U + (FlexPWM1_SM2_EXTSYNC2_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn9ToFlexPwm1Sm2Extsync2            = 29U + (FlexPWM1_SM2_EXTSYNC2_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn10ToFlexPwm1Sm2Extsync2           = 30U + (FlexPWM1_SM2_EXTSYNC2_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn11ToFlexPwm1Sm2Extsync2           = 31U + (FlexPWM1_SM2_EXTSYNC2_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio0PinEventTrig0ToFlexPwm1Sm2Extsync2 = 32U + (FlexPWM1_SM2_EXTSYNC2_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio1PinEventTrig0ToFlexPwm1Sm2Extsync2 = 33U + (FlexPWM1_SM2_EXTSYNC2_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio2PinEventTrig0ToFlexPwm1Sm2Extsync2 = 34U + (FlexPWM1_SM2_EXTSYNC2_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio3PinEventTrig0ToFlexPwm1Sm2Extsync2 = 35U + (FlexPWM1_SM2_EXTSYNC2_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio4PinEventTrig0ToFlexPwm1Sm2Extsync2 = 36U + (FlexPWM1_SM2_EXTSYNC2_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out0ToFlexPwm1Sm2Extsync2           = 37U + (FlexPWM1_SM2_EXTSYNC2_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out1ToFlexPwm1Sm2Extsync2           = 38U + (FlexPWM1_SM2_EXTSYNC2_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out2ToFlexPwm1Sm2Extsync2           = 39U + (FlexPWM1_SM2_EXTSYNC2_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out3ToFlexPwm1Sm2Extsync2           = 40U + (FlexPWM1_SM2_EXTSYNC2_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer3M2ToFlexPwm1Sm2Extsync2          = 45U + (FlexPWM1_SM2_EXTSYNC2_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer3M3ToFlexPwm1Sm2Extsync2          = 46U + (FlexPWM1_SM2_EXTSYNC2_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer4M2ToFlexPwm1Sm2Extsync2          = 47U + (FlexPWM1_SM2_EXTSYNC2_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer4M3ToFlexPwm1Sm2Extsync2          = 48U + (FlexPWM1_SM2_EXTSYNC2_REG << PMUX_SHIFT),
    kINPUTMUX_Qdc1CmpFlag0ToFlexPwm1Sm2Extsync2       = 49U + (FlexPWM1_SM2_EXTSYNC2_REG << PMUX_SHIFT),
    kINPUTMUX_Qdc1CmpFlag1ToFlexPwm1Sm2Extsync2       = 50U + (FlexPWM1_SM2_EXTSYNC2_REG << PMUX_SHIFT),
    kINPUTMUX_Qdc1CmpFlag2ToFlexPwm1Sm2Extsync2       = 51U + (FlexPWM1_SM2_EXTSYNC2_REG << PMUX_SHIFT),
    kINPUTMUX_Qdc1CmpFlag3ToFlexPwm1Sm2Extsync2       = 52U + (FlexPWM1_SM2_EXTSYNC2_REG << PMUX_SHIFT),
    kINPUTMUX_Qdc1PosMatch0ToFlexPwm1Sm2Extsync2      = 53U + (FlexPWM1_SM2_EXTSYNC2_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm0OutTrig0ToFlexPwm1Sm2Extsync2    = 54U + (FlexPWM1_SM2_EXTSYNC2_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm0OutTrig1ToFlexPwm1Sm2Extsync2    = 55U + (FlexPWM1_SM2_EXTSYNC2_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm1OutTrig0ToFlexPwm1Sm2Extsync2    = 56U + (FlexPWM1_SM2_EXTSYNC2_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm1OutTrig1ToFlexPwm1Sm2Extsync2    = 57U + (FlexPWM1_SM2_EXTSYNC2_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm2OutTrig0ToFlexPwm1Sm2Extsync2    = 58U + (FlexPWM1_SM2_EXTSYNC2_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm2OutTrig1ToFlexPwm1Sm2Extsync2    = 59U + (FlexPWM1_SM2_EXTSYNC2_REG << PMUX_SHIFT),

    /*!< FlexPWM1_FAULT input trigger connections. */
    kINPUTMUX_ArmTxevToFlexPwm1Fault            = 1U + (FlexPWM1_FAULT_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out0ToFlexPwm1Fault           = 2U + (FlexPWM1_FAULT_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out1ToFlexPwm1Fault           = 3U + (FlexPWM1_FAULT_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out2ToFlexPwm1Fault           = 4U + (FlexPWM1_FAULT_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out3ToFlexPwm1Fault           = 5U + (FlexPWM1_FAULT_REG << PMUX_SHIFT),
    kINPUTMUX_Cmp0OutToFlexPwm1Fault            = 6U + (FlexPWM1_FAULT_REG << PMUX_SHIFT),
    kINPUTMUX_Cmp1OutToFlexPwm1Fault            = 7U + (FlexPWM1_FAULT_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer0M2ToFlexPwm1Fault          = 9U + (FlexPWM1_FAULT_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer0M3ToFlexPwm1Fault          = 10U + (FlexPWM1_FAULT_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer1M2ToFlexPwm1Fault          = 11U + (FlexPWM1_FAULT_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer1M3ToFlexPwm1Fault          = 12U + (FlexPWM1_FAULT_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer2M2ToFlexPwm1Fault          = 13U + (FlexPWM1_FAULT_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer2M3ToFlexPwm1Fault          = 14U + (FlexPWM1_FAULT_REG << PMUX_SHIFT),
    kINPUTMUX_Qdc0CmpFlag0ToFlexPwm1Fault       = 15U + (FlexPWM1_FAULT_REG << PMUX_SHIFT),
    kINPUTMUX_Qdc0CmpFlag1ToFlexPwm1Fault       = 16U + (FlexPWM1_FAULT_REG << PMUX_SHIFT),
    kINPUTMUX_Qdc0CmpFlag2ToFlexPwm1Fault       = 17U + (FlexPWM1_FAULT_REG << PMUX_SHIFT),
    kINPUTMUX_Qdc0CmpFlag3ToFlexPwm1Fault       = 18U + (FlexPWM1_FAULT_REG << PMUX_SHIFT),
    kINPUTMUX_Qdc0PosMatch0ToFlexPwm1Fault      = 19U + (FlexPWM1_FAULT_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn0ToFlexPwm1Fault            = 20U + (FlexPWM1_FAULT_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn1ToFlexPwm1Fault            = 21U + (FlexPWM1_FAULT_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn2ToFlexPwm1Fault            = 22U + (FlexPWM1_FAULT_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn3ToFlexPwm1Fault            = 23U + (FlexPWM1_FAULT_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn4ToFlexPwm1Fault            = 24U + (FlexPWM1_FAULT_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn5ToFlexPwm1Fault            = 25U + (FlexPWM1_FAULT_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn6ToFlexPwm1Fault            = 26U + (FlexPWM1_FAULT_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn7ToFlexPwm1Fault            = 27U + (FlexPWM1_FAULT_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn8ToFlexPwm1Fault            = 28U + (FlexPWM1_FAULT_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn9ToFlexPwm1Fault            = 29U + (FlexPWM1_FAULT_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn10ToFlexPwm1Fault           = 30U + (FlexPWM1_FAULT_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn11ToFlexPwm1Fault           = 31U + (FlexPWM1_FAULT_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio0PinEventTrig0ToFlexPwm1Fault = 32U + (FlexPWM1_FAULT_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio1PinEventTrig0ToFlexPwm1Fault = 33U + (FlexPWM1_FAULT_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio2PinEventTrig0ToFlexPwm1Fault = 34U + (FlexPWM1_FAULT_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio3PinEventTrig0ToFlexPwm1Fault = 35U + (FlexPWM1_FAULT_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio4PinEventTrig0ToFlexPwm1Fault = 36U + (FlexPWM1_FAULT_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out0ToFlexPwm1Fault           = 37U + (FlexPWM1_FAULT_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out1ToFlexPwm1Fault           = 38U + (FlexPWM1_FAULT_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out2ToFlexPwm1Fault           = 39U + (FlexPWM1_FAULT_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out3ToFlexPwm1Fault           = 40U + (FlexPWM1_FAULT_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer3M2ToFlexPwm1Fault          = 45U + (FlexPWM1_FAULT_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer3M3ToFlexPwm1Fault          = 46U + (FlexPWM1_FAULT_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer4M2ToFlexPwm1Fault          = 47U + (FlexPWM1_FAULT_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer4M3ToFlexPwm1Fault          = 48U + (FlexPWM1_FAULT_REG << PMUX_SHIFT),
    kINPUTMUX_Qdc1CmpFlag0ToFlexPwm1Fault       = 49U + (FlexPWM1_FAULT_REG << PMUX_SHIFT),
    kINPUTMUX_Qdc1CmpFlag1ToFlexPwm1Fault       = 50U + (FlexPWM1_FAULT_REG << PMUX_SHIFT),
    kINPUTMUX_Qdc1CmpFlag2ToFlexPwm1Fault       = 51U + (FlexPWM1_FAULT_REG << PMUX_SHIFT),
    kINPUTMUX_Qdc1CmpFlag3ToFlexPwm1Fault       = 52U + (FlexPWM1_FAULT_REG << PMUX_SHIFT),
    kINPUTMUX_Qdc1PosMatch0ToFlexPwm1Fault      = 53U + (FlexPWM1_FAULT_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm0OutTrig0ToFlexPwm1Fault    = 54U + (FlexPWM1_FAULT_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm0OutTrig1ToFlexPwm1Fault    = 55U + (FlexPWM1_FAULT_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm1OutTrig0ToFlexPwm1Fault    = 56U + (FlexPWM1_FAULT_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm1OutTrig1ToFlexPwm1Fault    = 57U + (FlexPWM1_FAULT_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm2OutTrig0ToFlexPwm1Fault    = 58U + (FlexPWM1_FAULT_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm2OutTrig1ToFlexPwm1Fault    = 59U + (FlexPWM1_FAULT_REG << PMUX_SHIFT),

    /*!< FlexPWM1_FORCE input trigger connections. */
    kINPUTMUX_ArmTxevToFlexPwm1Force            = 1U + (FlexPWM1_FORCE_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out0ToFlexPwm1Force           = 2U + (FlexPWM1_FORCE_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out1ToFlexPwm1Force           = 3U + (FlexPWM1_FORCE_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out2ToFlexPwm1Force           = 4U + (FlexPWM1_FORCE_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out3ToFlexPwm1Force           = 5U + (FlexPWM1_FORCE_REG << PMUX_SHIFT),
    kINPUTMUX_Cmp0OutToFlexPwm1Force            = 6U + (FlexPWM1_FORCE_REG << PMUX_SHIFT),
    kINPUTMUX_Cmp1OutToFlexPwm1Force            = 7U + (FlexPWM1_FORCE_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer0M2ToFlexPwm1Force          = 9U + (FlexPWM1_FORCE_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer0M3ToFlexPwm1Force          = 10U + (FlexPWM1_FORCE_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer1M2ToFlexPwm1Force          = 11U + (FlexPWM1_FORCE_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer1M3ToFlexPwm1Force          = 12U + (FlexPWM1_FORCE_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer2M2ToFlexPwm1Force          = 13U + (FlexPWM1_FORCE_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer2M3ToFlexPwm1Force          = 14U + (FlexPWM1_FORCE_REG << PMUX_SHIFT),
    kINPUTMUX_Qdc0CmpFlag0ToFlexPwm1Force       = 15U + (FlexPWM1_FORCE_REG << PMUX_SHIFT),
    kINPUTMUX_Qdc0CmpFlag1ToFlexPwm1Force       = 16U + (FlexPWM1_FORCE_REG << PMUX_SHIFT),
    kINPUTMUX_Qdc0CmpFlag2ToFlexPwm1Force       = 17U + (FlexPWM1_FORCE_REG << PMUX_SHIFT),
    kINPUTMUX_Qdc0CmpFlag3ToFlexPwm1Force       = 18U + (FlexPWM1_FORCE_REG << PMUX_SHIFT),
    kINPUTMUX_Qdc0PosMatch0ToFlexPwm1Force      = 19U + (FlexPWM1_FORCE_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn0ToFlexPwm1Force            = 20U + (FlexPWM1_FORCE_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn1ToFlexPwm1Force            = 21U + (FlexPWM1_FORCE_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn2ToFlexPwm1Force            = 22U + (FlexPWM1_FORCE_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn3ToFlexPwm1Force            = 23U + (FlexPWM1_FORCE_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn4ToFlexPwm1Force            = 24U + (FlexPWM1_FORCE_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn5ToFlexPwm1Force            = 25U + (FlexPWM1_FORCE_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn6ToFlexPwm1Force            = 26U + (FlexPWM1_FORCE_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn7ToFlexPwm1Force            = 27U + (FlexPWM1_FORCE_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn8ToFlexPwm1Force            = 28U + (FlexPWM1_FORCE_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn9ToFlexPwm1Force            = 29U + (FlexPWM1_FORCE_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn10ToFlexPwm1Force           = 30U + (FlexPWM1_FORCE_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn11ToFlexPwm1Force           = 31U + (FlexPWM1_FORCE_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio0PinEventTrig0ToFlexPwm1Force = 32U + (FlexPWM1_FORCE_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio1PinEventTrig0ToFlexPwm1Force = 33U + (FlexPWM1_FORCE_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio2PinEventTrig0ToFlexPwm1Force = 34U + (FlexPWM1_FORCE_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio3PinEventTrig0ToFlexPwm1Force = 35U + (FlexPWM1_FORCE_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio4PinEventTrig0ToFlexPwm1Force = 36U + (FlexPWM1_FORCE_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out0ToFlexPwm1Force           = 37U + (FlexPWM1_FORCE_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out1ToFlexPwm1Force           = 38U + (FlexPWM1_FORCE_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out2ToFlexPwm1Force           = 39U + (FlexPWM1_FORCE_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out3ToFlexPwm1Force           = 40U + (FlexPWM1_FORCE_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer3M2ToFlexPwm1Force          = 45U + (FlexPWM1_FORCE_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer3M3ToFlexPwm1Force          = 46U + (FlexPWM1_FORCE_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer4M2ToFlexPwm1Force          = 47U + (FlexPWM1_FORCE_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer4M3ToFlexPwm1Force          = 48U + (FlexPWM1_FORCE_REG << PMUX_SHIFT),
    kINPUTMUX_Qdc1CmpFlag0ToFlexPwm1Force       = 49U + (FlexPWM1_FORCE_REG << PMUX_SHIFT),
    kINPUTMUX_Qdc1CmpFlag1ToFlexPwm1Force       = 50U + (FlexPWM1_FORCE_REG << PMUX_SHIFT),
    kINPUTMUX_Qdc1CmpFlag2ToFlexPwm1Force       = 51U + (FlexPWM1_FORCE_REG << PMUX_SHIFT),
    kINPUTMUX_Qdc1CmpFlag3ToFlexPwm1Force       = 52U + (FlexPWM1_FORCE_REG << PMUX_SHIFT),
    kINPUTMUX_Qdc1PosMatch0ToFlexPwm1Force      = 53U + (FlexPWM1_FORCE_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm0OutTrig0ToFlexPwm1Force    = 54U + (FlexPWM1_FORCE_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm0OutTrig1ToFlexPwm1Force    = 55U + (FlexPWM1_FORCE_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm1OutTrig0ToFlexPwm1Force    = 56U + (FlexPWM1_FORCE_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm1OutTrig1ToFlexPwm1Force    = 57U + (FlexPWM1_FORCE_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm2OutTrig0ToFlexPwm1Force    = 58U + (FlexPWM1_FORCE_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm2OutTrig1ToFlexPwm1Force    = 59U + (FlexPWM1_FORCE_REG << PMUX_SHIFT),

    /*!< PWM0 external clock trigger. */
    kINPUTMUX_Clk16K1ToPwm0ExtClk    = 1U + (PWM0_EXT_CLK_REG << PMUX_SHIFT),
    kINPUTMUX_ClkInToPwm0ExtClk      = 2U + (PWM0_EXT_CLK_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out0ToPwm0ExtClk   = 3U + (PWM0_EXT_CLK_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out1ToPwm0ExtClk   = 4U + (PWM0_EXT_CLK_REG << PMUX_SHIFT),
    kINPUTMUX_ExttrigIn0ToPwm0ExtClk = 5U + (PWM0_EXT_CLK_REG << PMUX_SHIFT),
    kINPUTMUX_ExttrigIn7ToPwm0ExtClk = 6U + (PWM0_EXT_CLK_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out0ToPwm0ExtClk   = 7U + (PWM0_EXT_CLK_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out1ToPwm0ExtClk   = 8U + (PWM0_EXT_CLK_REG << PMUX_SHIFT),

    /*!< PWM1 external clock trigger. */
    kINPUTMUX_Clk16K1ToPwm1ExtClk    = 1U + (PWM1_EXT_CLK_REG << PMUX_SHIFT),
    kINPUTMUX_ClkInToPwm1ExtClk      = 2U + (PWM1_EXT_CLK_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out0ToPwm1ExtClk   = 3U + (PWM1_EXT_CLK_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out1ToPwm1ExtClk   = 4U + (PWM1_EXT_CLK_REG << PMUX_SHIFT),
    kINPUTMUX_ExttrigIn0ToPwm1ExtClk = 5U + (PWM1_EXT_CLK_REG << PMUX_SHIFT),
    kINPUTMUX_ExttrigIn7ToPwm1ExtClk = 6U + (PWM1_EXT_CLK_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out0ToPwm1ExtClk   = 7U + (PWM1_EXT_CLK_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out1ToPwm1ExtClk   = 8U + (PWM1_EXT_CLK_REG << PMUX_SHIFT),

    /*!< AOI0 trigger input connections. */
    kINPUTMUX_Adc0Tcomp0ToAoi0Mux         = 1U + (AOI0_MUX_REG << PMUX_SHIFT),
    kINPUTMUX_Adc0Tcomp1ToAoi0Mux         = 2U + (AOI0_MUX_REG << PMUX_SHIFT),
    kINPUTMUX_Adc0Tcomp2ToAoi0Mux         = 3U + (AOI0_MUX_REG << PMUX_SHIFT),
    kINPUTMUX_Adc0Tcomp3ToAoi0Mux         = 4U + (AOI0_MUX_REG << PMUX_SHIFT),
    kINPUTMUX_Cmp0OutToAoi0Mux            = 5U + (AOI0_MUX_REG << PMUX_SHIFT),
    kINPUTMUX_Cmp1OutToAoi0Mux            = 6U + (AOI0_MUX_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer0M0ToAoi0Mux          = 8U + (AOI0_MUX_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer0M1ToAoi0Mux          = 9U + (AOI0_MUX_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer0M2ToAoi0Mux          = 10U + (AOI0_MUX_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer0M3ToAoi0Mux          = 11U + (AOI0_MUX_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer1M0ToAoi0Mux          = 12U + (AOI0_MUX_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer1M1ToAoi0Mux          = 13U + (AOI0_MUX_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer1M2ToAoi0Mux          = 14U + (AOI0_MUX_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer1M3ToAoi0Mux          = 15U + (AOI0_MUX_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer2M0ToAoi0Mux          = 16U + (AOI0_MUX_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer2M1ToAoi0Mux          = 17U + (AOI0_MUX_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer2M2ToAoi0Mux          = 18U + (AOI0_MUX_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer2M3ToAoi0Mux          = 19U + (AOI0_MUX_REG << PMUX_SHIFT),
    kINPUTMUX_Lptmr0ToAoi0Mux             = 20U + (AOI0_MUX_REG << PMUX_SHIFT),
    kINPUTMUX_Qdc0CmpFlag0ToAoi0Mux       = 22U + (AOI0_MUX_REG << PMUX_SHIFT),
    kINPUTMUX_Qdc0CmpFlag1ToAoi0Mux       = 23U + (AOI0_MUX_REG << PMUX_SHIFT),
    kINPUTMUX_Qdc0CmpFlag2ToAoi0Mux       = 24U + (AOI0_MUX_REG << PMUX_SHIFT),
    kINPUTMUX_Qdc0CmpFlag3ToAoi0Mux       = 25U + (AOI0_MUX_REG << PMUX_SHIFT),
    kINPUTMUX_Qdc0PosMatchToAoi0Mux       = 26U + (AOI0_MUX_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm0OutTrig0ToAoi0Mux    = 27U + (AOI0_MUX_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm0OutTrig1ToAoi0Mux    = 28U + (AOI0_MUX_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm1OutTrig0ToAoi0Mux    = 29U + (AOI0_MUX_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm1OutTrig1ToAoi0Mux    = 30U + (AOI0_MUX_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm2OutTrig0ToAoi0Mux    = 31U + (AOI0_MUX_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm2OutTrig1ToAoi0Mux    = 32U + (AOI0_MUX_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn0ToAoi0Mux            = 35U + (AOI0_MUX_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn1ToAoi0Mux            = 36U + (AOI0_MUX_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn2ToAoi0Mux            = 37U + (AOI0_MUX_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn3ToAoi0Mux            = 38U + (AOI0_MUX_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn4ToAoi0Mux            = 39U + (AOI0_MUX_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn5ToAoi0Mux            = 40U + (AOI0_MUX_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn6ToAoi0Mux            = 41U + (AOI0_MUX_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn7ToAoi0Mux            = 42U + (AOI0_MUX_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn8ToAoi0Mux            = 43U + (AOI0_MUX_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn9ToAoi0Mux            = 44U + (AOI0_MUX_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn10ToAoi0Mux           = 45U + (AOI0_MUX_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn11ToAoi0Mux           = 46U + (AOI0_MUX_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio0PinEventTrig0ToAoi0Mux = 47U + (AOI0_MUX_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio1PinEventTrig0ToAoi0Mux = 48U + (AOI0_MUX_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio2PinEventTrig0ToAoi0Mux = 49U + (AOI0_MUX_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio3PinEventTrig0ToAoi0Mux = 50U + (AOI0_MUX_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio4PinEventTrig0ToAoi0Mux = 51U + (AOI0_MUX_REG << PMUX_SHIFT),
    kINPUTMUX_Adc1Tcomp0ToAoi0Mux         = 52U + (AOI0_MUX_REG << PMUX_SHIFT),
    kINPUTMUX_Adc1Tcomp1ToAoi0Mux         = 53U + (AOI0_MUX_REG << PMUX_SHIFT),
    kINPUTMUX_Adc1Tcomp2ToAoi0Mux         = 54U + (AOI0_MUX_REG << PMUX_SHIFT),
    kINPUTMUX_Adc1Tcomp3ToAoi0Mux         = 55U + (AOI0_MUX_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer3M0ToAoi0Mux          = 56U + (AOI0_MUX_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer3M1ToAoi0Mux          = 57U + (AOI0_MUX_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer3M2ToAoi0Mux          = 58U + (AOI0_MUX_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer3M3ToAoi0Mux          = 59U + (AOI0_MUX_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer4M0ToAoi0Mux          = 60U + (AOI0_MUX_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer4M1ToAoi0Mux          = 61U + (AOI0_MUX_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer4M2ToAoi0Mux          = 62U + (AOI0_MUX_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer4M3ToAoi0Mux          = 63U + (AOI0_MUX_REG << PMUX_SHIFT),
    kINPUTMUX_FlexioCh0ToAoi0Mux          = 64U + (AOI0_MUX_REG << PMUX_SHIFT),
    kINPUTMUX_FlexioCh1ToAoi0Mux          = 65U + (AOI0_MUX_REG << PMUX_SHIFT),
    kINPUTMUX_FlexioCh2ToAoi0Mux          = 66U + (AOI0_MUX_REG << PMUX_SHIFT),
    kINPUTMUX_FlexioCh3ToAoi0Mux          = 67U + (AOI0_MUX_REG << PMUX_SHIFT),
    kINPUTMUX_Qdc1CmpFlag0ToAoi0Mux       = 68U + (AOI0_MUX_REG << PMUX_SHIFT),
    kINPUTMUX_Qdc1CmpFlag1ToAoi0Mux       = 69U + (AOI0_MUX_REG << PMUX_SHIFT),
    kINPUTMUX_Qdc1CmpFlag2ToAoi0Mux       = 70U + (AOI0_MUX_REG << PMUX_SHIFT),
    kINPUTMUX_Qdc1CmpFlag3ToAoi0Mux       = 71U + (AOI0_MUX_REG << PMUX_SHIFT),
    kINPUTMUX_Qdc1PosMatchToAoi0Mux       = 72U + (AOI0_MUX_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm1Sm0OutTrig0ToAoi0Mux    = 73U + (AOI0_MUX_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm1Sm0OutTrig1ToAoi0Mux    = 74U + (AOI0_MUX_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm1Sm1OutTrig0ToAoi0Mux    = 75U + (AOI0_MUX_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm1Sm1OutTrig1ToAoi0Mux    = 76U + (AOI0_MUX_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm1Sm2OutTrig0ToAoi0Mux    = 77U + (AOI0_MUX_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm1Sm2OutTrig1ToAoi0Mux    = 78U + (AOI0_MUX_REG << PMUX_SHIFT),

    /*!< AOI1 trigger input connections. */
    kINPUTMUX_Adc0Tcomp0ToAoi1Mux         = 1U + (AOI1_MUX_REG << PMUX_SHIFT),
    kINPUTMUX_Adc0Tcomp1ToAoi1Mux         = 2U + (AOI1_MUX_REG << PMUX_SHIFT),
    kINPUTMUX_Adc0Tcomp2ToAoi1Mux         = 3U + (AOI1_MUX_REG << PMUX_SHIFT),
    kINPUTMUX_Adc0Tcomp3ToAoi1Mux         = 4U + (AOI1_MUX_REG << PMUX_SHIFT),
    kINPUTMUX_Cmp0OutToAoi1Mux            = 5U + (AOI1_MUX_REG << PMUX_SHIFT),
    kINPUTMUX_Cmp1OutToAoi1Mux            = 6U + (AOI1_MUX_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer0M0ToAoi1Mux          = 8U + (AOI1_MUX_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer0M1ToAoi1Mux          = 9U + (AOI1_MUX_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer0M2ToAoi1Mux          = 10U + (AOI1_MUX_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer0M3ToAoi1Mux          = 11U + (AOI1_MUX_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer1M0ToAoi1Mux          = 12U + (AOI1_MUX_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer1M1ToAoi1Mux          = 13U + (AOI1_MUX_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer1M2ToAoi1Mux          = 14U + (AOI1_MUX_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer1M3ToAoi1Mux          = 15U + (AOI1_MUX_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer2M0ToAoi1Mux          = 16U + (AOI1_MUX_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer2M1ToAoi1Mux          = 17U + (AOI1_MUX_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer2M2ToAoi1Mux          = 18U + (AOI1_MUX_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer2M3ToAoi1Mux          = 19U + (AOI1_MUX_REG << PMUX_SHIFT),
    kINPUTMUX_Lptmr0ToAoi1Mux             = 20U + (AOI1_MUX_REG << PMUX_SHIFT),
    kINPUTMUX_Qdc0CmpFlag0ToAoi1Mux       = 22U + (AOI1_MUX_REG << PMUX_SHIFT),
    kINPUTMUX_Qdc0CmpFlag1ToAoi1Mux       = 23U + (AOI1_MUX_REG << PMUX_SHIFT),
    kINPUTMUX_Qdc0CmpFlag2ToAoi1Mux       = 24U + (AOI1_MUX_REG << PMUX_SHIFT),
    kINPUTMUX_Qdc0CmpFlag3ToAoi1Mux       = 25U + (AOI1_MUX_REG << PMUX_SHIFT),
    kINPUTMUX_Qdc0PosMatchToAoi1Mux       = 26U + (AOI1_MUX_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm0OutTrig0ToAoi1Mux    = 27U + (AOI1_MUX_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm0OutTrig1ToAoi1Mux    = 28U + (AOI1_MUX_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm1OutTrig0ToAoi1Mux    = 29U + (AOI1_MUX_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm1OutTrig1ToAoi1Mux    = 30U + (AOI1_MUX_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm2OutTrig0ToAoi1Mux    = 31U + (AOI1_MUX_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm2OutTrig1ToAoi1Mux    = 32U + (AOI1_MUX_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn0ToAoi1Mux            = 35U + (AOI1_MUX_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn1ToAoi1Mux            = 36U + (AOI1_MUX_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn2ToAoi1Mux            = 37U + (AOI1_MUX_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn3ToAoi1Mux            = 38U + (AOI1_MUX_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn4ToAoi1Mux            = 39U + (AOI1_MUX_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn5ToAoi1Mux            = 40U + (AOI1_MUX_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn6ToAoi1Mux            = 41U + (AOI1_MUX_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn7ToAoi1Mux            = 42U + (AOI1_MUX_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn8ToAoi1Mux            = 43U + (AOI1_MUX_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn9ToAoi1Mux            = 44U + (AOI1_MUX_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn10ToAoi1Mux           = 45U + (AOI1_MUX_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn11ToAoi1Mux           = 46U + (AOI1_MUX_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio0PinEventTrig0ToAoi1Mux = 47U + (AOI1_MUX_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio1PinEventTrig0ToAoi1Mux = 48U + (AOI1_MUX_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio2PinEventTrig0ToAoi1Mux = 49U + (AOI1_MUX_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio3PinEventTrig0ToAoi1Mux = 50U + (AOI1_MUX_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio4PinEventTrig0ToAoi1Mux = 51U + (AOI1_MUX_REG << PMUX_SHIFT),
    kINPUTMUX_Adc1Tcomp0ToAoi1Mux         = 52U + (AOI1_MUX_REG << PMUX_SHIFT),
    kINPUTMUX_Adc1Tcomp1ToAoi1Mux         = 53U + (AOI1_MUX_REG << PMUX_SHIFT),
    kINPUTMUX_Adc1Tcomp2ToAoi1Mux         = 54U + (AOI1_MUX_REG << PMUX_SHIFT),
    kINPUTMUX_Adc1Tcomp3ToAoi1Mux         = 55U + (AOI1_MUX_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer3M0ToAoi1Mux          = 56U + (AOI1_MUX_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer3M1ToAoi1Mux          = 57U + (AOI1_MUX_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer3M2ToAoi1Mux          = 58U + (AOI1_MUX_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer3M3ToAoi1Mux          = 59U + (AOI1_MUX_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer4M0ToAoi1Mux          = 60U + (AOI1_MUX_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer4M1ToAoi1Mux          = 61U + (AOI1_MUX_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer4M2ToAoi1Mux          = 62U + (AOI1_MUX_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer4M3ToAoi1Mux          = 63U + (AOI1_MUX_REG << PMUX_SHIFT),
    kINPUTMUX_FlexioCh0ToAoi1Mux          = 64U + (AOI1_MUX_REG << PMUX_SHIFT),
    kINPUTMUX_FlexioCh1ToAoi1Mux          = 65U + (AOI1_MUX_REG << PMUX_SHIFT),
    kINPUTMUX_FlexioCh2ToAoi1Mux          = 66U + (AOI1_MUX_REG << PMUX_SHIFT),
    kINPUTMUX_FlexioCh3ToAoi1Mux          = 67U + (AOI1_MUX_REG << PMUX_SHIFT),
    kINPUTMUX_Qdc1CmpFlag0ToAoi1Mux       = 68U + (AOI1_MUX_REG << PMUX_SHIFT),
    kINPUTMUX_Qdc1CmpFlag1ToAoi1Mux       = 69U + (AOI1_MUX_REG << PMUX_SHIFT),
    kINPUTMUX_Qdc1CmpFlag2ToAoi1Mux       = 70U + (AOI1_MUX_REG << PMUX_SHIFT),
    kINPUTMUX_Qdc1CmpFlag3ToAoi1Mux       = 71U + (AOI1_MUX_REG << PMUX_SHIFT),
    kINPUTMUX_Qdc1PosMatchToAoi1Mux       = 72U + (AOI1_MUX_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm1Sm0OutTrig0ToAoi1Mux    = 73U + (AOI1_MUX_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm1Sm0OutTrig1ToAoi1Mux    = 74U + (AOI1_MUX_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm1Sm1OutTrig0ToAoi1Mux    = 75U + (AOI1_MUX_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm1Sm1OutTrig1ToAoi1Mux    = 76U + (AOI1_MUX_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm1Sm2OutTrig0ToAoi1Mux    = 77U + (AOI1_MUX_REG << PMUX_SHIFT),
    kINPUTMUX_Pwm1Sm2OutTrig1ToAoi1Mux    = 78U + (AOI1_MUX_REG << PMUX_SHIFT),

    /*!< USB-FS trigger input connections. */
    kINPUTMUX_Lpuart0TrgTxdataToUsbfsTrigger = 1U + (USBFS_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Lpuart1TrgTxdataToUsbfsTrigger = 2U + (USBFS_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Lpuart2TrgTxdataToUsbfsTrigger = 3U + (USBFS_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Lpuart3TrgTxdataToUsbfsTrigger = 4U + (USBFS_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Lpuart4TrgTxdataToUsbfsTrigger = 5U + (USBFS_TRIG_REG << PMUX_SHIFT),

    /*!< EXT trigger connections. */
    kINPUTMUX_Aoi0Out0ToExtTrigger = 2U + (EXT_TRIG0_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out1ToExtTrigger = 3U + (EXT_TRIG0_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out2ToExtTrigger = 4U + (EXT_TRIG0_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out3ToExtTrigger = 5U + (EXT_TRIG0_REG << PMUX_SHIFT),
    kINPUTMUX_Cmp0OutToExtTrigger  = 6U + (EXT_TRIG0_REG << PMUX_SHIFT),
    kINPUTMUX_Cmp1OutToExtTrigger  = 7U + (EXT_TRIG0_REG << PMUX_SHIFT),
    kINPUTMUX_Lpuart0ToExtTrigger  = 9U + (EXT_TRIG0_REG << PMUX_SHIFT),
    kINPUTMUX_Lpuart1ToExtTrigger  = 10U + (EXT_TRIG0_REG << PMUX_SHIFT),
    kINPUTMUX_Lpuart2ToExtTrigger  = 11U + (EXT_TRIG0_REG << PMUX_SHIFT),
    kINPUTMUX_Lpuart3ToExtTrigger  = 12U + (EXT_TRIG0_REG << PMUX_SHIFT),
    kINPUTMUX_Lpuart4ToExtTrigger  = 13U + (EXT_TRIG0_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out0ToExtTrigger = 14U + (EXT_TRIG0_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out1ToExtTrigger = 15U + (EXT_TRIG0_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out2ToExtTrigger = 16U + (EXT_TRIG0_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out3ToExtTrigger = 17U + (EXT_TRIG0_REG << PMUX_SHIFT),

    /*!< LPI2C0 trigger input connections. */
    kINPUTMUX_Aoi0Out0ToLpi2c0Trigger           = 2U + (LPI2C0_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out1ToLpi2c0Trigger           = 3U + (LPI2C0_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out2ToLpi2c0Trigger           = 4U + (LPI2C0_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out3ToLpi2c0Trigger           = 5U + (LPI2C0_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Cmp0OutToLpi2c0Trigger            = 6U + (LPI2C0_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Cmp1OutToLpi2c0Trigger            = 7U + (LPI2C0_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer0M0ToLpi2c0Trigger          = 9U + (LPI2C0_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer0M1ToLpi2c0Trigger          = 10U + (LPI2C0_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer1M0ToLpi2c0Trigger          = 11U + (LPI2C0_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer1M1ToLpi2c0Trigger          = 12U + (LPI2C0_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer2M0ToLpi2c0Trigger          = 13U + (LPI2C0_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer2M1ToLpi2c0Trigger          = 14U + (LPI2C0_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Lptmr0ToLpi2c0Trigger             = 15U + (LPI2C0_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn0ToLpi2c0Trigger            = 17U + (LPI2C0_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn1ToLpi2c0Trigger            = 18U + (LPI2C0_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn2ToLpi2c0Trigger            = 19U + (LPI2C0_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn3ToLpi2c0Trigger            = 20U + (LPI2C0_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn4ToLpi2c0Trigger            = 21U + (LPI2C0_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn5ToLpi2c0Trigger            = 22U + (LPI2C0_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn6ToLpi2c0Trigger            = 23U + (LPI2C0_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn7ToLpi2c0Trigger            = 24U + (LPI2C0_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio0PinEventTrig0ToLpi2c0Trigger = 25U + (LPI2C0_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio1PinEventTrig0ToLpi2c0Trigger = 26U + (LPI2C0_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio2PinEventTrig0ToLpi2c0Trigger = 27U + (LPI2C0_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio3PinEventTrig0ToLpi2c0Trigger = 28U + (LPI2C0_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio4PinEventTrig0ToLpi2c0Trigger = 29U + (LPI2C0_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_WuuToLpi2c0Trigger                = 30U + (LPI2C0_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out0ToLpi2c0Trigger           = 31U + (LPI2C0_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out1ToLpi2c0Trigger           = 32U + (LPI2C0_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out2ToLpi2c0Trigger           = 33U + (LPI2C0_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out3ToLpi2c0Trigger           = 34U + (LPI2C0_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer3M2ToLpi2c0Trigger          = 35U + (LPI2C0_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer3M3ToLpi2c0Trigger          = 36U + (LPI2C0_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer4M2ToLpi2c0Trigger          = 37U + (LPI2C0_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer4M3ToLpi2c0Trigger          = 38U + (LPI2C0_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_FlexioCh0ToLpi2c0Trigger          = 39U + (LPI2C0_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_FlexioCh1ToLpi2c0Trigger          = 40U + (LPI2C0_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_FlexioCh2ToLpi2c0Trigger          = 41U + (LPI2C0_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_FlexioCh3ToLpi2c0Trigger          = 42U + (LPI2C0_TRIG_REG << PMUX_SHIFT),

    /*!< LPI2C1 trigger input connections. */
    kINPUTMUX_Aoi0Out0ToLpi2c1Trigger           = 2U + (LPI2C1_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out1ToLpi2c1Trigger           = 3U + (LPI2C1_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out2ToLpi2c1Trigger           = 4U + (LPI2C1_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out3ToLpi2c1Trigger           = 5U + (LPI2C1_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Cmp0OutToLpi2c1Trigger            = 6U + (LPI2C1_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Cmp1OutToLpi2c1Trigger            = 7U + (LPI2C1_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer0M0ToLpi2c1Trigger          = 9U + (LPI2C1_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer0M1ToLpi2c1Trigger          = 10U + (LPI2C1_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer1M0ToLpi2c1Trigger          = 11U + (LPI2C1_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer1M1ToLpi2c1Trigger          = 12U + (LPI2C1_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer2M0ToLpi2c1Trigger          = 13U + (LPI2C1_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer2M1ToLpi2c1Trigger          = 14U + (LPI2C1_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Lptmr0ToLpi2c1Trigger             = 15U + (LPI2C1_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn0ToLpi2c1Trigger            = 17U + (LPI2C1_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn1ToLpi2c1Trigger            = 18U + (LPI2C1_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn2ToLpi2c1Trigger            = 19U + (LPI2C1_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn3ToLpi2c1Trigger            = 20U + (LPI2C1_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn4ToLpi2c1Trigger            = 21U + (LPI2C1_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn5ToLpi2c1Trigger            = 22U + (LPI2C1_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn6ToLpi2c1Trigger            = 23U + (LPI2C1_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn7ToLpi2c1Trigger            = 24U + (LPI2C1_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio0PinEventTrig0ToLpi2c1Trigger = 25U + (LPI2C1_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio1PinEventTrig0ToLpi2c1Trigger = 26U + (LPI2C1_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio2PinEventTrig0ToLpi2c1Trigger = 27U + (LPI2C1_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio3PinEventTrig0ToLpi2c1Trigger = 28U + (LPI2C1_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio4PinEventTrig0ToLpi2c1Trigger = 29U + (LPI2C1_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_WuuToLpi2c1Trigger                = 30U + (LPI2C1_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out0ToLpi2c1Trigger           = 31U + (LPI2C1_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out1ToLpi2c1Trigger           = 32U + (LPI2C1_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out2ToLpi2c1Trigger           = 33U + (LPI2C1_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out3ToLpi2c1Trigger           = 34U + (LPI2C1_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer3M2ToLpi2c1Trigger          = 35U + (LPI2C1_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer3M3ToLpi2c1Trigger          = 36U + (LPI2C1_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer4M2ToLpi2c1Trigger          = 37U + (LPI2C1_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer4M3ToLpi2c1Trigger          = 38U + (LPI2C1_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_FlexioCh0ToLpi2c1Trigger          = 39U + (LPI2C1_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_FlexioCh1ToLpi2c1Trigger          = 40U + (LPI2C1_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_FlexioCh2ToLpi2c1Trigger          = 41U + (LPI2C1_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_FlexioCh3ToLpi2c1Trigger          = 42U + (LPI2C1_TRIG_REG << PMUX_SHIFT),

    /*!< LPI2C2 trigger input connections. */
    kINPUTMUX_Aoi0Out0ToLpi2c2Trigger           = 2U + (LPI2C2_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out1ToLpi2c2Trigger           = 3U + (LPI2C2_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out2ToLpi2c2Trigger           = 4U + (LPI2C2_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out3ToLpi2c2Trigger           = 5U + (LPI2C2_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Cmp0OutToLpi2c2Trigger            = 6U + (LPI2C2_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Cmp1OutToLpi2c2Trigger            = 7U + (LPI2C2_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer0M0ToLpi2c2Trigger          = 9U + (LPI2C2_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer0M1ToLpi2c2Trigger          = 10U + (LPI2C2_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer1M0ToLpi2c2Trigger          = 11U + (LPI2C2_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer1M1ToLpi2c2Trigger          = 12U + (LPI2C2_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer2M0ToLpi2c2Trigger          = 13U + (LPI2C2_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer2M1ToLpi2c2Trigger          = 14U + (LPI2C2_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Lptmr0ToLpi2c2Trigger             = 15U + (LPI2C2_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn0ToLpi2c2Trigger            = 17U + (LPI2C2_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn1ToLpi2c2Trigger            = 18U + (LPI2C2_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn2ToLpi2c2Trigger            = 19U + (LPI2C2_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn3ToLpi2c2Trigger            = 20U + (LPI2C2_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn4ToLpi2c2Trigger            = 21U + (LPI2C2_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn5ToLpi2c2Trigger            = 22U + (LPI2C2_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn6ToLpi2c2Trigger            = 23U + (LPI2C2_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn7ToLpi2c2Trigger            = 24U + (LPI2C2_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio0PinEventTrig0ToLpi2c2Trigger = 25U + (LPI2C2_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio1PinEventTrig0ToLpi2c2Trigger = 26U + (LPI2C2_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio2PinEventTrig0ToLpi2c2Trigger = 27U + (LPI2C2_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio3PinEventTrig0ToLpi2c2Trigger = 28U + (LPI2C2_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio4PinEventTrig0ToLpi2c2Trigger = 29U + (LPI2C2_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_WuuToLpi2c2Trigger                = 30U + (LPI2C2_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out0ToLpi2c2Trigger           = 31U + (LPI2C2_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out1ToLpi2c2Trigger           = 32U + (LPI2C2_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out2ToLpi2c2Trigger           = 33U + (LPI2C2_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out3ToLpi2c2Trigger           = 34U + (LPI2C2_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer3M2ToLpi2c2Trigger          = 35U + (LPI2C2_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer3M3ToLpi2c2Trigger          = 36U + (LPI2C2_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer4M2ToLpi2c2Trigger          = 37U + (LPI2C2_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer4M3ToLpi2c2Trigger          = 38U + (LPI2C2_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_FlexioCh0ToLpi2c2Trigger          = 39U + (LPI2C2_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_FlexioCh1ToLpi2c2Trigger          = 40U + (LPI2C2_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_FlexioCh2ToLpi2c2Trigger          = 41U + (LPI2C2_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_FlexioCh3ToLpi2c2Trigger          = 42U + (LPI2C2_TRIG_REG << PMUX_SHIFT),

    /*!< LPSPI0 trigger input connections. */
    kINPUTMUX_Aoi0Out0ToLpspi0Trigger           = 2U + (LPSPI0_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out1ToLpspi0Trigger           = 3U + (LPSPI0_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out2ToLpspi0Trigger           = 4U + (LPSPI0_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out3ToLpspi0Trigger           = 5U + (LPSPI0_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Cmp0OutToLpspi0Trigger            = 6U + (LPSPI0_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Cmp1OutToLpspi0Trigger            = 7U + (LPSPI0_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer0M1ToLpspi0Trigger          = 9U + (LPSPI0_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer0M2ToLpspi0Trigger          = 10U + (LPSPI0_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer1M1ToLpspi0Trigger          = 11U + (LPSPI0_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer1M2ToLpspi0Trigger          = 12U + (LPSPI0_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer2M1ToLpspi0Trigger          = 13U + (LPSPI0_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer2M2ToLpspi0Trigger          = 14U + (LPSPI0_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Lptmr0ToLpspi0Trigger             = 15U + (LPSPI0_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn0ToLpspi0Trigger            = 17U + (LPSPI0_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn1ToLpspi0Trigger            = 18U + (LPSPI0_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn2ToLpspi0Trigger            = 19U + (LPSPI0_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn3ToLpspi0Trigger            = 20U + (LPSPI0_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn4ToLpspi0Trigger            = 21U + (LPSPI0_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn5ToLpspi0Trigger            = 22U + (LPSPI0_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn6ToLpspi0Trigger            = 23U + (LPSPI0_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn7ToLpspi0Trigger            = 24U + (LPSPI0_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio0PinEventTrig0ToLpspi0Trigger = 25U + (LPSPI0_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio1PinEventTrig0ToLpspi0Trigger = 26U + (LPSPI0_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio2PinEventTrig0ToLpspi0Trigger = 27U + (LPSPI0_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio3PinEventTrig0ToLpspi0Trigger = 28U + (LPSPI0_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio4PinEventTrig0ToLpspi0Trigger = 29U + (LPSPI0_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_WuuToLpspi0Trigger                = 30U + (LPSPI0_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out0ToLpspi0Trigger           = 31U + (LPSPI0_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out1ToLpspi0Trigger           = 32U + (LPSPI0_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out2ToLpspi0Trigger           = 33U + (LPSPI0_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out3ToLpspi0Trigger           = 34U + (LPSPI0_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer3M2ToLpspi0Trigger          = 35U + (LPSPI0_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer3M3ToLpspi0Trigger          = 36U + (LPSPI0_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer4M2ToLpspi0Trigger          = 37U + (LPSPI0_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer4M3ToLpspi0Trigger          = 38U + (LPSPI0_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_FlexioCh0ToLpspi0Trigger          = 39U + (LPSPI0_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_FlexioCh1ToLpspi0Trigger          = 40U + (LPSPI0_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_FlexioCh2ToLpspi0Trigger          = 41U + (LPSPI0_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_FlexioCh3ToLpspi0Trigger          = 42U + (LPSPI0_TRIG_REG << PMUX_SHIFT),

    /*!< LPSPI1 trigger input connections. */
    kINPUTMUX_Aoi0Out0ToLpspi1Trigger           = 2U + (LPSPI1_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out1ToLpspi1Trigger           = 3U + (LPSPI1_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out2ToLpspi1Trigger           = 4U + (LPSPI1_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out3ToLpspi1Trigger           = 5U + (LPSPI1_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Cmp0OutToLpspi1Trigger            = 6U + (LPSPI1_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Cmp1OutToLpspi1Trigger            = 7U + (LPSPI1_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer0M1ToLpspi1Trigger          = 9U + (LPSPI1_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer0M2ToLpspi1Trigger          = 10U + (LPSPI1_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer1M1ToLpspi1Trigger          = 11U + (LPSPI1_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer1M2ToLpspi1Trigger          = 12U + (LPSPI1_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer2M1ToLpspi1Trigger          = 13U + (LPSPI1_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer2M2ToLpspi1Trigger          = 14U + (LPSPI1_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Lptmr0ToLpspi1Trigger             = 15U + (LPSPI1_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn0ToLpspi1Trigger            = 17U + (LPSPI1_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn1ToLpspi1Trigger            = 18U + (LPSPI1_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn2ToLpspi1Trigger            = 19U + (LPSPI1_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn3ToLpspi1Trigger            = 20U + (LPSPI1_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn4ToLpspi1Trigger            = 21U + (LPSPI1_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn5ToLpspi1Trigger            = 22U + (LPSPI1_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn6ToLpspi1Trigger            = 23U + (LPSPI1_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn7ToLpspi1Trigger            = 24U + (LPSPI1_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio0PinEventTrig0ToLpspi1Trigger = 25U + (LPSPI1_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio1PinEventTrig0ToLpspi1Trigger = 26U + (LPSPI1_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio2PinEventTrig0ToLpspi1Trigger = 27U + (LPSPI1_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio3PinEventTrig0ToLpspi1Trigger = 28U + (LPSPI1_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio4PinEventTrig0ToLpspi1Trigger = 29U + (LPSPI1_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_WuuToLpspi1Trigger                = 30U + (LPSPI1_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out0ToLpspi1Trigger           = 31U + (LPSPI1_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out1ToLpspi1Trigger           = 32U + (LPSPI1_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out2ToLpspi1Trigger           = 33U + (LPSPI1_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out3ToLpspi1Trigger           = 34U + (LPSPI1_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer3M2ToLpspi1Trigger          = 35U + (LPSPI1_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer3M3ToLpspi1Trigger          = 36U + (LPSPI1_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer4M2ToLpspi1Trigger          = 37U + (LPSPI1_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer4M3ToLpspi1Trigger          = 38U + (LPSPI1_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_FlexioCh0ToLpspi1Trigger          = 39U + (LPSPI1_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_FlexioCh1ToLpspi1Trigger          = 40U + (LPSPI1_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_FlexioCh2ToLpspi1Trigger          = 41U + (LPSPI1_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_FlexioCh3ToLpspi1Trigger          = 42U + (LPSPI1_TRIG_REG << PMUX_SHIFT),

    /*!< LPUART0 trigger input connections. */
    kINPUTMUX_Aoi0Out0ToLpuart0Trigger                = 2U + (LPUART0_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out1ToLpuart0Trigger                = 3U + (LPUART0_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out2ToLpuart0Trigger                = 4U + (LPUART0_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out3ToLpuart0Trigger                = 5U + (LPUART0_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Cmp0OutToLpuart0Trigger                 = 6U + (LPUART0_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Cmp1OutToLpuart0Trigger                 = 7U + (LPUART0_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer0M2ToLpuart0Trigger               = 9U + (LPUART0_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer0M3ToLpuart0Trigger               = 10U + (LPUART0_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer1M2ToLpuart0Trigger               = 11U + (LPUART0_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer1M3ToLpuart0Trigger               = 12U + (LPUART0_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer2M2ToLpuart0Trigger               = 13U + (LPUART0_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer2M3ToLpuart0Trigger               = 14U + (LPUART0_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Lptmr0ToLpuart0Trigger                  = 15U + (LPUART0_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn0ToLpuart0Trigger                 = 17U + (LPUART0_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn1ToLpuart0Trigger                 = 18U + (LPUART0_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn2ToLpuart0Trigger                 = 19U + (LPUART0_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn3ToLpuart0Trigger                 = 20U + (LPUART0_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn4ToLpuart0Trigger                 = 21U + (LPUART0_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn5ToLpuart0Trigger                 = 22U + (LPUART0_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn6ToLpuart0Trigger                 = 23U + (LPUART0_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn7ToLpuart0Trigger                 = 24U + (LPUART0_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn8ToLpuart0Trigger                 = 25U + (LPUART0_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn9ToLpuart0Trigger                 = 26U + (LPUART0_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn10ToLpuart0Trigger                = 27U + (LPUART0_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn11ToLpuart0Trigger                = 28U + (LPUART0_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio0PinEventTrig0ToLpuart0Trigger      = 29U + (LPUART0_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio1PinEventTrig0ToLpuart0Trigger      = 30U + (LPUART0_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio2PinEventTrig0ToLpuart0Trigger      = 31U + (LPUART0_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio3PinEventTrig0ToLpuart0Trigger      = 32U + (LPUART0_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio4PinEventTrig0ToLpuart0Trigger      = 33U + (LPUART0_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_WuuToLpuart0Trigger                     = 34U + (LPUART0_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Usb0IppIndUartRxdUsbmuxToLpuart0Trigger = 35U + (LPUART0_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out0ToLpuart0Trigger                = 36U + (LPUART0_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out1ToLpuart0Trigger                = 37U + (LPUART0_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out2ToLpuart0Trigger                = 38U + (LPUART0_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out3ToLpuart0Trigger                = 39U + (LPUART0_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer3M2ToLpuart0Trigger               = 40U + (LPUART0_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer3M3ToLpuart0Trigger               = 41U + (LPUART0_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer4M2ToLpuart0Trigger               = 42U + (LPUART0_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer4M3ToLpuart0Trigger               = 43U + (LPUART0_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_FlexioCh0ToLpuart0Trigger               = 44U + (LPUART0_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_FlexioCh1ToLpuart0Trigger               = 45U + (LPUART0_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_FlexioCh2ToLpuart0Trigger               = 46U + (LPUART0_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_FlexioCh3ToLpuart0Trigger               = 47U + (LPUART0_TRIG_REG << PMUX_SHIFT),

    /*!< LPUART1 trigger input connections. */
    kINPUTMUX_Aoi0Out0ToLpuart1Trigger                = 2U + (LPUART1_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out1ToLpuart1Trigger                = 3U + (LPUART1_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out2ToLpuart1Trigger                = 4U + (LPUART1_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out3ToLpuart1Trigger                = 5U + (LPUART1_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Cmp0OutToLpuart1Trigger                 = 6U + (LPUART1_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Cmp1OutToLpuart1Trigger                 = 7U + (LPUART1_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer0M2ToLpuart1Trigger               = 9U + (LPUART1_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer0M3ToLpuart1Trigger               = 10U + (LPUART1_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer1M2ToLpuart1Trigger               = 11U + (LPUART1_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer1M3ToLpuart1Trigger               = 12U + (LPUART1_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer2M2ToLpuart1Trigger               = 13U + (LPUART1_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer2M3ToLpuart1Trigger               = 14U + (LPUART1_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Lptmr0ToLpuart1Trigger                  = 15U + (LPUART1_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn0ToLpuart1Trigger                 = 17U + (LPUART1_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn1ToLpuart1Trigger                 = 18U + (LPUART1_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn2ToLpuart1Trigger                 = 19U + (LPUART1_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn3ToLpuart1Trigger                 = 20U + (LPUART1_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn4ToLpuart1Trigger                 = 21U + (LPUART1_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn5ToLpuart1Trigger                 = 22U + (LPUART1_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn6ToLpuart1Trigger                 = 23U + (LPUART1_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn7ToLpuart1Trigger                 = 24U + (LPUART1_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn8ToLpuart1Trigger                 = 25U + (LPUART1_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn9ToLpuart1Trigger                 = 26U + (LPUART1_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn10ToLpuart1Trigger                = 27U + (LPUART1_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn11ToLpuart1Trigger                = 28U + (LPUART1_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio0PinEventTrig0ToLpuart1Trigger      = 29U + (LPUART1_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio1PinEventTrig0ToLpuart1Trigger      = 30U + (LPUART1_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio2PinEventTrig0ToLpuart1Trigger      = 31U + (LPUART1_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio3PinEventTrig0ToLpuart1Trigger      = 32U + (LPUART1_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio4PinEventTrig0ToLpuart1Trigger      = 33U + (LPUART1_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_WuuToLpuart1Trigger                     = 34U + (LPUART1_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Usb0IppIndUartRxdUsbmuxToLpuart1Trigger = 35U + (LPUART1_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out0ToLpuart1Trigger                = 36U + (LPUART1_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out1ToLpuart1Trigger                = 37U + (LPUART1_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out2ToLpuart1Trigger                = 38U + (LPUART1_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out3ToLpuart1Trigger                = 39U + (LPUART1_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer3M2ToLpuart1Trigger               = 40U + (LPUART1_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer3M3ToLpuart1Trigger               = 41U + (LPUART1_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer4M2ToLpuart1Trigger               = 42U + (LPUART1_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer4M3ToLpuart1Trigger               = 43U + (LPUART1_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_FlexioCh0ToLpuart1Trigger               = 44U + (LPUART1_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_FlexioCh1ToLpuart1Trigger               = 45U + (LPUART1_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_FlexioCh2ToLpuart1Trigger               = 46U + (LPUART1_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_FlexioCh3ToLpuart1Trigger               = 47U + (LPUART1_TRIG_REG << PMUX_SHIFT),

    /*!< LPUART2 trigger input connections. */
    kINPUTMUX_Aoi0Out0ToLpuart2Trigger                = 2U + (LPUART2_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out1ToLpuart2Trigger                = 3U + (LPUART2_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out2ToLpuart2Trigger                = 4U + (LPUART2_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out3ToLpuart2Trigger                = 5U + (LPUART2_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Cmp0OutToLpuart2Trigger                 = 6U + (LPUART2_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Cmp1OutToLpuart2Trigger                 = 7U + (LPUART2_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer0M2ToLpuart2Trigger               = 9U + (LPUART2_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer0M3ToLpuart2Trigger               = 10U + (LPUART2_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer1M2ToLpuart2Trigger               = 11U + (LPUART2_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer1M3ToLpuart2Trigger               = 12U + (LPUART2_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer2M2ToLpuart2Trigger               = 13U + (LPUART2_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer2M3ToLpuart2Trigger               = 14U + (LPUART2_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Lptmr0ToLpuart2Trigger                  = 15U + (LPUART2_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn0ToLpuart2Trigger                 = 17U + (LPUART2_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn1ToLpuart2Trigger                 = 18U + (LPUART2_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn2ToLpuart2Trigger                 = 19U + (LPUART2_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn3ToLpuart2Trigger                 = 20U + (LPUART2_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn4ToLpuart2Trigger                 = 21U + (LPUART2_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn5ToLpuart2Trigger                 = 22U + (LPUART2_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn6ToLpuart2Trigger                 = 23U + (LPUART2_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn7ToLpuart2Trigger                 = 24U + (LPUART2_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn8ToLpuart2Trigger                 = 25U + (LPUART2_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn9ToLpuart2Trigger                 = 26U + (LPUART2_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn10ToLpuart2Trigger                = 27U + (LPUART2_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn11ToLpuart2Trigger                = 28U + (LPUART2_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio0PinEventTrig0ToLpuart2Trigger      = 29U + (LPUART2_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio1PinEventTrig0ToLpuart2Trigger      = 30U + (LPUART2_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio2PinEventTrig0ToLpuart2Trigger      = 31U + (LPUART2_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio3PinEventTrig0ToLpuart2Trigger      = 32U + (LPUART2_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio4PinEventTrig0ToLpuart2Trigger      = 33U + (LPUART2_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_WuuToLpuart2Trigger                     = 34U + (LPUART2_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Usb0IppIndUartRxdUsbmuxToLpuart2Trigger = 35U + (LPUART2_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out0ToLpuart2Trigger                = 36U + (LPUART2_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out1ToLpuart2Trigger                = 37U + (LPUART2_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out2ToLpuart2Trigger                = 38U + (LPUART2_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out3ToLpuart2Trigger                = 39U + (LPUART2_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer3M2ToLpuart2Trigger               = 40U + (LPUART2_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer3M3ToLpuart2Trigger               = 41U + (LPUART2_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer4M2ToLpuart2Trigger               = 42U + (LPUART2_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer4M3ToLpuart2Trigger               = 43U + (LPUART2_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_FlexioCh0ToLpuart2Trigger               = 44U + (LPUART2_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_FlexioCh1ToLpuart2Trigger               = 45U + (LPUART2_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_FlexioCh2ToLpuart2Trigger               = 46U + (LPUART2_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_FlexioCh3ToLpuart2Trigger               = 47U + (LPUART2_TRIG_REG << PMUX_SHIFT),

    /*!< LPUART3 trigger input connections. */
    kINPUTMUX_Aoi0Out0ToLpuart3Trigger                = 2U + (LPUART3_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out1ToLpuart3Trigger                = 3U + (LPUART3_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out2ToLpuart3Trigger                = 4U + (LPUART3_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out3ToLpuart3Trigger                = 5U + (LPUART3_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Cmp0OutToLpuart3Trigger                 = 6U + (LPUART3_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Cmp1OutToLpuart3Trigger                 = 7U + (LPUART3_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer0M2ToLpuart3Trigger               = 9U + (LPUART3_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer0M3ToLpuart3Trigger               = 10U + (LPUART3_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer1M2ToLpuart3Trigger               = 11U + (LPUART3_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer1M3ToLpuart3Trigger               = 12U + (LPUART3_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer2M2ToLpuart3Trigger               = 13U + (LPUART3_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer2M3ToLpuart3Trigger               = 14U + (LPUART3_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Lptmr0ToLpuart3Trigger                  = 15U + (LPUART3_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn0ToLpuart3Trigger                 = 17U + (LPUART3_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn1ToLpuart3Trigger                 = 18U + (LPUART3_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn2ToLpuart3Trigger                 = 19U + (LPUART3_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn3ToLpuart3Trigger                 = 20U + (LPUART3_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn4ToLpuart3Trigger                 = 21U + (LPUART3_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn5ToLpuart3Trigger                 = 22U + (LPUART3_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn6ToLpuart3Trigger                 = 23U + (LPUART3_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn7ToLpuart3Trigger                 = 24U + (LPUART3_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn8ToLpuart3Trigger                 = 25U + (LPUART3_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn9ToLpuart3Trigger                 = 26U + (LPUART3_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn10ToLpuart3Trigger                = 27U + (LPUART3_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn11ToLpuart3Trigger                = 28U + (LPUART3_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio0PinEventTrig0ToLpuart3Trigger      = 29U + (LPUART3_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio1PinEventTrig0ToLpuart3Trigger      = 30U + (LPUART3_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio2PinEventTrig0ToLpuart3Trigger      = 31U + (LPUART3_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio3PinEventTrig0ToLpuart3Trigger      = 32U + (LPUART3_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio4PinEventTrig0ToLpuart3Trigger      = 33U + (LPUART3_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_WuuToLpuart3Trigger                     = 34U + (LPUART3_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Usb0IppIndUartRxdUsbmuxToLpuart3Trigger = 35U + (LPUART3_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out0ToLpuart3Trigger                = 36U + (LPUART3_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out1ToLpuart3Trigger                = 37U + (LPUART3_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out2ToLpuart3Trigger                = 38U + (LPUART3_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out3ToLpuart3Trigger                = 39U + (LPUART3_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer3M2ToLpuart3Trigger               = 40U + (LPUART3_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer3M3ToLpuart3Trigger               = 41U + (LPUART3_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer4M2ToLpuart3Trigger               = 42U + (LPUART3_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer4M3ToLpuart3Trigger               = 43U + (LPUART3_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_FlexioCh0ToLpuart3Trigger               = 44U + (LPUART3_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_FlexioCh1ToLpuart3Trigger               = 45U + (LPUART3_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_FlexioCh2ToLpuart3Trigger               = 46U + (LPUART3_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_FlexioCh3ToLpuart3Trigger               = 47U + (LPUART3_TRIG_REG << PMUX_SHIFT),

    /*!< LPUART4 trigger input connections. */
    kINPUTMUX_Aoi0Out0ToLpuart4Trigger                = 2U + (LPUART4_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out1ToLpuart4Trigger                = 3U + (LPUART4_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out2ToLpuart4Trigger                = 4U + (LPUART4_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out3ToLpuart4Trigger                = 5U + (LPUART4_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Cmp0OutToLpuart4Trigger                 = 6U + (LPUART4_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Cmp1OutToLpuart4Trigger                 = 7U + (LPUART4_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer0M2ToLpuart4Trigger               = 9U + (LPUART4_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer0M3ToLpuart4Trigger               = 10U + (LPUART4_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer1M2ToLpuart4Trigger               = 11U + (LPUART4_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer1M3ToLpuart4Trigger               = 12U + (LPUART4_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer2M2ToLpuart4Trigger               = 13U + (LPUART4_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer2M3ToLpuart4Trigger               = 14U + (LPUART4_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Lptmr0ToLpuart4Trigger                  = 15U + (LPUART4_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn0ToLpuart4Trigger                 = 17U + (LPUART4_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn1ToLpuart4Trigger                 = 18U + (LPUART4_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn2ToLpuart4Trigger                 = 19U + (LPUART4_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn3ToLpuart4Trigger                 = 20U + (LPUART4_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn4ToLpuart4Trigger                 = 21U + (LPUART4_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn5ToLpuart4Trigger                 = 22U + (LPUART4_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn6ToLpuart4Trigger                 = 23U + (LPUART4_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn7ToLpuart4Trigger                 = 24U + (LPUART4_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn8ToLpuart4Trigger                 = 25U + (LPUART4_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn9ToLpuart4Trigger                 = 26U + (LPUART4_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn10ToLpuart4Trigger                = 27U + (LPUART4_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_TrigIn11ToLpuart4Trigger                = 28U + (LPUART4_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio0PinEventTrig0ToLpuart4Trigger      = 29U + (LPUART4_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio1PinEventTrig0ToLpuart4Trigger      = 30U + (LPUART4_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio2PinEventTrig0ToLpuart4Trigger      = 31U + (LPUART4_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio3PinEventTrig0ToLpuart4Trigger      = 32U + (LPUART4_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Gpio4PinEventTrig0ToLpuart4Trigger      = 33U + (LPUART4_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_WuuToLpuart4Trigger                     = 34U + (LPUART4_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Usb0IppIndUartRxdUsbmuxToLpuart4Trigger = 35U + (LPUART4_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out0ToLpuart4Trigger                = 36U + (LPUART4_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out1ToLpuart4Trigger                = 37U + (LPUART4_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out2ToLpuart4Trigger                = 38U + (LPUART4_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out3ToLpuart4Trigger                = 39U + (LPUART4_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer3M2ToLpuart4Trigger               = 40U + (LPUART4_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer3M3ToLpuart4Trigger               = 41U + (LPUART4_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer4M2ToLpuart4Trigger               = 42U + (LPUART4_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_Ctimer4M3ToLpuart4Trigger               = 43U + (LPUART4_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_FlexioCh0ToLpuart4Trigger               = 44U + (LPUART4_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_FlexioCh1ToLpuart4Trigger               = 45U + (LPUART4_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_FlexioCh2ToLpuart4Trigger               = 46U + (LPUART4_TRIG_REG << PMUX_SHIFT),
    kINPUTMUX_FlexioCh3ToLpuart4Trigger               = 47U + (LPUART4_TRIG_REG << PMUX_SHIFT),

/*!< Flexio trigger0 input connections. */
    kINPUTMUX_Aoi0Out0ToFlexioTrigger                   = 1U + (FLEXIO_TRIG0_REG  << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out1ToFlexioTrigger                   = 2U + (FLEXIO_TRIG0_REG  << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out2ToFlexioTrigger                   = 3U + (FLEXIO_TRIG0_REG  << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out3ToFlexioTrigger                   = 4U + (FLEXIO_TRIG0_REG  << PMUX_SHIFT),
    kINPUTMUX_Adc0Tcomp0ToFlexioTrigger                 = 5U + (FLEXIO_TRIG0_REG  << PMUX_SHIFT),
    kINPUTMUX_Adc0Tcomp1ToFlexioTrigger                 = 6U + (FLEXIO_TRIG0_REG  << PMUX_SHIFT),
    kINPUTMUX_Adc0Tcomp2ToFlexioTrigger                 = 7U + (FLEXIO_TRIG0_REG  << PMUX_SHIFT),
    kINPUTMUX_Adc0Tcomp3ToFlexioTrigger                 = 8U + (FLEXIO_TRIG0_REG  << PMUX_SHIFT),
    kINPUTMUX_Cmp0OutToFlexioTrigger                    = 9U + (FLEXIO_TRIG0_REG  << PMUX_SHIFT),
    kINPUTMUX_Cmp1OutToFlexioTrigger                    = 10U + (FLEXIO_TRIG0_REG  << PMUX_SHIFT),
    kINPUTMUX_Ctimer0M1ToFlexioTrigger                  = 12U + (FLEXIO_TRIG0_REG  << PMUX_SHIFT),
    kINPUTMUX_Ctimer0M2ToFlexioTrigger                  = 13U + (FLEXIO_TRIG0_REG  << PMUX_SHIFT),
    kINPUTMUX_Ctimer1M1ToFlexioTrigger                  = 14U + (FLEXIO_TRIG0_REG  << PMUX_SHIFT),
    kINPUTMUX_Ctimer1M2ToFlexioTrigger                  = 15U + (FLEXIO_TRIG0_REG  << PMUX_SHIFT),
    kINPUTMUX_Ctimer2M1ToFlexioTrigger                  = 16U + (FLEXIO_TRIG0_REG  << PMUX_SHIFT),
    kINPUTMUX_Ctimer2M2ToFlexioTrigger                  = 17U + (FLEXIO_TRIG0_REG  << PMUX_SHIFT),
    kINPUTMUX_Lptmr0ToFlexioTrigger                     = 18U + (FLEXIO_TRIG0_REG  << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm0OutTrig0ToFlexioTrigger            = 20U + (FLEXIO_TRIG0_REG  << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm1OutTrig0ToFlexioTrigger            = 21U + (FLEXIO_TRIG0_REG  << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm2OutTrig0ToFlexioTrigger            = 22U + (FLEXIO_TRIG0_REG  << PMUX_SHIFT),
    kINPUTMUX_TrigIn0ToFlexioTrigger                    = 24U + (FLEXIO_TRIG0_REG  << PMUX_SHIFT),
    kINPUTMUX_TrigIn1ToFlexioTrigger                    = 25U + (FLEXIO_TRIG0_REG  << PMUX_SHIFT),
    kINPUTMUX_TrigIn2ToFlexioTrigger                    = 26U + (FLEXIO_TRIG0_REG  << PMUX_SHIFT),
    kINPUTMUX_TrigIn3ToFlexioTrigger                    = 27U + (FLEXIO_TRIG0_REG  << PMUX_SHIFT),
    kINPUTMUX_TrigIn4ToFlexioTrigger                    = 28U + (FLEXIO_TRIG0_REG  << PMUX_SHIFT),
    kINPUTMUX_TrigIn5ToFlexioTrigger                    = 29U + (FLEXIO_TRIG0_REG  << PMUX_SHIFT),
    kINPUTMUX_TrigIn6ToFlexioTrigger                    = 30U + (FLEXIO_TRIG0_REG  << PMUX_SHIFT),
    kINPUTMUX_TrigIn7ToFlexioTrigger                    = 31U + (FLEXIO_TRIG0_REG  << PMUX_SHIFT),
    kINPUTMUX_Gpio0PinEventTrig0ToFlexioTrigger         = 32U + (FLEXIO_TRIG0_REG  << PMUX_SHIFT),
    kINPUTMUX_Gpio2PinEventTrig0ToFlexioTrigger         = 33U + (FLEXIO_TRIG0_REG  << PMUX_SHIFT),
    kINPUTMUX_Gpio3PinEventTrig0ToFlexioTrigger         = 34U + (FLEXIO_TRIG0_REG  << PMUX_SHIFT),
    kINPUTMUX_Gpio4PinEventTrig0ToFlexioTrigger         = 35U + (FLEXIO_TRIG0_REG  << PMUX_SHIFT),
    kINPUTMUX_WuuToFlexioTrigger                        = 36U + (FLEXIO_TRIG0_REG  << PMUX_SHIFT),
    kINPUTMUX_Pwm1A0Trig0ToFlexioTrigger                = 37U + (FLEXIO_TRIG0_REG  << PMUX_SHIFT),
    kINPUTMUX_Lpi2c0MasterEndOfPacketToFlexioTrigger    = 38U + (FLEXIO_TRIG0_REG  << PMUX_SHIFT),
    kINPUTMUX_Lpi2c0SlaveEndOfPacketToFlexioTrigger     = 39U + (FLEXIO_TRIG0_REG  << PMUX_SHIFT),
    kINPUTMUX_Lpi2c1MasterEndOfPacketToFlexioTrigger    = 40U + (FLEXIO_TRIG0_REG  << PMUX_SHIFT),
    kINPUTMUX_Lpi2c1SlaveEndOfPacketToFlexioTrigger     = 41U + (FLEXIO_TRIG0_REG  << PMUX_SHIFT),
    kINPUTMUX_Lpspi0EndOfFrameToFlexioTrigger           = 42U + (FLEXIO_TRIG0_REG  << PMUX_SHIFT),
    kINPUTMUX_Lpspi0ReceivedDataWordToFlexioTrigger     = 43U + (FLEXIO_TRIG0_REG  << PMUX_SHIFT),
    kINPUTMUX_Lpspi1EndOfFrameToFlexioTrigger           = 44U + (FLEXIO_TRIG0_REG  << PMUX_SHIFT),
    kINPUTMUX_Lpspi1ReceivedDataWordToFlexioTrigger     = 45U + (FLEXIO_TRIG0_REG  << PMUX_SHIFT),
    kINPUTMUX_Lpuart0ReceivedDataWordToFlexioTrigger    = 46U + (FLEXIO_TRIG0_REG  << PMUX_SHIFT),
    kINPUTMUX_Lpuart0TransmittedDataWordToFlexioTrigger = 47U + (FLEXIO_TRIG0_REG  << PMUX_SHIFT),
    kINPUTMUX_Lpuart0ReceiveLineIdleToFlexioTrigger     = 48U + (FLEXIO_TRIG0_REG  << PMUX_SHIFT),
    kINPUTMUX_Lpuart1ReceivedDataWordToFlexioTrigger    = 49U + (FLEXIO_TRIG0_REG  << PMUX_SHIFT),
    kINPUTMUX_Lpuart1TransmittedDataWordToFlexioTrigger = 50U + (FLEXIO_TRIG0_REG  << PMUX_SHIFT),
    kINPUTMUX_Lpuart1ReceiveLineIdleToFlexioTrigger     = 51U + (FLEXIO_TRIG0_REG  << PMUX_SHIFT),
    kINPUTMUX_Lpuart2ReceivedDataWordToFlexioTrigger    = 52U + (FLEXIO_TRIG0_REG  << PMUX_SHIFT),
    kINPUTMUX_Lpuart2TransmittedDataWordToFlexioTrigger = 53U + (FLEXIO_TRIG0_REG  << PMUX_SHIFT),
    kINPUTMUX_Lpuart2ReceiveLineIdleToFlexioTrigger     = 54U + (FLEXIO_TRIG0_REG  << PMUX_SHIFT),
    kINPUTMUX_Lpuart3ReceivedDataWordToFlexioTrigger    = 55U + (FLEXIO_TRIG0_REG  << PMUX_SHIFT),
    kINPUTMUX_Lpuart3TransmittedDataWordToFlexioTrigger = 56U + (FLEXIO_TRIG0_REG  << PMUX_SHIFT),
    kINPUTMUX_Lpuart3ReceiveLineIdleToFlexioTrigger     = 57U + (FLEXIO_TRIG0_REG  << PMUX_SHIFT),
    kINPUTMUX_Lpuart4ReceivedDataWordToFlexioTrigger    = 58U + (FLEXIO_TRIG0_REG  << PMUX_SHIFT),
    kINPUTMUX_Lpuart4TransmittedDataWordToFlexioTrigger = 59U + (FLEXIO_TRIG0_REG  << PMUX_SHIFT),
    kINPUTMUX_Lpuart4ReceiveLineIdleToFlexioTrigger     = 60U + (FLEXIO_TRIG0_REG  << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out0ToFlexioTrigger                   = 61U + (FLEXIO_TRIG0_REG  << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out1ToFlexioTrigger                   = 62U + (FLEXIO_TRIG0_REG  << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out2ToFlexioTrigger                   = 63U + (FLEXIO_TRIG0_REG  << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out3ToFlexioTrigger                   = 64U + (FLEXIO_TRIG0_REG  << PMUX_SHIFT),
    kINPUTMUX_Adc1Tcomp0ToFlexioTrigger                 = 65U + (FLEXIO_TRIG0_REG  << PMUX_SHIFT),
    kINPUTMUX_Adc1Tcomp1ToFlexioTrigger                 = 66U + (FLEXIO_TRIG0_REG  << PMUX_SHIFT),
    kINPUTMUX_Adc1Tcomp2ToFlexioTrigger                 = 67U + (FLEXIO_TRIG0_REG  << PMUX_SHIFT),
    kINPUTMUX_Adc1Tcomp3ToFlexioTrigger                 = 68U + (FLEXIO_TRIG0_REG  << PMUX_SHIFT),
    kINPUTMUX_Ctimer3M2ToFlexioTrigger                  = 69U + (FLEXIO_TRIG0_REG  << PMUX_SHIFT),
    kINPUTMUX_Ctimer3M3ToFlexioTrigger                  = 70U + (FLEXIO_TRIG0_REG  << PMUX_SHIFT),
    kINPUTMUX_Ctimer4M2ToFlexioTrigger                  = 71U + (FLEXIO_TRIG0_REG  << PMUX_SHIFT),
    kINPUTMUX_Ctimer4M3ToFlexioTrigger                  = 72U + (FLEXIO_TRIG0_REG  << PMUX_SHIFT),
    kINPUTMUX_Pwm1Sm0OutTrig0ToFlexioTrigger            = 73U + (FLEXIO_TRIG0_REG  << PMUX_SHIFT),
    kINPUTMUX_Pwm1Sm1OutTrig0ToFlexioTrigger            = 74U + (FLEXIO_TRIG0_REG  << PMUX_SHIFT),
    kINPUTMUX_Pwm1Sm2OutTrig0ToFlexioTrigger            = 75U + (FLEXIO_TRIG0_REG  << PMUX_SHIFT),
    kINPUTMUX_Lpi2c2MasterEndOfPacketToFlexioTrigger    = 77U + (FLEXIO_TRIG0_REG  << PMUX_SHIFT),
    kINPUTMUX_Lpi2c2SlaveEndOfPacketToFlexioTrigger     = 78U + (FLEXIO_TRIG0_REG  << PMUX_SHIFT),
    kINPUTMUX_Lpi2c3MasterEndOfPacketToFlexioTrigger    = 79U + (FLEXIO_TRIG0_REG  << PMUX_SHIFT),
    kINPUTMUX_Lpi2c3SlaveEndOfPacketToFlexioTrigger     = 80U + (FLEXIO_TRIG0_REG  << PMUX_SHIFT),

/*!< Opamp0 trigger input connections. */
    kINPUTMUX_Aoi0Out0ToOpamp0Trigger           = 2U + (OPAMP0_TRIG_REG  << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out1ToOpamp0Trigger           = 3U + (OPAMP0_TRIG_REG  << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out2ToOpamp0Trigger           = 4U + (OPAMP0_TRIG_REG  << PMUX_SHIFT),
    kINPUTMUX_Aoi0Out3ToOpamp0Trigger           = 5U + (OPAMP0_TRIG_REG  << PMUX_SHIFT),
    kINPUTMUX_Cmp0OutToOpamp0Trigger            = 6U + (OPAMP0_TRIG_REG  << PMUX_SHIFT),
    kINPUTMUX_Cmp1OutToOpamp0Trigger            = 7U + (OPAMP0_TRIG_REG  << PMUX_SHIFT),
    kINPUTMUX_Ctimer0M0ToOpamp0Trigger          = 9U + (OPAMP0_TRIG_REG  << PMUX_SHIFT),
    kINPUTMUX_Ctimer0M1ToOpamp0Trigger          = 10U + (OPAMP0_TRIG_REG  << PMUX_SHIFT),
    kINPUTMUX_Ctimer1M0ToOpamp0Trigger          = 11U + (OPAMP0_TRIG_REG  << PMUX_SHIFT),
    kINPUTMUX_Ctimer1M1ToOpamp0Trigger          = 12U + (OPAMP0_TRIG_REG  << PMUX_SHIFT),
    kINPUTMUX_Ctimer2M0ToOpamp0Trigger          = 13U + (OPAMP0_TRIG_REG  << PMUX_SHIFT),
    kINPUTMUX_Ctimer2M1ToOpamp0Trigger          = 14U + (OPAMP0_TRIG_REG  << PMUX_SHIFT),
    kINPUTMUX_Lptmr0ToOpamp0Trigger             = 15U + (OPAMP0_TRIG_REG  << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm0OutTrig0ToOpamp0Trigger    = 18U + (OPAMP0_TRIG_REG  << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm0OutTrig1ToOpamp0Trigger    = 19U + (OPAMP0_TRIG_REG  << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm1OutTrig0ToOpamp0Trigger    = 20U + (OPAMP0_TRIG_REG  << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm1OutTrig1ToOpamp0Trigger    = 21U + (OPAMP0_TRIG_REG  << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm2OutTrig0ToOpamp0Trigger    = 22U + (OPAMP0_TRIG_REG  << PMUX_SHIFT),
    kINPUTMUX_Pwm0Sm2OutTrig1ToOpamp0Trigger    = 23U + (OPAMP0_TRIG_REG  << PMUX_SHIFT),
    kINPUTMUX_Gpio0PinEventTrig0ToOpamp0Trigger = 26U + (OPAMP0_TRIG_REG  << PMUX_SHIFT),
    kINPUTMUX_Gpio1PinEventTrig0ToOpamp0Trigger = 27U + (OPAMP0_TRIG_REG  << PMUX_SHIFT),
    kINPUTMUX_Gpio2PinEventTrig0ToOpamp0Trigger = 28U + (OPAMP0_TRIG_REG  << PMUX_SHIFT),
    kINPUTMUX_Gpio3PinEventTrig0ToOpamp0Trigger = 29U + (OPAMP0_TRIG_REG  << PMUX_SHIFT),
    kINPUTMUX_Gpio4PinEventTrig0ToOpamp0Trigger = 30U + (OPAMP0_TRIG_REG  << PMUX_SHIFT),
    kINPUTMUX_WuuToOpamp0Trigger                = 31U + (OPAMP0_TRIG_REG  << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out0ToOpamp0Trigger           = 33U + (OPAMP0_TRIG_REG  << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out1ToOpamp0Trigger           = 34U + (OPAMP0_TRIG_REG  << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out2ToOpamp0Trigger           = 35U + (OPAMP0_TRIG_REG  << PMUX_SHIFT),
    kINPUTMUX_Aoi1Out3ToOpamp0Trigger           = 36U + (OPAMP0_TRIG_REG  << PMUX_SHIFT),
    kINPUTMUX_Adc1Tcomp0ToOpamp0Trigger         = 37U + (OPAMP0_TRIG_REG  << PMUX_SHIFT),
    kINPUTMUX_Adc1Tcomp1ToOpamp0Trigger         = 38U + (OPAMP0_TRIG_REG  << PMUX_SHIFT),
    kINPUTMUX_Adc1Tcomp2ToOpamp0Trigger         = 39U + (OPAMP0_TRIG_REG  << PMUX_SHIFT),
    kINPUTMUX_Adc1Tcomp3ToOpamp0Trigger         = 40U + (OPAMP0_TRIG_REG  << PMUX_SHIFT),
    kINPUTMUX_Ctimer3M0ToOpamp0Trigger          = 41U + (OPAMP0_TRIG_REG  << PMUX_SHIFT),
    kINPUTMUX_Ctimer3M1ToOpamp0Trigger          = 42U + (OPAMP0_TRIG_REG  << PMUX_SHIFT),
    kINPUTMUX_Ctimer4M0ToOpamp0Trigger          = 43U + (OPAMP0_TRIG_REG  << PMUX_SHIFT),
    kINPUTMUX_Ctimer4M1ToOpamp0Trigger          = 44U + (OPAMP0_TRIG_REG  << PMUX_SHIFT),
    kINPUTMUX_FlexioCh0ToOpamp0Trigger          = 45U + (OPAMP0_TRIG_REG  << PMUX_SHIFT),
    kINPUTMUX_FlexioCh1ToOpamp0Trigger          = 46U + (OPAMP0_TRIG_REG  << PMUX_SHIFT),
    kINPUTMUX_FlexioCh2ToOpamp0Trigger          = 47U + (OPAMP0_TRIG_REG  << PMUX_SHIFT),
    kINPUTMUX_FlexioCh3ToOpamp0Trigger          = 48U + (OPAMP0_TRIG_REG  << PMUX_SHIFT),
    kINPUTMUX_Pwm1Sm0OutTrig0ToOpamp0Trigger    = 50U + (OPAMP0_TRIG_REG  << PMUX_SHIFT),
    kINPUTMUX_Pwm1Sm0OutTrig1ToOpamp0Trigger    = 51U + (OPAMP0_TRIG_REG  << PMUX_SHIFT),
    kINPUTMUX_Pwm1Sm1OutTrig0ToOpamp0Trigger    = 52U + (OPAMP0_TRIG_REG  << PMUX_SHIFT),
    kINPUTMUX_Pwm1Sm1OutTrig1ToOpamp0Trigger    = 53U + (OPAMP0_TRIG_REG  << PMUX_SHIFT),
    kINPUTMUX_Pwm1Sm2OutTrig0ToOpamp0Trigger    = 54U + (OPAMP0_TRIG_REG  << PMUX_SHIFT),
    kINPUTMUX_Pwm1Sm2OutTrig1ToOpamp0Trigger    = 55U + (OPAMP0_TRIG_REG  << PMUX_SHIFT),
} inputmux_connection_t;

/*@}*/

/*@}*/

#endif /* _FSL_INPUTMUX_CONNECTIONS_ */
